@@ -7,8 +7,8 @@
 #define GUIINFOMANAGER_H_
 
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,9 +32,7 @@
 #include "inttypes.h"
 #include "XBDateTime.h"
 #include "utils/Observer.h"
-#include "interfaces/info/InfoBool.h"
 #include "interfaces/info/SkinVariable.h"
-#include "cores/IPlayer.h"
 
 #include <list>
 #include <map>
@@ -49,6 +47,7 @@ class CGUIListItem;
 class CDateTime;
 namespace INFO
 {
+  class InfoBool;
   class InfoSingle;
 }
 
@@ -109,8 +108,6 @@ namespace INFO
 #define PLAYER_CAN_SEEK              51
 #define PLAYER_START_TIME            52
 #define PLAYER_TITLE                 53
-#define PLAYER_ISINTERNETSTREAM      54
-#define PLAYER_FILENAME              55
 
 #define WEATHER_CONDITIONS          100
 #define WEATHER_TEMPERATURE         101
@@ -165,6 +162,21 @@ namespace INFO
 #define SYSTEM_HDD_LOCKSTATE        157
 #define SYSTEM_HDD_LOCKKEY          158
 #define SYSTEM_INTERNET_STATE       159
+#define LCD_PLAY_ICON               160
+#define LCD_PROGRESS_BAR            161
+#define LCD_CPU_TEMPERATURE         162
+#define LCD_GPU_TEMPERATURE         163
+#define LCD_HDD_TEMPERATURE         164
+#define LCD_FAN_SPEED               165
+#define LCD_DATE                    166
+#define LCD_TIME_21                 172 // Small bigfont
+#define LCD_TIME_22                 173
+#define LCD_TIME_W21                174 // Medum bigfont
+#define LCD_TIME_W22                175
+#define LCD_TIME_41                 176 // Big bigfont
+#define LCD_TIME_42                 177
+#define LCD_TIME_43                 178
+#define LCD_TIME_44                 179
 #define SYSTEM_ALARM_LESS_OR_EQUAL  180
 #define SYSTEM_PROFILECOUNT         181
 #define SYSTEM_ISFULLSCREEN         182
@@ -172,9 +184,6 @@ namespace INFO
 #define SYSTEM_ISINHIBIT            184
 #define SYSTEM_HAS_SHUTDOWN         185
 #define SYSTEM_HAS_PVR              186
-#define SYSTEM_STARTUP_WINDOW       187
-#define SYSTEM_STEREOSCOPIC_MODE    188
-#define SYSTEM_BUILD_VERSION_SHORT  189
 
 #define NETWORK_IP_ADDRESS          190
 #define NETWORK_MAC_ADDRESS         191
@@ -274,11 +283,15 @@ namespace INFO
 #define VIDEOPLAYER_CHANNEL_GROUP     306
 #define VIDEOPLAYER_PARENTAL_RATING   307
 #define VIDEOPLAYER_HAS_EPG           308
-#define VIDEOPLAYER_VOTES             309
-#define VIDEOPLAYER_IS_STEREOSCOPIC   310
-#define VIDEOPLAYER_STEREOSCOPIC_MODE 311
-#define VIDEOPLAYER_SUBTITLES_LANG    312
-#define VIDEOPLAYER_AUDIO_LANG        313
+
+#define AUDIOSCROBBLER_ENABLED      325
+#define AUDIOSCROBBLER_CONN_STATE   326
+#define AUDIOSCROBBLER_SUBMIT_INT   327
+#define AUDIOSCROBBLER_FILES_CACHED 328
+#define AUDIOSCROBBLER_SUBMIT_STATE 329
+#define LASTFM_RADIOPLAYING         330
+#define LASTFM_CANLOVE              331
+#define LASTFM_CANBAN               332
 
 #define CONTAINER_CAN_FILTER         342
 #define CONTAINER_CAN_FILTERADVANCED 343
@@ -289,7 +302,7 @@ namespace INFO
 #define CONTAINER_STATIC            347
 #define CONTAINER_MOVE_NEXT         348
 #define CONTAINER_SCROLL_NEXT       349
-#define CONTAINER_ISUPDATING        350
+
 #define CONTAINER_HASFILES          351
 #define CONTAINER_HASFOLDERS        352
 #define CONTAINER_STACKED           353
@@ -368,7 +381,7 @@ namespace INFO
 #define SYSTEM_CPUFREQUENCY         656
 #define SYSTEM_SCREEN_RESOLUTION    659
 #define SYSTEM_VIDEO_ENCODER_INFO   660
-#define SYSTEM_OS_VERSION_INFO      667
+#define SYSTEM_KERNEL_VERSION       667
 #define SYSTEM_FREE_SPACE           679
 #define SYSTEM_USED_SPACE           680
 #define SYSTEM_TOTAL_SPACE          681
@@ -389,8 +402,6 @@ namespace INFO
 #define SYSTEM_IDLE_TIME            715
 #define SYSTEM_FRIENDLY_NAME        716
 #define SYSTEM_SCREENSAVER_ACTIVE   717
-#define SYSTEM_ADDON_VERSION        718
-#define SYSTEM_DPMS_ACTIVE          719
 
 #define LIBRARY_HAS_MUSIC           720
 #define LIBRARY_HAS_VIDEO           721
@@ -409,7 +420,6 @@ namespace INFO
 #define SYSTEM_PLATFORM_DARWIN_IOS  745
 #define SYSTEM_PLATFORM_DARWIN_ATV2 746
 #define SYSTEM_PLATFORM_ANDROID     747
-#define SYSTEM_PLATFORM_LINUX_RASPBERRY_PI 748
 
 #define SYSTEM_CAN_POWERDOWN        750
 #define SYSTEM_CAN_SUSPEND          751
@@ -419,7 +429,6 @@ namespace INFO
 #define SLIDESHOW_ISPAUSED          800
 #define SLIDESHOW_ISRANDOM          801
 #define SLIDESHOW_ISACTIVE          802
-#define SLIDESHOW_ISVIDEO           803
 
 #define SLIDE_INFO_START            900
 #define SLIDE_INFO_END              980
@@ -429,8 +438,6 @@ namespace INFO
 #define FANART_COLOR3               1002
 #define FANART_IMAGE                1003
 
-#define SYSTEM_PROFILEAUTOLOGIN     1004
-
 #define PVR_CONDITIONS_START        1100
 #define PVR_IS_RECORDING            (PVR_CONDITIONS_START)
 #define PVR_HAS_TIMER               (PVR_CONDITIONS_START + 1)
@@ -479,10 +486,7 @@ namespace INFO
 #define PVR_ACTUAL_STREAM_AUDIO_BR  (PVR_STRINGS_START + 31)
 #define PVR_ACTUAL_STREAM_DOLBY_BR  (PVR_STRINGS_START + 32)
 #define PVR_ACTUAL_STREAM_CRYPTION  (PVR_STRINGS_START + 33)
-#define PVR_ACTUAL_STREAM_SERVICE   (PVR_STRINGS_START + 34)
-#define PVR_ACTUAL_STREAM_MUX       (PVR_STRINGS_START + 35)
-#define PVR_ACTUAL_STREAM_PROVIDER  (PVR_STRINGS_START + 36)
-#define PVR_STRINGS_END             PVR_ACTUAL_STREAM_PROVIDER
+#define PVR_STRINGS_END             PVR_ACTUAL_STREAM_CRYPTION
 
 #define WINDOW_PROPERTY             9993
 #define WINDOW_IS_TOPMOST           9994
@@ -498,9 +502,9 @@ namespace INFO
 #define CONTROL_GROUP_HAS_FOCUS     29999
 #define CONTROL_HAS_FOCUS           30000
 
-#define VERSION_MAJOR               14
-#define VERSION_MINOR               0
-#define VERSION_TAG                 "-ALPHA1"
+#define VERSION_MAJOR 12
+#define VERSION_MINOR 0
+#define VERSION_TAG ""
 
 #define LISTITEM_START              35000
 #define LISTITEM_THUMB              (LISTITEM_START)
@@ -536,120 +540,81 @@ namespace INFO
 // the ordering of LISTITEM_PICTURE_* is important as they're mapped to SLIDE_* items in GUIInfoManager.cpp
 #define LISTITEM_PICTURE_START      (LISTITEM_START + 30)
 #define LISTITEM_PICTURE_RESOLUTION (LISTITEM_PICTURE_START) // => SLIDE_RESOLUTION
-#define LISTITEM_PICTURE_LONGDATE   (LISTITEM_START + 31)    // => SLIDE_EXIF_LONG_DATE
-#define LISTITEM_PICTURE_LONGDATETIME (LISTITEM_START + 32)    // => SLIDE_EXIF_LONG_DATE_TIME
-#define LISTITEM_PICTURE_DATE       (LISTITEM_START + 33)    // => SLIDE_EXIF_DATE
-#define LISTITEM_PICTURE_DATETIME   (LISTITEM_START + 34)    // => SLIDE_EXIF_DATE_TIME
-#define LISTITEM_PICTURE_COMMENT    (LISTITEM_START + 35)    // => SLIDE_COMMENT
-#define LISTITEM_PICTURE_CAPTION    (LISTITEM_START + 36)    // => SLIDE_IPTC_CAPTION
-#define LISTITEM_PICTURE_DESC       (LISTITEM_START + 37)    // => SLIDE_EXIF_DESCRIPTION
-#define LISTITEM_PICTURE_KEYWORDS   (LISTITEM_START + 38)    // => SLIDE_IPTC_KEYWORDS
-#define LISTITEM_PICTURE_CAM_MAKE   (LISTITEM_START + 39)    // => SLIDE_EXIF_CAMERA_MAKE
-#define LISTITEM_PICTURE_CAM_MODEL  (LISTITEM_START + 40)    // => SLIDE_EXIF_CAMERA_MODEL
-#define LISTITEM_PICTURE_APERTURE   (LISTITEM_START + 41)    // => SLIDE_EXIF_APERTURE
-#define LISTITEM_PICTURE_FOCAL_LEN  (LISTITEM_START + 42)    // => SLIDE_EXIF_FOCAL_LENGTH
-#define LISTITEM_PICTURE_FOCUS_DIST (LISTITEM_START + 43)    // => SLIDE_EXIF_FOCUS_DIST
-#define LISTITEM_PICTURE_EXP_MODE   (LISTITEM_START + 44)    // => SLIDE_EXIF_EXPOSURE_MODE
-#define LISTITEM_PICTURE_EXP_TIME   (LISTITEM_START + 45)    // => SLIDE_EXIF_EXPOSURE_TIME
-#define LISTITEM_PICTURE_ISO        (LISTITEM_START + 46)    // => SLIDE_EXIF_ISO_EQUIV
-#define LISTITEM_PICTURE_AUTHOR           (LISTITEM_START + 47) // => SLIDE_IPTC_AUTHOR
-#define LISTITEM_PICTURE_BYLINE           (LISTITEM_START + 48) // => SLIDE_IPTC_BYLINE
-#define LISTITEM_PICTURE_BYLINE_TITLE     (LISTITEM_START + 49) // => SLIDE_IPTC_BYLINE_TITLE
-#define LISTITEM_PICTURE_CATEGORY         (LISTITEM_START + 50) // => SLIDE_IPTC_CATEGORY
-#define LISTITEM_PICTURE_CCD_WIDTH        (LISTITEM_START + 51) // => SLIDE_EXIF_CCD_WIDTH
-#define LISTITEM_PICTURE_CITY             (LISTITEM_START + 52) // => SLIDE_IPTC_CITY
-#define LISTITEM_PICTURE_URGENCY          (LISTITEM_START + 53) // => SLIDE_IPTC_URGENCY
-#define LISTITEM_PICTURE_COPYRIGHT_NOTICE (LISTITEM_START + 54) // => SLIDE_IPTC_COPYRIGHT_NOTICE
-#define LISTITEM_PICTURE_COUNTRY          (LISTITEM_START + 55) // => SLIDE_IPTC_COUNTRY
-#define LISTITEM_PICTURE_COUNTRY_CODE     (LISTITEM_START + 56) // => SLIDE_IPTC_COUNTRY_CODE
-#define LISTITEM_PICTURE_CREDIT           (LISTITEM_START + 57) // => SLIDE_IPTC_CREDIT
-#define LISTITEM_PICTURE_IPTCDATE         (LISTITEM_START + 58) // => SLIDE_IPTC_DATE
-#define LISTITEM_PICTURE_DIGITAL_ZOOM     (LISTITEM_START + 59) // => SLIDE_EXIF_DIGITAL_ZOOM
-#define LISTITEM_PICTURE_EXPOSURE         (LISTITEM_START + 60) // => SLIDE_EXIF_EXPOSURE
-#define LISTITEM_PICTURE_EXPOSURE_BIAS    (LISTITEM_START + 61) // => SLIDE_EXIF_EXPOSURE_BIAS
-#define LISTITEM_PICTURE_FLASH_USED       (LISTITEM_START + 62) // => SLIDE_EXIF_FLASH_USED
-#define LISTITEM_PICTURE_HEADLINE         (LISTITEM_START + 63) // => SLIDE_IPTC_HEADLINE
-#define LISTITEM_PICTURE_COLOUR           (LISTITEM_START + 64) // => SLIDE_COLOUR
-#define LISTITEM_PICTURE_LIGHT_SOURCE     (LISTITEM_START + 65) // => SLIDE_EXIF_LIGHT_SOURCE
-#define LISTITEM_PICTURE_METERING_MODE    (LISTITEM_START + 66) // => SLIDE_EXIF_METERING_MODE
-#define LISTITEM_PICTURE_OBJECT_NAME      (LISTITEM_START + 67) // => SLIDE_IPTC_OBJECT_NAME
-#define LISTITEM_PICTURE_ORIENTATION      (LISTITEM_START + 68) // => SLIDE_EXIF_ORIENTATION
-#define LISTITEM_PICTURE_PROCESS          (LISTITEM_START + 69) // => SLIDE_PROCESS
-#define LISTITEM_PICTURE_REF_SERVICE      (LISTITEM_START + 70) // => SLIDE_IPTC_REF_SERVICE
-#define LISTITEM_PICTURE_SOURCE           (LISTITEM_START + 71) // => SLIDE_IPTC_SOURCE
-#define LISTITEM_PICTURE_SPEC_INSTR       (LISTITEM_START + 72) // => SLIDE_IPTC_SPEC_INSTR
-#define LISTITEM_PICTURE_STATE            (LISTITEM_START + 73) // => SLIDE_IPTC_STATE
-#define LISTITEM_PICTURE_SUP_CATEGORIES   (LISTITEM_START + 74) // => SLIDE_IPTC_SUP_CATEGORIES
-#define LISTITEM_PICTURE_TX_REFERENCE     (LISTITEM_START + 75) // => SLIDE_IPTC_TX_REFERENCE
-#define LISTITEM_PICTURE_WHITE_BALANCE    (LISTITEM_START + 76) // => SLIDE_EXIF_WHITE_BALANCE
-#define LISTITEM_PICTURE_IMAGETYPE        (LISTITEM_START + 77) // => SLIDE_IPTC_IMAGETYPE
-#define LISTITEM_PICTURE_SUBLOCATION      (LISTITEM_START + 78) // => SLIDE_IPTC_SUBLOCATION
-#define LISTITEM_PICTURE_TIMECREATED      (LISTITEM_START + 79) // => SLIDE_IPTC_TIMECREATED
-#define LISTITEM_PICTURE_GPS_LAT    (LISTITEM_START + 80)    // => SLIDE_EXIF_GPS_LATITUDE
-#define LISTITEM_PICTURE_GPS_LON    (LISTITEM_START + 81)    // => SLIDE_EXIF_GPS_LONGITUDE
-#define LISTITEM_PICTURE_GPS_ALT    (LISTITEM_START + 82)    // => SLIDE_EXIF_GPS_ALTITUDE
+#define LISTITEM_PICTURE_DATE       (LISTITEM_START + 31)    // => SLIDE_EXIF_DATE
+#define LISTITEM_PICTURE_DATETIME   (LISTITEM_START + 32)    // => SLIDE_EXIF_DATE_TIME
+#define LISTITEM_PICTURE_COMMENT    (LISTITEM_START + 33)    // => SLIDE_COMMENT
+#define LISTITEM_PICTURE_CAPTION    (LISTITEM_START + 34)    // => SLIDE_IPTC_CAPTION
+#define LISTITEM_PICTURE_DESC       (LISTITEM_START + 35)    // => SLIDE_EXIF_DESCRIPTION
+#define LISTITEM_PICTURE_KEYWORDS   (LISTITEM_START + 36)    // => SLIDE_IPTC_KEYWORDS
+#define LISTITEM_PICTURE_CAM_MAKE   (LISTITEM_START + 37)    // => SLIDE_EXIF_CAMERA_MAKE
+#define LISTITEM_PICTURE_CAM_MODEL  (LISTITEM_START + 38)    // => SLIDE_EXIF_CAMERA_MODEL
+#define LISTITEM_PICTURE_APERTURE   (LISTITEM_START + 39)    // => SLIDE_EXIF_APERTURE
+#define LISTITEM_PICTURE_FOCAL_LEN  (LISTITEM_START + 40)    // => SLIDE_EXIF_FOCAL_LENGTH
+#define LISTITEM_PICTURE_FOCUS_DIST (LISTITEM_START + 41)    // => SLIDE_EXIF_FOCUS_DIST
+#define LISTITEM_PICTURE_EXP_MODE   (LISTITEM_START + 42)    // => SLIDE_EXIF_EXPOSURE_MODE
+#define LISTITEM_PICTURE_EXP_TIME   (LISTITEM_START + 43)    // => SLIDE_EXIF_EXPOSURE_TIME
+#define LISTITEM_PICTURE_ISO        (LISTITEM_START + 44)    // => SLIDE_EXIF_ISO_EQUIV
+#define LISTITEM_PICTURE_GPS_LAT    (LISTITEM_START + 45)    // => SLIDE_EXIF_GPS_LATITUDE
+#define LISTITEM_PICTURE_GPS_LON    (LISTITEM_START + 46)    // => SLIDE_EXIF_GPS_LONGITUDE
+#define LISTITEM_PICTURE_GPS_ALT    (LISTITEM_START + 47)    // => SLIDE_EXIF_GPS_ALTITUDE
 #define LISTITEM_PICTURE_END        (LISTITEM_PICTURE_GPS_ALT)
-#define LISTITEM_STUDIO             (LISTITEM_START + 83)
-#define LISTITEM_MPAA               (LISTITEM_START + 84)
-#define LISTITEM_CAST               (LISTITEM_START + 85)
-#define LISTITEM_CAST_AND_ROLE      (LISTITEM_START + 86)
-#define LISTITEM_WRITER             (LISTITEM_START + 87)
-#define LISTITEM_TAGLINE            (LISTITEM_START + 88)
-#define LISTITEM_TOP250             (LISTITEM_START + 89)
-#define LISTITEM_RATING_AND_VOTES   (LISTITEM_START + 90)
-#define LISTITEM_TRAILER            (LISTITEM_START + 91)
-#define LISTITEM_STAR_RATING        (LISTITEM_START + 92)
-#define LISTITEM_FILENAME_AND_PATH  (LISTITEM_START + 93)
-#define LISTITEM_SORT_LETTER        (LISTITEM_START + 94)
-#define LISTITEM_ALBUM_ARTIST       (LISTITEM_START + 95)
-#define LISTITEM_FOLDERNAME         (LISTITEM_START + 96)
-#define LISTITEM_VIDEO_CODEC        (LISTITEM_START + 97)
-#define LISTITEM_VIDEO_RESOLUTION   (LISTITEM_START + 98)
-#define LISTITEM_VIDEO_ASPECT       (LISTITEM_START + 99)
-#define LISTITEM_AUDIO_CODEC        (LISTITEM_START + 100)
-#define LISTITEM_AUDIO_CHANNELS     (LISTITEM_START + 101)
-#define LISTITEM_AUDIO_LANGUAGE     (LISTITEM_START + 102)
-#define LISTITEM_SUBTITLE_LANGUAGE  (LISTITEM_START + 103)
-#define LISTITEM_IS_FOLDER          (LISTITEM_START + 104)
-#define LISTITEM_ORIGINALTITLE      (LISTITEM_START + 105)
-#define LISTITEM_COUNTRY            (LISTITEM_START + 106)
-#define LISTITEM_PLAYCOUNT          (LISTITEM_START + 107)
-#define LISTITEM_LASTPLAYED         (LISTITEM_START + 108)
-#define LISTITEM_FOLDERPATH         (LISTITEM_START + 109)
-#define LISTITEM_DISC_NUMBER        (LISTITEM_START + 110)
-#define LISTITEM_FILE_EXTENSION     (LISTITEM_START + 111)
-#define LISTITEM_IS_RESUMABLE       (LISTITEM_START + 112)
-#define LISTITEM_PERCENT_PLAYED     (LISTITEM_START + 113)
-#define LISTITEM_DATE_ADDED         (LISTITEM_START + 114)
-#define LISTITEM_DBTYPE             (LISTITEM_START + 115)
-#define LISTITEM_DBID               (LISTITEM_START + 116)
+#define LISTITEM_STUDIO             (LISTITEM_START + 48)
+#define LISTITEM_MPAA               (LISTITEM_START + 49)
+#define LISTITEM_CAST               (LISTITEM_START + 50)
+#define LISTITEM_CAST_AND_ROLE      (LISTITEM_START + 51)
+#define LISTITEM_WRITER             (LISTITEM_START + 52)
+#define LISTITEM_TAGLINE            (LISTITEM_START + 53)
+#define LISTITEM_TOP250             (LISTITEM_START + 54)
+#define LISTITEM_RATING_AND_VOTES   (LISTITEM_START + 55)
+#define LISTITEM_TRAILER            (LISTITEM_START + 56)
+#define LISTITEM_STAR_RATING        (LISTITEM_START + 57)
+#define LISTITEM_FILENAME_AND_PATH  (LISTITEM_START + 58)
+#define LISTITEM_SORT_LETTER        (LISTITEM_START + 59)
+#define LISTITEM_ALBUM_ARTIST       (LISTITEM_START + 60)
+#define LISTITEM_FOLDERNAME         (LISTITEM_START + 61)
+#define LISTITEM_VIDEO_CODEC        (LISTITEM_START + 62)
+#define LISTITEM_VIDEO_RESOLUTION   (LISTITEM_START + 63)
+#define LISTITEM_VIDEO_ASPECT       (LISTITEM_START + 64)
+#define LISTITEM_AUDIO_CODEC        (LISTITEM_START + 65)
+#define LISTITEM_AUDIO_CHANNELS     (LISTITEM_START + 66)
+#define LISTITEM_AUDIO_LANGUAGE     (LISTITEM_START + 67)
+#define LISTITEM_SUBTITLE_LANGUAGE  (LISTITEM_START + 68)
+#define LISTITEM_IS_FOLDER          (LISTITEM_START + 69)
+#define LISTITEM_ORIGINALTITLE      (LISTITEM_START + 70)
+#define LISTITEM_COUNTRY            (LISTITEM_START + 71)
+#define LISTITEM_PLAYCOUNT          (LISTITEM_START + 72)
+#define LISTITEM_LASTPLAYED         (LISTITEM_START + 73)
+#define LISTITEM_FOLDERPATH         (LISTITEM_START + 74)
+#define LISTITEM_DISC_NUMBER        (LISTITEM_START + 75)
+#define LISTITEM_FILE_EXTENSION     (LISTITEM_START + 76)
+#define LISTITEM_IS_RESUMABLE       (LISTITEM_START + 77)
+#define LISTITEM_PERCENT_PLAYED     (LISTITEM_START + 78)
+#define LISTITEM_DATE_ADDED         (LISTITEM_START + 79)
+#define LISTITEM_DBTYPE             (LISTITEM_START + 80)
+#define LISTITEM_DBID               (LISTITEM_START + 81)
 
-#define LISTITEM_STARTTIME          (LISTITEM_START + 117)
-#define LISTITEM_ENDTIME            (LISTITEM_START + 118)
-#define LISTITEM_STARTDATE          (LISTITEM_START + 119)
-#define LISTITEM_ENDDATE            (LISTITEM_START + 120)
-#define LISTITEM_NEXT_TITLE         (LISTITEM_START + 121)
-#define LISTITEM_NEXT_GENRE         (LISTITEM_START + 122)
-#define LISTITEM_NEXT_PLOT          (LISTITEM_START + 123)
-#define LISTITEM_NEXT_PLOT_OUTLINE  (LISTITEM_START + 124)
-#define LISTITEM_NEXT_STARTTIME     (LISTITEM_START + 125)
-#define LISTITEM_NEXT_ENDTIME       (LISTITEM_START + 126)
-#define LISTITEM_NEXT_STARTDATE     (LISTITEM_START + 127)
-#define LISTITEM_NEXT_ENDDATE       (LISTITEM_START + 128)
-#define LISTITEM_NEXT_DURATION      (LISTITEM_START + 129)
-#define LISTITEM_CHANNEL_NAME       (LISTITEM_START + 130)
-#define LISTITEM_CHANNEL_NUMBER     (LISTITEM_START + 131)
-#define LISTITEM_CHANNEL_GROUP      (LISTITEM_START + 132)
-#define LISTITEM_HASTIMER           (LISTITEM_START + 133)
-#define LISTITEM_ISRECORDING        (LISTITEM_START + 134)
-#define LISTITEM_ISENCRYPTED        (LISTITEM_START + 135)
-#define LISTITEM_PARENTALRATING     (LISTITEM_START + 136)
-#define LISTITEM_PROGRESS           (LISTITEM_START + 137)
-#define LISTITEM_HAS_EPG            (LISTITEM_START + 138)
-#define LISTITEM_VOTES              (LISTITEM_START + 139)
-#define LISTITEM_STEREOSCOPIC_MODE  (LISTITEM_START + 140)
-#define LISTITEM_IS_STEREOSCOPIC    (LISTITEM_START + 141)
-#define LISTITEM_INPROGRESS         (LISTITEM_START + 142)
+#define LISTITEM_STARTTIME          (LISTITEM_START + 82)
+#define LISTITEM_ENDTIME            (LISTITEM_START + 83)
+#define LISTITEM_STARTDATE          (LISTITEM_START + 84)
+#define LISTITEM_ENDDATE            (LISTITEM_START + 85)
+#define LISTITEM_NEXT_TITLE         (LISTITEM_START + 86)
+#define LISTITEM_NEXT_GENRE         (LISTITEM_START + 87)
+#define LISTITEM_NEXT_PLOT          (LISTITEM_START + 88)
+#define LISTITEM_NEXT_PLOT_OUTLINE  (LISTITEM_START + 89)
+#define LISTITEM_NEXT_STARTTIME     (LISTITEM_START + 90)
+#define LISTITEM_NEXT_ENDTIME       (LISTITEM_START + 91)
+#define LISTITEM_NEXT_STARTDATE     (LISTITEM_START + 92)
+#define LISTITEM_NEXT_ENDDATE       (LISTITEM_START + 93)
+#define LISTITEM_NEXT_DURATION      (LISTITEM_START + 94)
+#define LISTITEM_CHANNEL_NAME       (LISTITEM_START + 95)
+#define LISTITEM_CHANNEL_NUMBER     (LISTITEM_START + 96)
+#define LISTITEM_CHANNEL_GROUP      (LISTITEM_START + 97)
+#define LISTITEM_HASTIMER           (LISTITEM_START + 98)
+#define LISTITEM_ISRECORDING        (LISTITEM_START + 99)
+#define LISTITEM_ISENCRYPTED        (LISTITEM_START + 100)
+#define LISTITEM_PARENTALRATING     (LISTITEM_START + 101)
+#define LISTITEM_PROGRESS           (LISTITEM_START + 102)
+#define LISTITEM_HAS_EPG            (LISTITEM_START + 103)
 
 #define LISTITEM_PROPERTY_START     (LISTITEM_START + 200)
 #define LISTITEM_PROPERTY_END       (LISTITEM_PROPERTY_START + 1000)
@@ -726,14 +691,22 @@ public:
    \param expression the boolean condition or expression
    \param context the context window
    \return an identifier used to reference this expression
+
+   \sa GetBoolValue
    */
-  INFO::InfoPtr Register(const CStdString &expression, int context = 0);
+  unsigned int Register(const CStdString &expression, int context = 0);
+
+  /*! \brief Get a previously registered boolean expression's value
+   Checks the cache and evaluates the boolean expression if required.
+   \sa Register
+   */
+  bool GetBoolValue(unsigned int expression, const CGUIListItem *item = NULL);
 
   /*! \brief Evaluate a boolean expression
    \param expression the expression to evaluate
    \param context the context in which to evaluate the expression (currently windows)
    \return the value of the evaluated expression.
-   \sa Register
+   \sa Register, GetBoolValue
    */
   bool EvaluateBool(const CStdString &expression, int context = 0);
 
@@ -753,6 +726,7 @@ public:
   CStdString GetImage(int info, int contextWindow, CStdString *fallback = NULL);
 
   CStdString GetTime(TIME_FORMAT format = TIME_FORMAT_GUESS) const;
+  CStdString GetLcdTime( int _eInfo ) const;
   CStdString GetDate(bool bNumbersOnly = false);
   CStdString GetDuration(TIME_FORMAT format = TIME_FORMAT_GUESS) const;
 
@@ -761,7 +735,7 @@ public:
   // Current song stuff
   /// \brief Retrieves tag info (if necessary) and fills in our current song path.
   void SetCurrentSong(CFileItem &item);
-  void SetCurrentAlbumThumb(const CStdString &thumbFileName);
+  void SetCurrentAlbumThumb(const CStdString thumbFileName);
   void SetCurrentMovie(CFileItem &item);
   void SetCurrentSlide(CFileItem &item);
   const CFileItem &GetCurrentSlide() const;
@@ -775,7 +749,7 @@ public:
   CStdString GetMusicLabel(int item);
   CStdString GetMusicTagLabel(int info, const CFileItem *item);
   CStdString GetVideoLabel(int item);
-  CStdString GetPlaylistLabel(int item, int playlistid = -1 /* PLAYLIST_NONE */) const;
+  CStdString GetPlaylistLabel(int item) const;
   CStdString GetMusicPartyModeLabel(int item);
   const CStdString GetMusicPlaylistInfo(const GUIInfo& info);
   CStdString GetPictureLabel(int item);
@@ -786,7 +760,6 @@ public:
   int GetPlayTimeRemaining() const;
   int GetTotalPlayTime() const;
   CStdString GetCurrentPlayTimeRemaining(TIME_FORMAT format) const;
-  std::string GetVersionShort(void);
   CStdString GetVersion();
   CStdString GetBuild();
 
@@ -804,7 +777,6 @@ public:
   CTemperature GetGPUTemperature();
 
   void UpdateFPS();
-  void UpdateAVInfo();
   inline float GetFPS() const { return m_fps; };
 
   void SetNextWindow(int windowID) { m_nextWindowID = windowID; };
@@ -841,14 +813,10 @@ public:
   CStdString GetSkinVariableString(int info, bool preferImage = false, const CGUIListItem *item=NULL);
 
   /// \brief iterates through boolean conditions and compares their stored values to current values. Returns true if any condition changed value.
-  bool ConditionsChangedValues(const std::map<INFO::InfoPtr, bool>& map);
-
-  bool m_AVInfoValid;
-
+  bool ConditionsChangedValues(const std::map<int, bool>& map);
 protected:
   friend class INFO::InfoSingle;
   bool GetBool(int condition, int contextWindow = 0, const CGUIListItem *item=NULL);
-  int TranslateSingleString(const CStdString &strCondition, bool &listItemDependent);
 
   // routines for window retrieval
   bool CheckWindowCondition(CGUIWindow *window, int condition) const;
@@ -861,12 +829,12 @@ protected:
   public:
     Property(const CStdString &property, const CStdString &parameters);
 
-    const std::string &param(unsigned int n = 0) const;
+    const CStdString &param(unsigned int n = 0) const;
     unsigned int num_params() const;
 
     CStdString name;
   private:
-    std::vector<std::string> params;
+    std::vector<CStdString> params;
   };
 
   bool GetMultiInfoBool(const GUIInfo &info, int contextWindow = 0, const CGUIListItem *item = NULL);
@@ -895,6 +863,8 @@ protected:
   int AddMultiInfo(const GUIInfo &info);
   int AddListItemProp(const CStdString &str, int offset=0);
 
+  CStdString GetAudioScrobblerLabel(int item);
+
   /*!
    * @brief Get the EPG tag that is currently active
    * @param tag The active tag
@@ -903,7 +873,7 @@ protected:
   bool GetEpgInfoTag(EPG::CEpgInfoTag& tag) const;
 
   // Conditional string parameters are stored here
-  std::vector<std::string> m_stringParameters;
+  CStdStringArray m_stringParameters;
 
   // Array of multiple information mapped to a single integer lookup
   std::vector<GUIInfo> m_multiInfo;
@@ -914,6 +884,8 @@ protected:
   // Current playing stuff
   CFileItem* m_currentFile;
   CStdString m_currentMovieThumb;
+  unsigned int m_lastMusicBitrateTime;
+  unsigned int m_MusicBitrate;
   CFileItem* m_currentSlide;
 
   // fan stuff
@@ -939,8 +911,9 @@ protected:
   int m_nextWindowID;
   int m_prevWindowID;
 
-  std::vector<INFO::InfoPtr> m_bools;
+  std::vector<INFO::InfoBool*> m_bools;
   std::vector<INFO::CSkinVariableString> m_skinVariableStrings;
+  unsigned int m_updateTime;
 
   int m_libraryHasMusic;
   int m_libraryHasMovies;
@@ -948,9 +921,6 @@ protected:
   int m_libraryHasMusicVideos;
   int m_libraryHasMovieSets;
 
-  SPlayerVideoStreamInfo m_videoInfo;
-  SPlayerAudioStreamInfo m_audioInfo;
-
   CCriticalSection m_critInfo;
 };
 
