@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,16 +21,14 @@
 #include "Addon.h"
 #include "AddonManager.h"
 #include "settings/Settings.h"
+#include "settings/GUISettings.h"
 #include "filesystem/Directory.h"
 #include "filesystem/File.h"
-#include "system.h"
-#ifdef HAS_PYTHON
 #include "interfaces/python/XBPython.h"
-#endif
 #if defined(TARGET_DARWIN)
 #include "../osx/OSXGNUReplacements.h"
 #endif
-#ifdef TARGET_FREEBSD
+#ifdef __FreeBSD__
 #include "freebsd/FreeBSDGNUReplacements.h"
 #endif
 #include "utils/log.h"
@@ -68,17 +66,17 @@ static const TypeMapping types[] =
    {"xbmc.metadata.scraper.movies",      ADDON_SCRAPER_MOVIES,      24007, "DefaultAddonMovieInfo.png" },
    {"xbmc.metadata.scraper.musicvideos", ADDON_SCRAPER_MUSICVIDEOS, 24015, "DefaultAddonMusicVideoInfo.png" },
    {"xbmc.metadata.scraper.tvshows",     ADDON_SCRAPER_TVSHOWS,     24014, "DefaultAddonTvInfo.png" },
-   {"xbmc.metadata.scraper.library",     ADDON_SCRAPER_LIBRARY,     24083, "" },
+   {"xbmc.metadata.scraper.library",     ADDON_SCRAPER_LIBRARY,         0, "" },
    {"xbmc.ui.screensaver",               ADDON_SCREENSAVER,         24008, "DefaultAddonScreensaver.png" },
    {"xbmc.player.musicviz",              ADDON_VIZ,                 24010, "DefaultAddonVisualization.png" },
-   {"visualization-library",             ADDON_VIZ_LIBRARY,         24084, "" },
+   {"visualization-library",             ADDON_VIZ_LIBRARY,             0, "" },
    {"xbmc.python.pluginsource",          ADDON_PLUGIN,              24005, "" },
    {"xbmc.python.script",                ADDON_SCRIPT,              24009, "" },
    {"xbmc.python.weather",               ADDON_SCRIPT_WEATHER,      24027, "DefaultAddonWeather.png" },
+   {"xbmc.python.subtitles",             ADDON_SCRIPT_SUBTITLES,    24012, "DefaultAddonSubtitles.png" },
    {"xbmc.python.lyrics",                ADDON_SCRIPT_LYRICS,       24013, "DefaultAddonLyrics.png" },
-   {"xbmc.python.library",               ADDON_SCRIPT_LIBRARY,      24081, "" },
-   {"xbmc.python.module",                ADDON_SCRIPT_MODULE,       24082, "" },
-   {"xbmc.subtitle.module",              ADDON_SUBTITLE_MODULE,     24012, "DefaultAddonSubtitles.png" },
+   {"xbmc.python.library",               ADDON_SCRIPT_LIBRARY,      24014, "" },
+   {"xbmc.python.module",                ADDON_SCRIPT_MODULE,           0, "" },
    {"xbmc.gui.skin",                     ADDON_SKIN,                  166, "DefaultAddonSkin.png" },
    {"xbmc.gui.webinterface",             ADDON_WEB_INTERFACE,         199, "DefaultAddonWebSkin.png" },
    {"xbmc.addon.repository",             ADDON_REPOSITORY,          24011, "DefaultAddonRepository.png" },
@@ -131,15 +129,13 @@ const CStdString GetIcon(const ADDON::TYPE& type)
   { \
     CStdString fan=CAddonMgr::Get().GetExtValue(metadata->configuration, x); \
     if (fan.Equals("true")) \
-      y.clear(); \
+      y.Empty(); \
   }
 
-#define SS(x) (x) ? x : ""
-
 AddonProps::AddonProps(const cp_extension_t *ext)
   : id(ext->plugin->identifier)
-  , version(SS(ext->plugin->version))
-  , minversion(SS(ext->plugin->abi_bw_compatibility))
+  , version(ext->plugin->version)
+  , minversion(ext->plugin->abi_bw_compatibility)
   , name(ext->plugin->name)
   , path(ext->plugin->plugin_path)
   , author(ext->plugin->provider_name)
@@ -161,7 +157,7 @@ AddonProps::AddonProps(const cp_extension_t *ext)
     license = CAddonMgr::Get().GetExtValue(metadata->configuration, "license");
     CStdString language;
     language = CAddonMgr::Get().GetExtValue(metadata->configuration, "language");
-    if (!language.empty())
+    if (!language.IsEmpty())
       extrainfo.insert(make_pair("language",language));
     broken = CAddonMgr::Get().GetExtValue(metadata->configuration, "broken");
     EMPTY_IF("nofanart",fanart)
@@ -173,8 +169,8 @@ AddonProps::AddonProps(const cp_extension_t *ext)
 
 AddonProps::AddonProps(const cp_plugin_info_t *plugin)
   : id(plugin->identifier)
-  , version(SS(plugin->version))
-  , minversion(SS(plugin->abi_bw_compatibility))
+  , version(plugin->version)
+  , minversion(plugin->abi_bw_compatibility)
   , name(plugin->name)
   , path(plugin->plugin_path)
   , author(plugin->provider_name)
@@ -187,9 +183,10 @@ void AddonProps::Serialize(CVariant &variant) const
 {
   variant["addonid"] = id;
   variant["type"] = TranslateType(type);
-  variant["version"] = version.asString();
-  variant["minversion"] = minversion.asString();
+  variant["version"] = version.c_str();
+  variant["minversion"] = minversion.c_str();
   variant["name"] = name;
+  variant["parent"] = parent;
   variant["license"] = license;
   variant["summary"] = summary;
   variant["description"] = description;
@@ -217,7 +214,7 @@ void AddonProps::Serialize(CVariant &variant) const
   {
     CVariant dep(CVariant::VariantTypeObject);
     dep["addonid"] = it->first;
-    dep["version"] = it->second.first.asString();
+    dep["version"] = it->second.first.c_str();
     dep["optional"] = it->second.second;
     variant["dependencies"].push_back(dep);
   }
@@ -242,7 +239,7 @@ void AddonProps::BuildDependencies(const cp_plugin_info_t *plugin)
     return;
   for (unsigned int i = 0; i < plugin->num_imports; ++i)
     dependencies.insert(make_pair(CStdString(plugin->imports[i].plugin_id),
-                                  make_pair(AddonVersion(SS(plugin->imports[i].version)), plugin->imports[i].optional != 0)));
+                        make_pair(AddonVersion(plugin->imports[i].version), plugin->imports[i].optional != 0)));
 }
 
 /**
@@ -252,11 +249,11 @@ void AddonProps::BuildDependencies(const cp_plugin_info_t *plugin)
 
 CAddon::CAddon(const cp_extension_t *ext)
   : m_props(ext)
+  , m_parent(AddonPtr())
 {
   BuildLibName(ext);
-  Props().libname = m_strLibName;
   BuildProfilePath();
-  m_userSettingsPath = URIUtils::AddFileToFolder(Profile(), "settings.xml");
+  URIUtils::AddFileToFolder(Profile(), "settings.xml", m_userSettingsPath);
   m_enabled = true;
   m_hasSettings = true;
   m_hasStrings = false;
@@ -267,6 +264,7 @@ CAddon::CAddon(const cp_extension_t *ext)
 
 CAddon::CAddon(const cp_plugin_info_t *plugin)
   : m_props(plugin)
+  , m_parent(AddonPtr())
 {
   m_enabled = true;
   m_hasSettings = false;
@@ -278,11 +276,12 @@ CAddon::CAddon(const cp_plugin_info_t *plugin)
 
 CAddon::CAddon(const AddonProps &props)
   : m_props(props)
+  , m_parent(AddonPtr())
 {
-  if (props.libname.empty()) BuildLibName();
+  if (props.libname.IsEmpty()) BuildLibName();
   else m_strLibName = props.libname;
   BuildProfilePath();
-  m_userSettingsPath = URIUtils::AddFileToFolder(Profile(), "settings.xml");
+  URIUtils::AddFileToFolder(Profile(), "settings.xml", m_userSettingsPath);
   m_enabled = true;
   m_hasSettings = true;
   m_hasStrings = false;
@@ -291,8 +290,9 @@ CAddon::CAddon(const AddonProps &props)
   m_userSettingsLoaded = false;
 }
 
-CAddon::CAddon(const CAddon &rhs)
+CAddon::CAddon(const CAddon &rhs, const AddonPtr &parent)
   : m_props(rhs.Props())
+  , m_parent(parent)
 {
   m_settings  = rhs.m_settings;
   m_addonXmlDoc = rhs.m_addonXmlDoc;
@@ -300,16 +300,16 @@ CAddon::CAddon(const CAddon &rhs)
   m_userSettingsLoaded = rhs.m_userSettingsLoaded;
   m_hasSettings = rhs.m_hasSettings;
   BuildProfilePath();
-  m_userSettingsPath = URIUtils::AddFileToFolder(Profile(), "settings.xml");
+  URIUtils::AddFileToFolder(Profile(), "settings.xml", m_userSettingsPath);
   m_strLibName  = rhs.m_strLibName;
   m_enabled = rhs.Enabled();
   m_hasStrings  = false;
   m_checkedStrings  = false;
 }
 
-AddonPtr CAddon::Clone() const
+AddonPtr CAddon::Clone(const AddonPtr &self) const
 {
-  return AddonPtr(new CAddon(*this));
+  return AddonPtr(new CAddon(*this, self));
 }
 
 bool CAddon::MeetsVersion(const AddonVersion &version) const
@@ -317,7 +317,8 @@ bool CAddon::MeetsVersion(const AddonVersion &version) const
   // if the addon is one of xbmc's extension point definitions (addonid starts with "xbmc.")
   // and the minversion is "0.0.0" i.e. no <backwards-compatibility> tag has been specified
   // we need to assume that the current version is not backwards-compatible and therefore check against the actual version
-  if (StringUtils::StartsWithNoCase(m_props.id, "xbmc.") && m_props.minversion.empty())
+  if (StringUtils::StartsWith(m_props.id, "xbmc.") &&
+     (strlen(m_props.minversion.c_str()) == 0 || StringUtils::EqualsNoCase(m_props.minversion.c_str(), "0.0.0")))
     return m_props.version == version;
 
   return m_props.minversion <= version && version <= m_props.version;
@@ -357,7 +358,7 @@ void CAddon::BuildLibName(const cp_extension_t *extension)
     case ADDON_SCRIPT_LIBRARY:
     case ADDON_SCRIPT_LYRICS:
     case ADDON_SCRIPT_WEATHER:
-    case ADDON_SUBTITLE_MODULE:        
+    case ADDON_SCRIPT_SUBTITLES:
     case ADDON_PLUGIN:
     case ADDON_SERVICE:
       ext = ADDON_PYTHON_EXT;
@@ -380,8 +381,8 @@ void CAddon::BuildLibName(const cp_extension_t *extension)
       case ADDON_SCRIPT_LIBRARY:
       case ADDON_SCRIPT_LYRICS:
       case ADDON_SCRIPT_WEATHER:
+      case ADDON_SCRIPT_SUBTITLES:
       case ADDON_SCRIPT_MODULE:
-      case ADDON_SUBTITLE_MODULE:
       case ADDON_SCRAPER_ALBUMS:
       case ADDON_SCRAPER_ARTISTS:
       case ADDON_SCRAPER_MOVIES:
@@ -391,7 +392,6 @@ void CAddon::BuildLibName(const cp_extension_t *extension)
       case ADDON_PVRDLL:
       case ADDON_PLUGIN:
       case ADDON_SERVICE:
-      case ADDON_REPOSITORY:
         {
           CStdString temp = CAddonMgr::Get().GetExtValue(extension->configuration, "@library");
           m_strLibName = temp;
@@ -412,7 +412,7 @@ bool CAddon::LoadStrings()
   // Path where the language strings reside
   CStdString chosenPath = URIUtils::AddFileToFolder(m_props.path, "resources/language/");
 
-  m_hasStrings = m_strings.Load(chosenPath, CSettings::Get().GetString("locale.language"));
+  m_hasStrings = m_strings.Load(chosenPath, g_guiSettings.GetString("locale.language"));
   return m_checkedStrings = true;
 }
 
@@ -492,13 +492,14 @@ bool CAddon::LoadUserSettings()
 
 void CAddon::SaveSettings(void)
 {
-  if (m_settings.empty())
+  if (!m_settings.size())
     return; // no settings to save
 
   // break down the path into directories
-  CStdString strAddon = URIUtils::GetDirectory(m_userSettingsPath);
+  CStdString strRoot, strAddon;
+  URIUtils::GetDirectory(m_userSettingsPath, strAddon);
   URIUtils::RemoveSlashAtEnd(strAddon);
-  CStdString strRoot = URIUtils::GetDirectory(strAddon);
+  URIUtils::GetDirectory(strAddon, strRoot);
   URIUtils::RemoveSlashAtEnd(strRoot);
 
   // create the individual folders
@@ -511,12 +512,9 @@ void CAddon::SaveSettings(void)
   CXBMCTinyXML doc;
   SettingsToXML(doc);
   doc.SaveFile(m_userSettingsPath);
-  m_userSettingsLoaded = true;
   
   CAddonMgr::Get().ReloadSettings(ID());//push the settings changes to the running addon instance
-#ifdef HAS_PYTHON
   g_pythonParser.OnSettingsChanged(ID());
-#endif
 }
 
 CStdString CAddon::GetSetting(const CStdString& key)
@@ -590,7 +588,7 @@ TiXmlElement* CAddon::GetSettingsXML()
 
 void CAddon::BuildProfilePath()
 {
-  m_profile = StringUtils::Format("special://profile/addon_data/%s/", ID().c_str());
+  m_profile.Format("special://profile/addon_data/%s/", ID().c_str());
 }
 
 const CStdString CAddon::Icon() const
@@ -605,15 +603,6 @@ const CStdString CAddon::LibPath() const
   return URIUtils::AddFileToFolder(m_props.path, m_strLibName);
 }
 
-AddonVersion CAddon::GetDependencyVersion(const std::string &dependencyID) const
-{
-  const ADDON::ADDONDEPS &deps = GetDeps();
-  ADDONDEPS::const_iterator it = deps.find(dependencyID);
-  if (it != deps.end())
-    return it->second.first;
-  return AddonVersion("0.0.0");
-}
-
 /**
  * CAddonLibrary
  *
@@ -631,11 +620,6 @@ CAddonLibrary::CAddonLibrary(const AddonProps& props)
 {
 }
 
-AddonPtr CAddonLibrary::Clone() const
-{
-  return AddonPtr(new CAddonLibrary(*this));
-}
-
 TYPE CAddonLibrary::SetAddonType()
 {
   if (Type() == ADDON_VIZ_LIBRARY)
@@ -644,5 +628,25 @@ TYPE CAddonLibrary::SetAddonType()
     return ADDON_UNKNOWN;
 }
 
+CStdString GetXbmcApiVersionDependency(ADDON::AddonPtr addon)
+{
+  CStdString version("1.0");
+  if (addon.get() != NULL)
+  {
+    const ADDON::ADDONDEPS &deps = addon->GetDeps();
+    ADDON::ADDONDEPS::const_iterator it;
+    CStdString key("xbmc.python");
+    it = deps.find(key);
+    if (!(it == deps.end()))
+    {
+      const ADDON::AddonVersion * xbmcApiVersion = &(it->second.first);
+      version = xbmcApiVersion->c_str();
+    }
+  }
+
+  return version;
+}
+
+
 } /* namespace ADDON */
 
