@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,12 +22,11 @@
 #include "GUIInfoManager.h"
 #include "Key.h"
 #include "utils/MathUtils.h"
-#include "utils/StringUtils.h"
 #include "GUIWindowManager.h"
 
 static const SliderAction actions[] = {
-  {"seek",    "PlayerControl(SeekPercentage(%2f))", PLAYER_PROGRESS, false},
-  {"volume",  "SetVolume(%2f)",                     PLAYER_VOLUME,   true}
+  {"seek",    "PlayerControl(SeekPercentage(%2d))", PLAYER_PROGRESS, false},
+  {"volume",  "SetVolume(%2d)",                     PLAYER_VOLUME,   true}
  };
 
 CGUISliderControl::CGUISliderControl(int parentID, int controlID, float posX, float posY, float width, float height, const CTextureInfo& backGroundTexture, const CTextureInfo& nibTexture, const CTextureInfo& nibTextureFocus, int iType)
@@ -142,7 +141,7 @@ bool CGUISliderControl::OnMessage(CGUIMessage& message)
     switch (message.GetMessage())
     {
     case GUI_MSG_ITEM_SELECT:
-      SetPercentage( (float)message.GetParam1() );
+      SetPercentage( message.GetParam1() );
       return true;
       break;
 
@@ -175,8 +174,7 @@ bool CGUISliderControl::OnAction(const CAction &action)
 
   case ACTION_SELECT_ITEM:
     // switch between the two sliders
-    if (m_rangeSelection)
-      SwitchRangeSelector();
+    SwitchRangeSelector();
     return true;
 
   default:
@@ -191,7 +189,7 @@ void CGUISliderControl::Move(int iNumSteps)
   bool rangeSwap = false;
   switch (m_iType)
   {
-  case SLIDER_CONTROL_TYPE_FLOAT:
+  case SPIN_CONTROL_TYPE_FLOAT:
     {
       float &value = m_floatValues[m_currentSelector];
       value += m_fInterval * iNumSteps;
@@ -207,7 +205,7 @@ void CGUISliderControl::Move(int iNumSteps)
       break;
     }
 
-  case SLIDER_CONTROL_TYPE_INT:
+  case SPIN_CONTROL_TYPE_INT:
     {
       int &value = m_intValues[m_currentSelector];
       value += m_iInterval * iNumSteps;
@@ -223,16 +221,15 @@ void CGUISliderControl::Move(int iNumSteps)
       break;
     }
 
-  case SLIDER_CONTROL_TYPE_PERCENTAGE:
   default:
     {
-      float &value = m_percentValues[m_currentSelector];
+      int &value = m_percentValues[m_currentSelector];
       value += m_iInterval * iNumSteps;
       if (value < 0) value = 0;
       if (value > 100) value = 100;
       if (m_percentValues[0] > m_percentValues[1])
       {
-        float valueLower = m_percentValues[0];
+        int valueLower = m_percentValues[0];
         m_percentValues[0] = m_percentValues[1];
         m_percentValues[1] = valueLower;
         rangeSwap = true;
@@ -249,11 +246,12 @@ void CGUISliderControl::Move(int iNumSteps)
 
 void CGUISliderControl::SendClick()
 {
-  float percent = 100*GetProportion();
-  SEND_CLICK_MESSAGE(GetID(), GetParentID(), MathUtils::round_int(percent));
+  int percent = MathUtils::round_int(100*GetProportion());
+  SEND_CLICK_MESSAGE(GetID(), GetParentID(), percent);
   if (m_action && (!m_dragging || m_action->fireOnDrag))
   {
-    CStdString action = StringUtils::Format(m_action->formatString, percent);
+    CStdString action;
+    action.Format(m_action->formatString, percent);
     CGUIMessage message(GUI_MSG_EXECUTE, m_controlID, m_parentID);
     message.SetStringParam(action);
     g_windowManager.SendMessage(message);    
@@ -287,40 +285,40 @@ void CGUISliderControl::SwitchRangeSelector()
     SetRangeSelector(RangeSelectorLower);
 }
 
-void CGUISliderControl::SetPercentage(float percent, RangeSelector selector /* = RangeSelectorLower */, bool updateCurrent /* = false */)
+void CGUISliderControl::SetPercentage(int iPercent, RangeSelector selector /* = RangeSelectorLower */, bool updateCurrent /* = false */)
 {
-  if (percent > 100) percent = 100;
-  else if (percent < 0) percent = 0;
+  if (iPercent > 100) iPercent = 100;
+  else if (iPercent < 0) iPercent = 0;
 
-  float percentLower = selector == RangeSelectorLower ? percent : m_percentValues[0];
-  float percentUpper = selector == RangeSelectorUpper ? percent : m_percentValues[1];
+  int iPercentLower = selector == RangeSelectorLower ? iPercent : m_percentValues[0];
+  int iPercentUpper = selector == RangeSelectorUpper ? iPercent : m_percentValues[1];
 
-  if (!m_rangeSelection || percentLower <= percentUpper)
+  if (!m_rangeSelection || iPercentLower <= iPercentUpper)
   {
-    m_percentValues[0] = percentLower;
-    m_percentValues[1] = percentUpper;
+    m_percentValues[0] = iPercentLower;
+    m_percentValues[1] = iPercentUpper;
     if (updateCurrent)
       m_currentSelector = selector;
   }
   else
   {
-    m_percentValues[0] = percentUpper;
-    m_percentValues[1] = percentLower;
+    m_percentValues[0] = iPercentUpper;
+    m_percentValues[1] = iPercentLower;
     if (updateCurrent)
         m_currentSelector = (selector == RangeSelectorLower ? RangeSelectorUpper : RangeSelectorLower);
   }
 }
 
-float CGUISliderControl::GetPercentage(RangeSelector selector /* = RangeSelectorLower */) const
+int CGUISliderControl::GetPercentage(RangeSelector selector /* = RangeSelectorLower */) const
 {
   return m_percentValues[selector];
 }
 
 void CGUISliderControl::SetIntValue(int iValue, RangeSelector selector /* = RangeSelectorLower */, bool updateCurrent /* = false */)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
-    SetFloatValue((float)iValue, selector, updateCurrent);
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
+    SetFloatValue((float)iValue);
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
   {
     if (iValue > m_iEnd) iValue = m_iEnd;
     else if (iValue < m_iStart) iValue = m_iStart;
@@ -344,22 +342,22 @@ void CGUISliderControl::SetIntValue(int iValue, RangeSelector selector /* = Rang
     }
   }
   else
-    SetPercentage((float)iValue, selector, updateCurrent);
+    SetPercentage(iValue);
 }
 
 int CGUISliderControl::GetIntValue(RangeSelector selector /* = RangeSelectorLower */) const
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     return (int)m_floatValues[selector];
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
     return m_intValues[selector];
   else
-    return MathUtils::round_int(m_percentValues[selector]);
+    return m_percentValues[selector];
 }
 
 void CGUISliderControl::SetFloatValue(float fValue, RangeSelector selector /* = RangeSelectorLower */, bool updateCurrent /* = false */)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
   {
     if (fValue > m_fEnd) fValue = m_fEnd;
     else if (fValue < m_fStart) fValue = m_fStart;
@@ -382,25 +380,25 @@ void CGUISliderControl::SetFloatValue(float fValue, RangeSelector selector /* =
         m_currentSelector = (selector == RangeSelectorLower ? RangeSelectorUpper : RangeSelectorLower);
     }
   }
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
-    SetIntValue((int)fValue, selector, updateCurrent);
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
+    SetIntValue((int)fValue);
   else
-    SetPercentage(fValue, selector, updateCurrent);
+    SetPercentage((int)fValue);
 }
 
 float CGUISliderControl::GetFloatValue(RangeSelector selector /* = RangeSelectorLower */) const
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     return m_floatValues[selector];
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
     return (float)m_intValues[selector];
   else
-    return m_percentValues[selector];
+    return (float)m_percentValues[selector];
 }
 
 void CGUISliderControl::SetIntInterval(int iInterval)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     m_fInterval = (float)iInterval;
   else
     m_iInterval = iInterval;
@@ -408,7 +406,7 @@ void CGUISliderControl::SetIntInterval(int iInterval)
 
 void CGUISliderControl::SetFloatInterval(float fInterval)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     m_fInterval = fInterval;
   else
     m_iInterval = (int)fInterval;
@@ -416,7 +414,7 @@ void CGUISliderControl::SetFloatInterval(float fInterval)
 
 void CGUISliderControl::SetRange(int iStart, int iEnd)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     SetFloatRange((float)iStart,(float)iEnd);
   else
   {
@@ -427,7 +425,7 @@ void CGUISliderControl::SetRange(int iStart, int iEnd)
 
 void CGUISliderControl::SetFloatRange(float fStart, float fEnd)
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  if (m_iType == SPIN_CONTROL_TYPE_INT)
     SetRange((int)fStart, (int)fEnd);
   else
   {
@@ -501,24 +499,24 @@ void CGUISliderControl::SetFromPosition(const CPoint &point, bool guessSelector
 
   switch (m_iType)
   {
-  case SLIDER_CONTROL_TYPE_FLOAT:
+  case SPIN_CONTROL_TYPE_FLOAT:
     {
       float fValue = m_fStart + (m_fEnd - m_fStart) * fPercent;
       SetFloatValue(fValue, m_currentSelector, true);
       break;
     }
 
-  case SLIDER_CONTROL_TYPE_INT:
+  case SPIN_CONTROL_TYPE_INT:
     {
       int iValue = (int)(m_iStart + (float)(m_iEnd - m_iStart) * fPercent + 0.49f);
       SetIntValue(iValue, m_currentSelector, true);
       break;
     }
 
-  case SLIDER_CONTROL_TYPE_PERCENTAGE:
   default:
     {
-      SetPercentage(fPercent * 100, m_currentSelector, true);
+      int iValue = (int)(fPercent * 100 + 0.49f);
+      SetPercentage(iValue, m_currentSelector, true);
       break;
     }
   }
@@ -593,29 +591,29 @@ void CGUISliderControl::SetInfo(int iInfo)
 
 CStdString CGUISliderControl::GetDescription() const
 {
-  if (!m_textValue.empty())
+  if (!m_textValue.IsEmpty())
     return m_textValue;
   CStdString description;
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
   {
     if (m_rangeSelection)
-      description = StringUtils::Format("[%2.2f, %2.2f]", m_floatValues[0], m_floatValues[1]);
+      description.Format("[%2.2f, %2.2f]", m_floatValues[0], m_floatValues[1]);
     else
-      description = StringUtils::Format("%2.2f", m_floatValues[0]);
+      description.Format("%2.2f", m_floatValues[0]);
   }
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
   {
     if (m_rangeSelection)
-      description = StringUtils::Format("[%i, %i]", m_intValues[0], m_intValues[1]);
+      description.Format("[%i, %i]", m_intValues[0], m_intValues[1]);
     else
-      description = StringUtils::Format("%i", m_intValues[0]);
+      description.Format("%i", m_intValues[0]);
   }
   else
   {
     if (m_rangeSelection)
-      description = StringUtils::Format("[%i%%, %i%%]", MathUtils::round_int(m_percentValues[0]), MathUtils::round_int(m_percentValues[1]));
+      description.Format("[%i%%, %i%%]", m_percentValues[0], m_percentValues[1]);
     else
-      description = StringUtils::Format("%i%%", MathUtils::round_int(m_percentValues[0]));
+      description.Format("%i%%", m_percentValues[0]);
   }
   return description;
 }
@@ -634,9 +632,9 @@ bool CGUISliderControl::UpdateColors()
 
 float CGUISliderControl::GetProportion(RangeSelector selector /* = RangeSelectorLower */) const
 {
-  if (m_iType == SLIDER_CONTROL_TYPE_FLOAT)
+  if (m_iType == SPIN_CONTROL_TYPE_FLOAT)
     return (GetFloatValue(selector) - m_fStart) / (m_fEnd - m_fStart);
-  else if (m_iType == SLIDER_CONTROL_TYPE_INT)
+  else if (m_iType == SPIN_CONTROL_TYPE_INT)
     return (float)(GetIntValue(selector) - m_iStart) / (float)(m_iEnd - m_iStart);
   return 0.01f * GetPercentage(selector);
 }
@@ -645,7 +643,7 @@ void CGUISliderControl::SetAction(const CStdString &action)
 {
   for (size_t i = 0; i < sizeof(actions)/sizeof(SliderAction); i++)
   {
-    if (StringUtils::EqualsNoCase(action, actions[i].action))
+    if (action.CompareNoCase(actions[i].action) == 0)
     {
       m_action = &actions[i];
       return;
