@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,8 @@
 
 using namespace std;
 
+#define WORK_AROUND_NEEDED_FOR_LINE_BREAKS
+
 CGUIString::CGUIString(iString start, iString end, bool carriageReturn)
 {
   m_text.assign(start, end);
@@ -50,7 +52,6 @@ CGUITextLayout::CGUITextLayout(CGUIFont *font, bool wrap, float fHeight, CGUIFon
   m_maxHeight = fHeight;
   m_textWidth = 0;
   m_textHeight = 0;
-  m_lastUpdateW = false;
 }
 
 void CGUITextLayout::SetWrap(bool bWrap)
@@ -80,7 +81,7 @@ void CGUITextLayout::Render(float x, float y, float angle, color_t color, color_
     alignment &= ~XBFONT_CENTER_Y;
   }
   m_font->Begin();
-  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); i++)
   {
     const CGUIString &string = *i;
     uint32_t align = alignment;
@@ -97,18 +98,8 @@ void CGUITextLayout::Render(float x, float y, float angle, color_t color, color_
     g_graphicsContext.RemoveTransform();
 }
 
-bool CGUITextLayout::UpdateScrollinfo(CScrollInfo &scrollInfo)
-{
-  if (!m_font)
-    return false;
-  if (m_lines.empty())
-    return false;
 
-  return m_font->UpdateScrollInfo(m_lines[0].m_text, scrollInfo);
-}
-
-
-void CGUITextLayout::RenderScrolling(float x, float y, float angle, color_t color, color_t shadowColor, uint32_t alignment, float maxWidth, const CScrollInfo &scrollInfo)
+void CGUITextLayout::RenderScrolling(float x, float y, float angle, color_t color, color_t shadowColor, uint32_t alignment, float maxWidth, CScrollInfo &scrollInfo)
 {
   if (!m_font)
     return;
@@ -136,12 +127,15 @@ void CGUITextLayout::RenderScrolling(float x, float y, float angle, color_t colo
   //       any difference to the smoothness of scrolling though which will be
   //       jumpy with this sort of thing.  It's not exactly a well used situation
   //       though, so this hack is probably OK.
-  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+  float speed = scrollInfo.pixelSpeed;
+  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); i++)
   {
     const CGUIString &string = *i;
     m_font->DrawScrollingText(x, y, m_colors, shadowColor, string.m_text, alignment, maxWidth, scrollInfo);
     y += m_font->GetLineHeight();
+    scrollInfo.pixelSpeed = 0;
   }
+  scrollInfo.pixelSpeed = speed;
   m_font->End();
   if (angle)
     g_graphicsContext.RemoveTransform();
@@ -168,7 +162,7 @@ void CGUITextLayout::RenderOutline(float x, float y, color_t color, color_t outl
     // adjust so the baselines of the fonts align
     float by = y + m_font->GetTextBaseLine() - m_borderFont->GetTextBaseLine();
     m_borderFont->Begin();
-    for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+    for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); i++)
     {
       const CGUIString &string = *i;
       uint32_t align = alignment;
@@ -187,9 +181,7 @@ void CGUITextLayout::RenderOutline(float x, float y, color_t color, color_t outl
         align &= ~XBFONT_CENTER_X;
       }
 
-      // don't pass maxWidth through to the renderer for the same reason above: it will cause clipping
-      // on the left.
-      m_borderFont->DrawText(bx, by, outlineColors, 0, string.m_text, align, 0);
+      m_borderFont->DrawText(bx, by, outlineColors, 0, string.m_text, align, maxWidth);
       by += m_borderFont->GetLineHeight();
     }
     m_borderFont->End();
@@ -200,15 +192,14 @@ void CGUITextLayout::RenderOutline(float x, float y, color_t color, color_t outl
     m_colors[0] = color;
 
   m_font->Begin();
-  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); i++)
   {
     const CGUIString &string = *i;
     uint32_t align = alignment;
     if (align & XBFONT_JUSTIFIED && string.m_carriageReturn)
       align &= ~XBFONT_JUSTIFIED;
 
-    // don't pass maxWidth through to the renderer for the reason above.
-    m_font->DrawText(x, y, m_colors, 0, string.m_text, align, 0);
+    m_font->DrawText(x, y, m_colors, 0, string.m_text, align, maxWidth);
     y += m_font->GetLineHeight();
   }
   m_font->End();
@@ -216,50 +207,37 @@ void CGUITextLayout::RenderOutline(float x, float y, color_t color, color_t outl
 
 bool CGUITextLayout::Update(const CStdString &text, float maxWidth, bool forceUpdate /*= false*/, bool forceLTRReadingOrder /*= false*/)
 {
-  if (text == m_lastUtf8Text && !forceUpdate && !m_lastUpdateW)
-    return false;
-
-  m_lastUtf8Text = text;
-  m_lastUpdateW = false;
+  // convert to utf16
   CStdStringW utf16;
-  g_charsetConverter.utf8ToW(text, utf16, false);
-  UpdateCommon(utf16, maxWidth, forceLTRReadingOrder);
-  return true;
+  utf8ToW(text, utf16);
+
+  // update
+  return UpdateW(utf16, maxWidth, forceUpdate, forceLTRReadingOrder);
 }
 
 bool CGUITextLayout::UpdateW(const CStdStringW &text, float maxWidth /*= 0*/, bool forceUpdate /*= false*/, bool forceLTRReadingOrder /*= false*/)
 {
-  if (text == m_lastText && !forceUpdate && m_lastUpdateW)
+  if (text.Equals(m_lastText) && !forceUpdate)
     return false;
 
-  m_lastText = text;
-  m_lastUpdateW = true;
-  UpdateCommon(text, maxWidth, forceLTRReadingOrder);
-  return true;
-}
-
-void CGUITextLayout::UpdateCommon(const CStdStringW &text, float maxWidth, bool forceLTRReadingOrder)
-{
-  // parse the text for style information
   vecText parsedText;
-  vecColors colors;
-  ParseText(text, m_font ? m_font->GetStyle() : 0, m_textColor, colors, parsedText);
 
-  // and update
-  UpdateStyled(parsedText, colors, maxWidth, forceLTRReadingOrder);
-}
-
-void CGUITextLayout::UpdateStyled(const vecText &text, const vecColors &colors, float maxWidth, bool forceLTRReadingOrder)
-{
   // empty out our previous string
   m_lines.clear();
-  m_colors = colors;
+  m_colors.clear();
+  m_colors.push_back(m_textColor);
+
+  // parse the text into our string objects
+  ParseText(text, parsedText);
+
+  // add \n to the end of the string
+  parsedText.push_back(L'\n');
 
   // if we need to wrap the text, then do so
   if (m_wrap && maxWidth > 0)
-    WrapText(text, maxWidth);
+    WrapText(parsedText, maxWidth);
   else
-    LineBreakText(text, m_lines);
+    LineBreakText(parsedText, m_lines);
 
   // remove any trailing blank lines
   while (!m_lines.empty() && m_lines.back().m_text.empty())
@@ -269,6 +247,9 @@ void CGUITextLayout::UpdateStyled(const vecText &text, const vecColors &colors,
 
   // and cache the width and height for later reading
   CalcTextExtent();
+
+  m_lastText = text;
+  return true;
 }
 
 // BidiTransform is used to handle RTL text flipping in the string
@@ -289,7 +270,7 @@ void CGUITextLayout::BidiTransform(vector<CGUIString> &lines, bool forceLTRReadi
       character_t style = *it & 0xffff0000;
       if (style != sectionStyle)
       {
-        if (!sectionText.empty())
+        if (!sectionText.IsEmpty())
         { // style has changed, bidi flip text
           CStdStringW sectionFlipped = BidiFlip(sectionText, forceLTRReadingOrder);
           for (unsigned int j = 0; j < sectionFlipped.size(); j++)
@@ -302,7 +283,7 @@ void CGUITextLayout::BidiTransform(vector<CGUIString> &lines, bool forceLTRReadi
     }
 
     // handle the last section
-    if (!sectionText.empty())
+    if (!sectionText.IsEmpty())
     {
       CStdStringW sectionFlipped = BidiFlip(sectionText, forceLTRReadingOrder);
       for (unsigned int j = 0; j < sectionFlipped.size(); j++)
@@ -329,17 +310,24 @@ CStdStringW CGUITextLayout::BidiFlip(const CStdStringW &text, bool forceLTRReadi
 void CGUITextLayout::Filter(CStdString &text)
 {
   CStdStringW utf16;
-  g_charsetConverter.utf8ToW(text, utf16, false);
+  utf8ToW(text, utf16);
   vecColors colors;
   vecText parsedText;
-  ParseText(utf16, 0, 0xffffffff, colors, parsedText);
-  utf16.clear();
+  ParseText(utf16, 0, colors, parsedText);
+  utf16.Empty();
   for (unsigned int i = 0; i < parsedText.size(); i++)
     utf16 += (wchar_t)(0xffff & parsedText[i]);
   g_charsetConverter.wToUTF8(utf16, text);
 }
 
-void CGUITextLayout::ParseText(const CStdStringW &text, uint32_t defaultStyle, color_t defaultColor, vecColors &colors, vecText &parsedText)
+void CGUITextLayout::ParseText(const CStdStringW &text, vecText &parsedText)
+{
+  if (!m_font)
+    return;
+  ParseText(text, m_font->GetStyle(), m_colors, parsedText);
+}
+
+void CGUITextLayout::ParseText(const CStdStringW &text, uint32_t defaultStyle, vecColors &colors, vecText &parsedText)
 {
   // run through the string, searching for:
   // [B] or [/B] -> toggle bold on and off
@@ -350,7 +338,6 @@ void CGUITextLayout::ParseText(const CStdStringW &text, uint32_t defaultStyle, c
   uint32_t currentStyle = defaultStyle; // start with the default font's style
   color_t currentColor = 0;
 
-  colors.push_back(defaultColor);
   stack<color_t> colorStack;
   colorStack.push(0);
 
@@ -359,97 +346,79 @@ void CGUITextLayout::ParseText(const CStdStringW &text, uint32_t defaultStyle, c
   // but [LOWERCASE]Glah[UPPERCASE]FReD[/UPPERCASE]Georeg[/LOWERCASE] won't
 
   int startPos = 0;
-  size_t pos = text.find(L'[');
-  while (pos != std::string::npos && pos + 1 < text.size())
+  size_t pos = text.Find(L'[');
+  while (pos != CStdString::npos && pos + 1 < text.size())
   {
     uint32_t newStyle = 0;
     color_t newColor = currentColor;
-    bool colorTagChange = false;
     bool newLine = false;
     // have a [ - check if it's an ON or OFF switch
     bool on(true);
-    size_t endPos = pos++; // finish of string
+    int endPos = pos++; // finish of string
     if (text[pos] == L'/')
     {
       on = false;
       pos++;
     }
     // check for each type
-    if (text.compare(pos, 2, L"B]") == 0)
+    if (text.Mid(pos,2) == L"B]")
     { // bold - finish the current text block and assign the bold state
       pos += 2;
-      if ((on && text.find(L"[/B]",pos) != std::string::npos) ||          // check for a matching end point
+      if ((on && text.Find(L"[/B]",pos) >= 0) ||          // check for a matching end point
          (!on && (currentStyle & FONT_STYLE_BOLD)))       // or matching start point
         newStyle = FONT_STYLE_BOLD;
     }
-    else if (text.compare(pos, 2, L"I]") == 0)
+    else if (text.Mid(pos,2) == L"I]")
     { // italics
       pos += 2;
-      if ((on && text.find(L"[/I]", pos) != std::string::npos) ||          // check for a matching end point
+      if ((on && text.Find(L"[/I]",pos) >= 0) ||          // check for a matching end point
          (!on && (currentStyle & FONT_STYLE_ITALICS)))    // or matching start point
         newStyle = FONT_STYLE_ITALICS;
     }
-    else if (text.compare(pos, 10, L"UPPERCASE]") == 0)
+    else if (text.Mid(pos,10) == L"UPPERCASE]")
     {
       pos += 10;
-      if ((on && text.find(L"[/UPPERCASE]", pos) != std::string::npos) ||  // check for a matching end point
+      if ((on && text.Find(L"[/UPPERCASE]",pos) >= 0) ||  // check for a matching end point
          (!on && (currentStyle & FONT_STYLE_UPPERCASE)))  // or matching start point
         newStyle = FONT_STYLE_UPPERCASE;
     }
-    else if (text.compare(pos, 10, L"LOWERCASE]") == 0)
+    else if (text.Mid(pos,10) == L"LOWERCASE]")
     {
       pos += 10;
-      if ((on && text.find(L"[/LOWERCASE]", pos) != std::string::npos) ||  // check for a matching end point
+      if ((on && text.Find(L"[/LOWERCASE]",pos) >= 0) ||  // check for a matching end point
          (!on && (currentStyle & FONT_STYLE_LOWERCASE)))  // or matching start point
         newStyle = FONT_STYLE_LOWERCASE;
     }
-    else if (text.compare(pos, 3, L"CR]") == 0 && on)
+    else if (text.Mid(pos,3) == L"CR]" && on)
     {
       newLine = true;
       pos += 3;
     }
-    else if (text.compare(pos,5, L"COLOR") == 0)
+    else if (text.Mid(pos,5) == L"COLOR")
     { // color
-      size_t finish = text.find(L']', pos + 5);
-      if (on && finish != std::string::npos && text.find(L"[/COLOR]",finish) != std::string::npos)
-      {
-        std::string t;
-        g_charsetConverter.wToUTF8(text.substr(pos + 5, finish - pos - 5), t);
-        color_t color = g_colorManager.GetColor(t);
-        vecColors::const_iterator it = std::find(colors.begin(), colors.end(), color);
-        if (it == colors.end())
-        { // create new color
-          if (colors.size() <= 0xFF)
-          {
-            newColor = colors.size();
-            colors.push_back(color);
-          }
-          else // we have only 8 bits for color index, fallback to first color if reach max.
-            newColor = 0;
-        }
-        else
-          // reuse existing color
-          newColor = it - colors.begin();
+      size_t finish = text.Find(L']', pos + 5);
+      if (on && finish != CStdString::npos && (size_t)text.Find(L"[/COLOR]",finish) != CStdString::npos)
+      { // create new color
+        newColor = colors.size();
+        colors.push_back(g_colorManager.GetColor(text.Mid(pos + 5, finish - pos - 5)));
         colorStack.push(newColor);
-        colorTagChange = true;
       }
       else if (!on && finish == pos + 5 && colorStack.size() > 1)
       { // revert to previous color
         colorStack.pop();
         newColor = colorStack.top();
-        colorTagChange = true;
       }
       if (finish != CStdString::npos)
         pos = finish + 1;
     }
 
-    if (newStyle || colorTagChange || newLine)
+    if (newStyle || newColor != currentColor || newLine)
     { // we have a new style or a new color, so format up the previous segment
-      CStdStringW subText = text.substr(startPos, endPos - startPos);
+      CStdStringW subText = text.Mid(startPos, endPos - startPos);
       if (currentStyle & FONT_STYLE_UPPERCASE)
-        StringUtils::ToUpper(subText);
+        subText.ToUpper();
       if (currentStyle & FONT_STYLE_LOWERCASE)
-        StringUtils::ToLower(subText);
+        subText.ToLower();
       AppendToUTF32(subText, ((currentStyle & 3) << 24) | (currentColor << 16), parsedText);
       if (newLine)
         parsedText.push_back(L'\n');
@@ -462,14 +431,14 @@ void CGUITextLayout::ParseText(const CStdStringW &text, uint32_t defaultStyle, c
       else
         currentStyle &= ~newStyle;
     }
-    pos = text.find(L'[', pos);
+    pos = text.Find(L'[',pos);
   }
   // now grab the remainder of the string
-  CStdStringW subText = text.substr(startPos);
+  CStdStringW subText = text.Mid(startPos, text.GetLength() - startPos);
   if (currentStyle & FONT_STYLE_UPPERCASE)
-    StringUtils::ToUpper(subText);
+    subText.ToUpper();
   if (currentStyle & FONT_STYLE_LOWERCASE)
-    StringUtils::ToLower(subText);
+    subText.ToLower();
   AppendToUTF32(subText, ((currentStyle & 3) << 24) | (currentColor << 16), parsedText);
 }
 
@@ -517,7 +486,7 @@ void CGUITextLayout::WrapText(const vecText &text, float maxWidth)
             // skip over spaces
             pos = lastSpace;
             while (pos != line.m_text.end() && IsSpace(*pos))
-              ++pos;
+              pos++;
             curLine.clear();
             lastSpaceInLine = 0;
             lastSpace = line.m_text.begin();
@@ -528,7 +497,7 @@ void CGUITextLayout::WrapText(const vecText &text, float maxWidth)
         lastSpaceInLine = curLine.size();
       }
       curLine.push_back(letter);
-      ++pos;
+      pos++;
     }
     // now add whatever we have left to the string
     float width = m_font->GetTextWidth(curLine);
@@ -572,13 +541,7 @@ void CGUITextLayout::LineBreakText(const vecText &text, vector<CGUIString> &line
       lines.push_back(string);
       lineStart = pos + 1;
     }
-    ++pos;
-  }
-  // handle the last line if non-empty
-  if (lineStart < text.end() && (nMaxLines <= 0 || lines.size() < (size_t)nMaxLines))
-  {
-    CGUIString string(lineStart, text.end(), true);
-    lines.push_back(string);
+    pos++;
   }
 }
 
@@ -594,7 +557,7 @@ void CGUITextLayout::CalcTextExtent()
   m_textHeight = 0;
   if (!m_font) return;
 
-  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+  for (vector<CGUIString>::iterator i = m_lines.begin(); i != m_lines.end(); i++)
   {
     const CGUIString &string = *i;
     float w = m_font->GetTextWidth(string.m_text);
@@ -607,7 +570,7 @@ void CGUITextLayout::CalcTextExtent()
 unsigned int CGUITextLayout::GetTextLength() const
 {
   unsigned int length = 0;
-  for (vector<CGUIString>::const_iterator i = m_lines.begin(); i != m_lines.end(); ++i)
+  for (vector<CGUIString>::const_iterator i = m_lines.begin(); i != m_lines.end(); i++)
     length += i->m_text.size();
   return length;
 }
@@ -628,17 +591,6 @@ float CGUITextLayout::GetTextWidth(const CStdStringW &text) const
   return m_font->GetTextWidth(utf32);
 }
 
-std::string CGUITextLayout::GetText() const
-{
-  if (m_lastUpdateW)
-  {
-    std::string utf8;
-    g_charsetConverter.wToUTF8(m_lastText, utf8);
-    return utf8;
-  }
-  return m_lastUtf8Text;
-}
-
 void CGUITextLayout::DrawText(CGUIFont *font, float x, float y, color_t color, color_t shadowColor, const CStdString &text, uint32_t align)
 {
   if (!font) return;
@@ -655,19 +607,40 @@ void CGUITextLayout::AppendToUTF32(const CStdStringW &utf16, character_t colStyl
     utf32.push_back(utf16[i] | colStyle);
 }
 
+void CGUITextLayout::utf8ToW(const CStdString &utf8, CStdStringW &utf16)
+{
+#ifdef WORK_AROUND_NEEDED_FOR_LINE_BREAKS
+  // NOTE: This appears to strip \n characters from text.  This may be a consequence of incorrect
+  //       expression of the \n in utf8 (we just use character code 10) or it might be something
+  //       more sinister.  For now, we use the workaround below.
+  CStdStringArray multiLines;
+  StringUtils::SplitString(utf8, "\n", multiLines);
+  for (unsigned int i = 0; i < multiLines.size(); i++)
+  {
+    CStdStringW line;
+    // no need to bidiflip here - it's done in BidiTransform above
+    g_charsetConverter.utf8ToW(multiLines[i], line, false);
+    utf16 += line;
+    if (i < multiLines.size() - 1)
+      utf16.push_back(L'\n');
+  }
+#else
+  // no need to bidiflip here - it's done in BidiTransform above
+  g_charsetConverter.utf8ToW(utf8, utf16, false);
+#endif
+}
+
 void CGUITextLayout::AppendToUTF32(const CStdString &utf8, character_t colStyle, vecText &utf32)
 {
   CStdStringW utf16;
-  // no need to bidiflip here - it's done in BidiTransform above
-  g_charsetConverter.utf8ToW(utf8, utf16, false);
+  utf8ToW(utf8, utf16);
   AppendToUTF32(utf16, colStyle, utf32);
 }
 
 void CGUITextLayout::Reset()
 {
   m_lines.clear();
-  m_lastText.clear();
-  m_lastUtf8Text.clear();
+  m_lastText.Empty();
   m_textWidth = m_textHeight = 0;
 }
 
