@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,8 @@
 
 #include "VideoLibrary.h"
 #include "ApplicationMessenger.h"
-#include "TextureDatabase.h"
+#include "TextureCache.h"
 #include "Util.h"
-#include "utils/StringUtils.h"
 #include "utils/URIUtils.h"
 #include "video/VideoDatabase.h"
 
@@ -40,7 +39,7 @@ JSONRPC_STATUS CVideoLibrary::GetMovies(const CStdString &method, ITransportLaye
     return InvalidParams;
 
   CVideoDbUrl videoUrl;
-  videoUrl.FromString("videodb://movies/titles/");
+  videoUrl.FromString("videodb://1/2/");
   int genreID = -1, year = -1, setID = 0;
   const CVariant &filter = parameterObject["filter"];
   if (filter.isMember("genreid"))
@@ -106,7 +105,7 @@ JSONRPC_STATUS CVideoLibrary::GetMovieSets(const CStdString &method, ITransportL
     return InternalError;
 
   CFileItemList items;
-  if (!videodatabase.GetSetsNav("videodb://movies/sets/", items, VIDEODB_CONTENT_MOVIES))
+  if (!videodatabase.GetSetsNav("videodb://1/7/", items, VIDEODB_CONTENT_MOVIES))
     return InternalError;
 
   HandleFileItemList("setid", false, "sets", items, parameterObject, result);
@@ -130,7 +129,7 @@ JSONRPC_STATUS CVideoLibrary::GetMovieSetDetails(const CStdString &method, ITran
 
   // Get movies from the set
   CFileItemList items;
-  if (!videodatabase.GetMoviesNav("videodb://movies/titles/", items, -1, -1, -1, -1, -1, -1, id))
+  if (!videodatabase.GetMoviesNav("videodb://1/2/", items, -1, -1, -1, -1, -1, -1, id))
     return InternalError;
 
   return GetAdditionalMovieDetails(parameterObject["movies"], items, result["setdetails"], videodatabase, true);
@@ -148,7 +147,7 @@ JSONRPC_STATUS CVideoLibrary::GetTVShows(const CStdString &method, ITransportLay
     return InvalidParams;
 
   CVideoDbUrl videoUrl;
-  videoUrl.FromString("videodb://tvshows/titles/");
+  videoUrl.FromString("videodb://2/2/");
   int genreID = -1, year = -1;
   const CVariant &filter = parameterObject["filter"];
   if (filter.isMember("genreid"))
@@ -206,13 +205,11 @@ JSONRPC_STATUS CVideoLibrary::GetTVShowDetails(const CStdString &method, ITransp
 
   int id = (int)parameterObject["tvshowid"].asInteger();
 
-  CFileItemPtr fileItem(new CFileItem());
   CVideoInfoTag infos;
-  if (!videodatabase.GetTvShowInfo("", infos, id, fileItem.get()) || infos.m_iDbId <= 0)
+  if (!videodatabase.GetTvShowInfo("", infos, id) || infos.m_iDbId <= 0)
     return InvalidParams;
 
-  fileItem->SetFromVideoInfoTag(infos);
-  HandleFileItem("tvshowid", true, "tvshowdetails", fileItem, parameterObject, parameterObject["properties"], result, false);
+  HandleFileItem("tvshowid", true, "tvshowdetails", CFileItemPtr(new CFileItem(infos)), parameterObject, parameterObject["properties"], result, false);
   return OK;
 }
 
@@ -224,30 +221,13 @@ JSONRPC_STATUS CVideoLibrary::GetSeasons(const CStdString &method, ITransportLay
 
   int tvshowID = (int)parameterObject["tvshowid"].asInteger();
 
-  CStdString strPath = StringUtils::Format("videodb://tvshows/titles/%i/", tvshowID);
+  CStdString strPath;
+  strPath.Format("videodb://2/2/%i/", tvshowID);
   CFileItemList items;
   if (!videodatabase.GetSeasonsNav(strPath, items, -1, -1, -1, -1, tvshowID, false))
     return InternalError;
 
-  HandleFileItemList("seasonid", false, "seasons", items, parameterObject, result);
-  return OK;
-}
-
-JSONRPC_STATUS CVideoLibrary::GetSeasonDetails(const CStdString &method, ITransportLayer *transport, IClient *client, const CVariant &parameterObject, CVariant &result)
-{
-  CVideoDatabase videodatabase;
-  if (!videodatabase.Open())
-    return InternalError;
-
-  int id = (int)parameterObject["seasonid"].asInteger();
-
-  CVideoInfoTag infos;
-  if (!videodatabase.GetSeasonInfo(id, infos) ||
-      infos.m_iDbId <= 0 || infos.m_iIdShow <= 0)
-    return InvalidParams;
-  
-  CFileItemPtr pItem = CFileItemPtr(new CFileItem(infos));
-  HandleFileItem("seasonid", false, "seasondetails", pItem, parameterObject, parameterObject["properties"], result, false);
+  HandleFileItemList(NULL, false, "seasons", items, parameterObject, result);
   return OK;
 }
 
@@ -265,7 +245,8 @@ JSONRPC_STATUS CVideoLibrary::GetEpisodes(const CStdString &method, ITransportLa
   int tvshowID = (int)parameterObject["tvshowid"].asInteger();
   int season   = (int)parameterObject["season"].asInteger();
   
-  CStdString strPath = StringUtils::Format("videodb://tvshows/titles/%i/%i/", tvshowID, season);
+  CStdString strPath;
+  strPath.Format("videodb://2/2/%i/%i/", tvshowID, season);
 
   CVideoDbUrl videoUrl;
   videoUrl.FromString(strPath);
@@ -324,7 +305,7 @@ JSONRPC_STATUS CVideoLibrary::GetEpisodeDetails(const CStdString &method, ITrans
   if (tvshowid <= 0)
     tvshowid = videodatabase.GetTvShowForEpisode(id);
 
-  CStdString basePath = StringUtils::Format("videodb://tvshows/titles/%ld/%ld/%ld", tvshowid, infos.m_iSeason, id);
+  CStdString basePath; basePath.Format("videodb://2/2/%ld/%ld/%ld", tvshowid, infos.m_iSeason, id);
   pItem->SetPath(basePath);
 
   HandleFileItem("episodeid", true, "episodedetails", pItem, parameterObject, parameterObject["properties"], result, false);
@@ -343,7 +324,7 @@ JSONRPC_STATUS CVideoLibrary::GetMusicVideos(const CStdString &method, ITranspor
     return InvalidParams;
 
   CVideoDbUrl videoUrl;
-  videoUrl.FromString("videodb://musicvideos/titles/");
+  videoUrl.FromString("videodb://3/2/");
   int genreID = -1, year = -1;
   const CVariant &filter = parameterObject["filter"];
   if (filter.isMember("artist"))
@@ -399,7 +380,7 @@ JSONRPC_STATUS CVideoLibrary::GetRecentlyAddedMovies(const CStdString &method, I
     return InternalError;
 
   CFileItemList items;
-  if (!videodatabase.GetRecentlyAddedMoviesNav("videodb://recentlyaddedmovies/", items))
+  if (!videodatabase.GetRecentlyAddedMoviesNav("videodb://4/", items))
     return InternalError;
 
   return GetAdditionalMovieDetails(parameterObject, items, result, videodatabase, true);
@@ -412,7 +393,7 @@ JSONRPC_STATUS CVideoLibrary::GetRecentlyAddedEpisodes(const CStdString &method,
     return InternalError;
 
   CFileItemList items;
-  if (!videodatabase.GetRecentlyAddedEpisodesNav("videodb://recentlyaddedepisodes/", items))
+  if (!videodatabase.GetRecentlyAddedEpisodesNav("videodb://5/", items))
     return InternalError;
 
   return GetAdditionalEpisodeDetails(parameterObject, items, result, videodatabase, true);
@@ -425,7 +406,7 @@ JSONRPC_STATUS CVideoLibrary::GetRecentlyAddedMusicVideos(const CStdString &meth
     return InternalError;
 
   CFileItemList items;
-  if (!videodatabase.GetRecentlyAddedMusicVideosNav("videodb://recentlyaddedmusicvideos/", items))
+  if (!videodatabase.GetRecentlyAddedMusicVideosNav("videodb://6/", items))
     return InternalError;
 
   return GetAdditionalMusicVideoDetails(parameterObject, items, result, videodatabase, true);
@@ -434,27 +415,27 @@ JSONRPC_STATUS CVideoLibrary::GetRecentlyAddedMusicVideos(const CStdString &meth
 JSONRPC_STATUS CVideoLibrary::GetGenres(const CStdString &method, ITransportLayer *transport, IClient *client, const CVariant &parameterObject, CVariant &result)
 {
   CStdString media = parameterObject["type"].asString();
-  StringUtils::ToLower(media);
+  media = media.ToLower();
   int idContent = -1;
 
   CStdString strPath = "videodb://";
   /* select which video content to get genres from*/
-  if (media.Equals(MediaTypeMovie))
+  if (media.Equals("movie"))
   {
     idContent = VIDEODB_CONTENT_MOVIES;
-    strPath += "movies";
+    strPath += "1";
   }
-  else if (media.Equals(MediaTypeTvShow))
+  else if (media.Equals("tvshow"))
   {
     idContent = VIDEODB_CONTENT_TVSHOWS;
-    strPath += "tvshows";
+    strPath += "2";
   }
-  else if (media.Equals(MediaTypeMusicVideo))
+  else if (media.Equals("musicvideo"))
   {
     idContent = VIDEODB_CONTENT_MUSICVIDEOS;
-    strPath += "musicvideos";
+    strPath += "3";
   }
-  strPath += "/genres/";
+  strPath += "/1/";
  
   CVideoDatabase videodatabase;
   if (!videodatabase.Open())
@@ -491,62 +472,23 @@ JSONRPC_STATUS CVideoLibrary::SetMovieDetails(const CStdString &method, ITranspo
   int playcount = infos.m_playCount;
   CDateTime lastPlayed = infos.m_lastPlayed;
 
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
+  UpdateVideoTag(parameterObject, infos, artwork);
 
   // we need to manually remove tags/taglinks for now because they aren't replaced
   // due to scrapers not supporting them
-  videodatabase.RemoveTagsFromItem(id, MediaTypeMovie);
+  videodatabase.RemoveTagsFromItem(id, "movie");
 
   if (videodatabase.SetDetailsForMovie(infos.m_strFileNameAndPath, infos, artwork, id) <= 0)
     return InternalError;
 
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, MediaTypeMovie, removedArtwork))
-    return InternalError;
-
   if (playcount != infos.m_playCount || lastPlayed != infos.m_lastPlayed)
   {
     // restore original playcount or the new one won't be announced
     int newPlaycount = infos.m_playCount;
     infos.m_playCount = playcount;
-    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed);
+    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed.IsValid() ? infos.m_lastPlayed : CDateTime::GetCurrentDateTime());
   }
 
-  UpdateResumePoint(parameterObject, infos, videodatabase);
-
-  CJSONRPCUtils::NotifyItemUpdated();
-  return ACK;
-}
-
-JSONRPC_STATUS CVideoLibrary::SetMovieSetDetails(const CStdString &method, ITransportLayer *transport, IClient *client, const CVariant &parameterObject, CVariant &result)
-{
-  int id = (int)parameterObject["setid"].asInteger();
-
-  CVideoDatabase videodatabase;
-  if (!videodatabase.Open())
-    return InternalError;
-
-  CVideoInfoTag infos;
-  videodatabase.GetSetInfo(id, infos);
-  if (infos.m_iDbId <= 0)
-  {
-    videodatabase.Close();
-    return InvalidParams;
-  }
-
-  // get artwork
-  std::map<std::string, std::string> artwork;
-  videodatabase.GetArtForItem(infos.m_iDbId, infos.m_type, artwork);
-
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
-
-  if (videodatabase.SetDetailsForMovieSet(infos, artwork, id) <= 0)
-    return InternalError;
-
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, "set", removedArtwork))
-    return InternalError;
-
   CJSONRPCUtils::NotifyItemUpdated();
   return ACK;
 }
@@ -570,52 +512,26 @@ JSONRPC_STATUS CVideoLibrary::SetTVShowDetails(const CStdString &method, ITransp
   std::map<int, std::map<std::string, std::string> > seasonArt;
   videodatabase.GetTvShowSeasonArt(infos.m_iDbId, seasonArt);
 
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
+  int playcount = infos.m_playCount;
+  CDateTime lastPlayed = infos.m_lastPlayed;
+
+  UpdateVideoTag(parameterObject, infos, artwork);
 
   // we need to manually remove tags/taglinks for now because they aren't replaced
   // due to scrapers not supporting them
-  videodatabase.RemoveTagsFromItem(id, MediaTypeTvShow);
+  videodatabase.RemoveTagsFromItem(id, "tvshow");
 
   if (videodatabase.SetDetailsForTvShow(infos.m_strFileNameAndPath, infos, artwork, seasonArt, id) <= 0)
     return InternalError;
 
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, MediaTypeTvShow, removedArtwork))
-    return InternalError;
-
-  CJSONRPCUtils::NotifyItemUpdated();
-  return ACK;
-}
-
-JSONRPC_STATUS CVideoLibrary::SetSeasonDetails(const CStdString &method, ITransportLayer *transport, IClient *client, const CVariant &parameterObject, CVariant &result)
-{
-  int id = (int)parameterObject["seasonid"].asInteger();
-
-  CVideoDatabase videodatabase;
-  if (!videodatabase.Open())
-    return InternalError;
-
-  CVideoInfoTag infos;
-  videodatabase.GetSeasonInfo(id, infos);
-  if (infos.m_iDbId <= 0 || infos.m_iIdShow <= 0)
+  if (playcount != infos.m_playCount || lastPlayed != infos.m_lastPlayed)
   {
-    videodatabase.Close();
-    return InvalidParams;
+    // restore original playcount or the new one won't be announced
+    int newPlaycount = infos.m_playCount;
+    infos.m_playCount = playcount;
+    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed.IsValid() ? infos.m_lastPlayed : CDateTime::GetCurrentDateTime());
   }
 
-  // get artwork
-  std::map<std::string, std::string> artwork;
-  videodatabase.GetArtForItem(infos.m_iDbId, infos.m_type, artwork);
-
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
-
-  if (videodatabase.SetDetailsForSeason(infos, artwork, infos.m_iIdShow, id) <= 0)
-    return InternalError;
-
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, MediaTypeSeason, removedArtwork))
-    return InternalError;
-
   CJSONRPCUtils::NotifyItemUpdated();
   return ACK;
 }
@@ -650,25 +566,19 @@ JSONRPC_STATUS CVideoLibrary::SetEpisodeDetails(const CStdString &method, ITrans
   int playcount = infos.m_playCount;
   CDateTime lastPlayed = infos.m_lastPlayed;
 
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
+  UpdateVideoTag(parameterObject, infos, artwork);
 
   if (videodatabase.SetDetailsForEpisode(infos.m_strFileNameAndPath, infos, artwork, tvshowid, id) <= 0)
     return InternalError;
 
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, MediaTypeEpisode, removedArtwork))
-    return InternalError;
-
   if (playcount != infos.m_playCount || lastPlayed != infos.m_lastPlayed)
   {
     // restore original playcount or the new one won't be announced
     int newPlaycount = infos.m_playCount;
     infos.m_playCount = playcount;
-    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed);
+    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed.IsValid() ? infos.m_lastPlayed : CDateTime::GetCurrentDateTime());
   }
 
-  UpdateResumePoint(parameterObject, infos, videodatabase);
-
   CJSONRPCUtils::NotifyItemUpdated();
   return ACK;
 }
@@ -696,29 +606,23 @@ JSONRPC_STATUS CVideoLibrary::SetMusicVideoDetails(const CStdString &method, ITr
   int playcount = infos.m_playCount;
   CDateTime lastPlayed = infos.m_lastPlayed;
 
-  std::set<std::string> removedArtwork;
-  UpdateVideoTag(parameterObject, infos, artwork, removedArtwork);
+  UpdateVideoTag(parameterObject, infos, artwork);
 
   // we need to manually remove tags/taglinks for now because they aren't replaced
   // due to scrapers not supporting them
-  videodatabase.RemoveTagsFromItem(id, MediaTypeMusicVideo);
+  videodatabase.RemoveTagsFromItem(id, "musicvideo");
 
   if (videodatabase.SetDetailsForMusicVideo(infos.m_strFileNameAndPath, infos, artwork, id) <= 0)
     return InternalError;
 
-  if (!videodatabase.RemoveArtForItem(infos.m_iDbId, MediaTypeMusicVideo, removedArtwork))
-    return InternalError;
-
   if (playcount != infos.m_playCount || lastPlayed != infos.m_lastPlayed)
   {
     // restore original playcount or the new one won't be announced
     int newPlaycount = infos.m_playCount;
     infos.m_playCount = playcount;
-    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed);
+    videodatabase.SetPlayCount(CFileItem(infos), newPlaycount, infos.m_lastPlayed.IsValid() ? infos.m_lastPlayed : CDateTime::GetCurrentDateTime());
   }
 
-  UpdateResumePoint(parameterObject, infos, videodatabase);
-
   CJSONRPCUtils::NotifyItemUpdated();
   return ACK;
 }
@@ -750,7 +654,7 @@ JSONRPC_STATUS CVideoLibrary::Scan(const CStdString &method, ITransportLayer *tr
   if (directory.empty())
     cmd = "updatelibrary(video)";
   else
-    cmd = StringUtils::Format("updatelibrary(video, %s)", StringUtils::Paramify(directory).c_str());
+    cmd.Format("updatelibrary(video, %s)", directory.c_str());
 
   CApplicationMessenger::Get().ExecBuiltIn(cmd);
   return ACK;
@@ -760,12 +664,12 @@ JSONRPC_STATUS CVideoLibrary::Export(const CStdString &method, ITransportLayer *
 {
   CStdString cmd;
   if (parameterObject["options"].isMember("path"))
-    cmd = StringUtils::Format("exportlibrary(video, false, %s)", StringUtils::Paramify(parameterObject["options"]["path"].asString()).c_str());
+    cmd.Format("exportlibrary(video, false, %s)", parameterObject["options"]["path"].asString());
   else
-    cmd = StringUtils::Format("exportlibrary(video, true, %s, %s, %s)",
-                              parameterObject["options"]["images"].asBoolean() ? "true" : "false",
-                              parameterObject["options"]["overwrite"].asBoolean() ? "true" : "false",
-                              parameterObject["options"]["actorthumbs"].asBoolean() ? "true" : "false");
+    cmd.Format("exportlibrary(video, true, %s, %s, %s)",
+      parameterObject["options"]["images"].asBoolean() ? "true" : "false",
+      parameterObject["options"]["overwrite"].asBoolean() ? "true" : "false",
+      parameterObject["options"]["actorthumbs"].asBoolean() ? "true" : "false");
 
   CApplicationMessenger::Get().ExecBuiltIn(cmd);
   return ACK;
@@ -780,28 +684,19 @@ JSONRPC_STATUS CVideoLibrary::Clean(const CStdString &method, ITransportLayer *t
 bool CVideoLibrary::FillFileItem(const CStdString &strFilename, CFileItemPtr &item, const CVariant &parameterObject /* = CVariant(CVariant::VariantTypeArray) */)
 {
   CVideoDatabase videodatabase;
-  if (strFilename.empty())
+  if (strFilename.empty() || !videodatabase.Open())
     return false;
-  
-  bool filled = false;
-  if (videodatabase.Open())
-  {
-    CVideoInfoTag details;
-    if (videodatabase.LoadVideoInfo(strFilename, details))
-    {
-      item->SetFromVideoInfoTag(details);
-      filled = true;
-    }
-  }
 
+  CVideoInfoTag details;
+  if (!videodatabase.LoadVideoInfo(strFilename, details))
+    return false;
+
+  item->SetFromVideoInfoTag(details);
   if (item->GetLabel().empty())
-  {
     item->SetLabel(CUtil::GetTitleFromPath(strFilename, false));
-    if (item->GetLabel().empty())
-      item->SetLabel(URIUtils::GetFileName(strFilename));
-  }
-
-  return filled;
+  if (item->GetLabel())
+    item->SetLabel(URIUtils::GetFileName(strFilename));
+  return true;
 }
 
 bool CVideoLibrary::FillFileItemList(const CVariant &parameterObject, CFileItemList &list)
@@ -961,29 +856,7 @@ JSONRPC_STATUS CVideoLibrary::RemoveVideo(const CVariant &parameterObject)
   return ACK;
 }
 
-void CVideoLibrary::UpdateResumePoint(const CVariant &parameterObject, CVideoInfoTag &details, CVideoDatabase &videodatabase)
-{
-  if (!parameterObject["resume"].isNull())
-  {
-    int position = (int)parameterObject["resume"]["position"].asInteger();
-    if (position == 0)
-      videodatabase.ClearBookMarksOfFile(details.m_strFileNameAndPath, CBookmark::RESUME);
-    else
-    {
-      CBookmark bookmark;
-      int total = (int)parameterObject["resume"]["total"].asInteger();
-      if (total <= 0 && !videodatabase.GetResumeBookMark(details.m_strFileNameAndPath, bookmark))
-        bookmark.totalTimeInSeconds = details.m_streamDetails.GetVideoDuration();
-      else
-        bookmark.totalTimeInSeconds = total;
-
-      bookmark.timeInSeconds = position;
-      videodatabase.AddBookMarkToFile(details.m_strFileNameAndPath, bookmark, CBookmark::RESUME);
-    }
-  }
-}
-
-void CVideoLibrary::UpdateVideoTag(const CVariant &parameterObject, CVideoInfoTag& details, std::map<std::string, std::string> &artwork, std::set<std::string> &removedArtwork)
+void CVideoLibrary::UpdateVideoTag(const CVariant &parameterObject, CVideoInfoTag& details, std::map<std::string, std::string> &artwork)
 {
   if (ParameterNotNull(parameterObject, "title"))
     details.m_strTitle = parameterObject["title"].asString();
@@ -1014,13 +887,13 @@ void CVideoLibrary::UpdateVideoTag(const CVariant &parameterObject, CVideoInfoTa
   if (ParameterNotNull(parameterObject, "imdbnumber"))
     details.m_strIMDBNumber = parameterObject["imdbnumber"].asString();
   if (ParameterNotNull(parameterObject, "premiered"))
-    SetFromDBDate(parameterObject["premiered"], details.m_premiered);
+    details.m_premiered.SetFromDBDate(parameterObject["premiered"].asString());
   if (ParameterNotNull(parameterObject, "votes"))
     details.m_strVotes = parameterObject["votes"].asString();
   if (ParameterNotNull(parameterObject, "lastplayed"))
-    SetFromDBDateTime(parameterObject["lastplayed"], details.m_lastPlayed);
+    details.m_lastPlayed.SetFromDBDateTime(parameterObject["lastplayed"].asString());
   if (ParameterNotNull(parameterObject, "firstaired"))
-    SetFromDBDateTime(parameterObject["firstaired"], details.m_firstAired);
+    details.m_firstAired.SetFromDBDateTime(parameterObject["firstaired"].asString());
   if (ParameterNotNull(parameterObject, "productioncode"))
     details.m_strProductionCode = parameterObject["productioncode"].asString();
   if (ParameterNotNull(parameterObject, "season"))
@@ -1060,13 +933,8 @@ void CVideoLibrary::UpdateVideoTag(const CVariant &parameterObject, CVideoInfoTa
     CVariant art = parameterObject["art"];
     for (CVariant::const_iterator_map artIt = art.begin_map(); artIt != art.end_map(); artIt++)
     {
-      if (artIt->second.isString() && !artIt->second.asString().empty())
-        artwork[artIt->first] = CTextureUtils::UnwrapImageURL(artIt->second.asString());
-      else if (artIt->second.isNull())
-      {
-        artwork.erase(artIt->first);
-        removedArtwork.insert(artIt->first);
-      }
+      if (!artIt->second.asString().empty())
+        artwork[artIt->first] = CTextureCache::UnwrapImageURL(artIt->second.asString());
     }
   }
 }
