@@ -2,22 +2,19 @@
  * Many concepts and protocol specification in this code are taken
  * from Airplayer. https://github.com/PascalW/Airplayer
  *
- *      http://xbmc.org
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- *  This Program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2.1, or (at your option)
- *  any later version.
- *
- *  This Program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "network/Network.h"
@@ -38,17 +35,8 @@
 #include "ApplicationMessenger.h"
 #include "utils/md5.h"
 #include "utils/Variant.h"
-#include "settings/Settings.h"
-#include "guilib/Key.h"
+#include "guilib/GUIWindowManager.h"
 #include "URL.h"
-#include "cores/IPlayer.h"
-#include "interfaces/AnnouncementManager.h"
-#ifdef HAS_ZEROCONF
-#include "network/Zeroconf.h"
-#endif // HAS_ZEROCONF
-
-using namespace ANNOUNCEMENT;
-using namespace std;
 
 #ifdef TARGET_WINDOWS
 #define close closesocket
@@ -64,7 +52,6 @@ using namespace std;
 #define AIRPLAY_STATUS_NOT_IMPLEMENTED     501
 #define AIRPLAY_STATUS_NO_RESPONSE_NEEDED  1000
 
-CCriticalSection CAirPlayServer::ServerInstanceLock;
 CAirPlayServer *CAirPlayServer::ServerInstance = NULL;
 int CAirPlayServer::m_isPlaying = 0;
 
@@ -132,7 +119,7 @@ const char *eventStrings[] = {"playing", "paused", "loading", "stopped"};
 "<key>features</key>\r\n"\
 "<integer>119</integer>\r\n"\
 "<key>model</key>\r\n"\
-"<string>Xbmc,1</string>\r\n"\
+"<string>AppleTV2,1</string>\r\n"\
 "<key>protovers</key>\r\n"\
 "<string>1.0</string>\r\n"\
 "<key>srcvers</key>\r\n"\
@@ -146,8 +133,6 @@ const char *eventStrings[] = {"playing", "paused", "loading", "stopped"};
 "<dict>\r\n"\
 "<key>category</key>\r\n"\
 "<string>video</string>\r\n"\
-"<key>sessionID</key>\r\n"\
-"<integer>%d</integer>\r\n"\
 "<key>state</key>\r\n"\
 "<string>%s</string>\r\n"\
 "</dict>\r\n"\
@@ -156,40 +141,10 @@ const char *eventStrings[] = {"playing", "paused", "loading", "stopped"};
 #define AUTH_REALM "AirPlay"
 #define AUTH_REQUIRED "WWW-Authenticate: Digest realm=\""  AUTH_REALM  "\", nonce=\"%s\"\r\n"
 
-void CAirPlayServer::Announce(AnnouncementFlag flag, const char *sender, const char *message, const CVariant &data)
-{
-  CSingleLock lock(ServerInstanceLock);
-
-  if ( (flag & Player) && strcmp(sender, "xbmc") == 0 && ServerInstance)
-  {
-    if (strcmp(message, "OnStop") == 0)
-    {
-      bool shouldRestoreVolume = true;
-      if (data.isMember("player") && data["player"].isMember("playerid"))
-        shouldRestoreVolume = (data["player"]["playerid"] != PLAYLIST_PICTURE);
-
-      if (shouldRestoreVolume)
-        restoreVolume();
-
-      ServerInstance->AnnounceToClients(EVENT_STOPPED);
-    }
-    else if (strcmp(message, "OnPlay") == 0)
-    {
-      ServerInstance->AnnounceToClients(EVENT_PLAYING);
-    }
-    else if (strcmp(message, "OnPause") == 0)
-    {
-      ServerInstance->AnnounceToClients(EVENT_PAUSED);
-    }
-  }
-}
-
 bool CAirPlayServer::StartServer(int port, bool nonlocal)
 {
   StopServer(true);
 
-  CSingleLock lock(ServerInstanceLock);
-
   ServerInstance = new CAirPlayServer(port, nonlocal);
   if (ServerInstance->Initialize())
   {
@@ -202,7 +157,6 @@ bool CAirPlayServer::StartServer(int port, bool nonlocal)
 
 bool CAirPlayServer::SetCredentials(bool usePassword, const CStdString& password)
 {
-  CSingleLock lock(ServerInstanceLock);
   bool ret = false;
 
   if (ServerInstance)
@@ -221,7 +175,6 @@ bool CAirPlayServer::SetInternalCredentials(bool usePassword, const CStdString&
 
 void CAirPlayServer::StopServer(bool bWait)
 {
-  CSingleLock lock(ServerInstanceLock);
   if (ServerInstance)
   {
     ServerInstance->StopThread(bWait);
@@ -233,83 +186,17 @@ void CAirPlayServer::StopServer(bool bWait)
   }
 }
 
-bool CAirPlayServer::IsRunning()
-{
-  if (ServerInstance == NULL)
-    return false;
-
-  return ((CThread*)ServerInstance)->IsRunning();
-}
-
-void CAirPlayServer::AnnounceToClients(int state)
-{
-  CSingleLock lock (m_connectionLock);
-  
-  std::vector<CTCPClient>::iterator it;
-  for (it = m_connections.begin(); it != m_connections.end(); it++)
-  {
-    CStdString reverseHeader;
-    CStdString reverseBody;
-    CStdString response;
-    int reverseSocket = INVALID_SOCKET;
-    it->ComposeReverseEvent(reverseHeader, reverseBody, state);
-  
-    // Send event status per reverse http socket (play, loading, paused)
-    // if we have a reverse header and a reverse socket
-    if (reverseHeader.size() > 0 && m_reverseSockets.find(it->m_sessionId) != m_reverseSockets.end())
-    {
-      //search the reverse socket to this sessionid
-      response = StringUtils::Format("POST /event HTTP/1.1\r\n");
-      reverseSocket = m_reverseSockets[it->m_sessionId]; //that is our reverse socket
-      response += reverseHeader;
-    }
-    response += "\r\n";
-  
-    if (reverseBody.size() > 0)
-    {
-      response += reverseBody;
-    }
-  
-    // don't send it to the connection object
-    // the reverse socket itself belongs to
-    if (reverseSocket != INVALID_SOCKET && reverseSocket != it->m_socket)
-    {
-      send(reverseSocket, response.c_str(), response.size(), 0);//send the event status on the eventSocket
-    }
-  }
-}
-
 CAirPlayServer::CAirPlayServer(int port, bool nonlocal) : CThread("AirPlayServer")
 {
   m_port = port;
   m_nonlocal = nonlocal;
   m_ServerSocket = INVALID_SOCKET;
   m_usePassword = false;
-  m_origVolume = -1;
-  CAnnouncementManager::Get().AddAnnouncer(this);
-}
-
-CAirPlayServer::~CAirPlayServer()
-{
-  CAnnouncementManager::Get().RemoveAnnouncer(this);
-}
-
-void handleZeroconfAnnouncement()
-{
-#if defined(HAS_ZEROCONF)
-  static XbmcThreads::EndTime timeout(10000);
-  if(timeout.IsTimePast())
-  {
-    CZeroconf::GetInstance()->ForceReAnnounceService("servers.airplay");
-    timeout.Set(10000);
-  }
-#endif
 }
 
 void CAirPlayServer::Process()
 {
   m_bStop = false;
-  static int sessionCounter = 0;
 
   while (!m_bStop)
   {
@@ -352,7 +239,6 @@ void CAirPlayServer::Process()
           }
           if (nread <= 0)
           {
-            CSingleLock lock (m_connectionLock);
             CLog::Log(LOGINFO, "AIRPLAY Server: Disconnection detected");
             m_connections[i].Disconnect();
             m_connections.erase(m_connections.begin() + i);
@@ -364,34 +250,17 @@ void CAirPlayServer::Process()
       {
         CLog::Log(LOGDEBUG, "AIRPLAY Server: New connection detected");
         CTCPClient newconnection;
-        newconnection.m_socket = accept(m_ServerSocket, (struct sockaddr*) &newconnection.m_cliaddr, &newconnection.m_addrlen);
-        sessionCounter++;
-        newconnection.m_sessionCounter = sessionCounter;
+        newconnection.m_socket = accept(m_ServerSocket, &newconnection.m_cliaddr, &newconnection.m_addrlen);
 
         if (newconnection.m_socket == INVALID_SOCKET)
-        {
-          CLog::Log(LOGERROR, "AIRPLAY Server: Accept of new connection failed: %d", errno);
-          if (EBADF == errno)
-          {
-            Sleep(1000);
-            Initialize();
-            break;
-          }
-        }
+          CLog::Log(LOGERROR, "AIRPLAY Server: Accept of new connection failed");
         else
         {
-          CSingleLock lock (m_connectionLock);
           CLog::Log(LOGINFO, "AIRPLAY Server: New connection added");
           m_connections.push_back(newconnection);
         }
       }
     }
-    
-    // by reannouncing the zeroconf service
-    // we fix issues where xbmc is detected
-    // as audio-only target on devices with
-    // ios7 and later
-    handleZeroconfAnnouncement();    
   }
 
   Deinitialize();
@@ -400,17 +269,47 @@ void CAirPlayServer::Process()
 bool CAirPlayServer::Initialize()
 {
   Deinitialize();
-  
-  if ((m_ServerSocket = CreateTCPServerSocket(m_port, !m_nonlocal, 10, "AIRPLAY")) == INVALID_SOCKET)
+
+  struct sockaddr_in myaddr;
+  memset(&myaddr, 0, sizeof(myaddr));
+
+  myaddr.sin_family = AF_INET;
+  myaddr.sin_port = htons(m_port);
+
+  if (m_nonlocal)
+    myaddr.sin_addr.s_addr = INADDR_ANY;
+  else
+    inet_pton(AF_INET, "127.0.0.1", &myaddr.sin_addr.s_addr);
+
+  m_ServerSocket = socket(PF_INET, SOCK_STREAM, 0);
+
+  if (m_ServerSocket == INVALID_SOCKET)
+  {
+
+    CLog::Log(LOGERROR, "AIRPLAY Server: Failed to create serversocket");
     return false;
-  
+  }
+
+  if (bind(m_ServerSocket, (struct sockaddr*)&myaddr, sizeof myaddr) < 0)
+  {
+    CLog::Log(LOGERROR, "AIRPLAY Server: Failed to bind serversocket");
+    close(m_ServerSocket);
+    return false;
+  }
+
+  if (listen(m_ServerSocket, 10) < 0)
+  {
+    CLog::Log(LOGERROR, "AIRPLAY Server: Failed to set listen");
+    close(m_ServerSocket);
+    return false;
+  }
+
   CLog::Log(LOGINFO, "AIRPLAY Server: Successfully initialized");
   return true;
 }
 
 void CAirPlayServer::Deinitialize()
 {
-  CSingleLock lock (m_connectionLock);
   for (unsigned int i = 0; i < m_connections.size(); i++)
     m_connections[i].Disconnect();
 
@@ -430,7 +329,7 @@ CAirPlayServer::CTCPClient::CTCPClient()
   m_socket = INVALID_SOCKET;
   m_httpParser = new HttpParser();
 
-  m_addrlen = sizeof(struct sockaddr_storage);
+  m_addrlen = sizeof(struct sockaddr);
   m_pLibPlist = new DllLibPlist();
 
   m_bAuthenticated = false;
@@ -473,9 +372,11 @@ void CAirPlayServer::CTCPClient::PushBuffer(CAirPlayServer *host, const char *bu
     // Parse the request
     CStdString responseHeader;
     CStdString responseBody;
-    int status = ProcessRequest(responseHeader, responseBody);
-    sessionId = m_sessionId;
+    CStdString reverseHeader;
+    CStdString reverseBody;
+    int status = ProcessRequest(responseHeader, responseBody, reverseHeader, reverseBody, sessionId);
     CStdString statusMsg = "OK";
+    int reverseSocket = INVALID_SOCKET;
 
     switch(status)
     {
@@ -502,7 +403,7 @@ void CAirPlayServer::CTCPClient::PushBuffer(CAirPlayServer *host, const char *bu
     const time_t ltime = time(NULL);
     char *date = asctime(gmtime(&ltime)); //Fri, 17 Dec 2010 11:18:01 GMT;
     date[strlen(date) - 1] = '\0'; // remove \n
-    response = StringUtils::Format("HTTP/1.1 %d %s\nDate: %s\r\n", status, statusMsg.c_str(), date);
+    response.Format("HTTP/1.1 %d %s\nDate: %s\r\n", status, statusMsg.c_str(), date);
     if (responseHeader.size() > 0)
     {
       response += responseHeader;
@@ -510,7 +411,7 @@ void CAirPlayServer::CTCPClient::PushBuffer(CAirPlayServer *host, const char *bu
 
     if (responseBody.size() > 0)
     {
-      response = StringUtils::Format("%sContent-Length: %d\r\n", response.c_str(), responseBody.size());
+      response.Format("%sContent-Length: %d\r\n", response.c_str(), responseBody.size());
     }
     response += "\r\n";
 
@@ -525,6 +426,28 @@ void CAirPlayServer::CTCPClient::PushBuffer(CAirPlayServer *host, const char *bu
     {
       send(m_socket, response.c_str(), response.size(), 0);
     }
+
+    // Send event status per reverse http socket (play, loading, paused)
+    // if we have a reverse header and a reverse socket
+    if (reverseHeader.size() > 0 && reverseSockets.find(sessionId) != reverseSockets.end())
+    {
+      //search the reverse socket to this sessionid
+      response.Format("POST /event HTTP/1.1\r\n");
+      reverseSocket = reverseSockets[sessionId]; //that is our reverse socket
+      response += reverseHeader;
+    }
+    response += "\r\n";
+
+    if (reverseBody.size() > 0)
+    {
+      response += reverseBody;
+    }
+
+    if (reverseSocket != INVALID_SOCKET)
+    {
+      send(reverseSocket, response.c_str(), response.size(), 0);//send the event status on the eventSocket
+    }
+
     // We need a new parser...
     delete m_httpParser;
     m_httpParser = new HttpParser;
@@ -552,12 +475,12 @@ void CAirPlayServer::CTCPClient::Copy(const CTCPClient& client)
   m_httpParser        = client.m_httpParser;
   m_authNonce         = client.m_authNonce;
   m_bAuthenticated    = client.m_bAuthenticated;
-  m_sessionCounter    = client.m_sessionCounter;
 }
 
 
 void CAirPlayServer::CTCPClient::ComposeReverseEvent( CStdString& reverseHeader,
                                                       CStdString& reverseBody,
+                                                      CStdString sessionId,
                                                       int state)
 {
 
@@ -569,23 +492,24 @@ void CAirPlayServer::CTCPClient::ComposeReverseEvent( CStdString& reverseHeader,
       case EVENT_LOADING:
       case EVENT_PAUSED:
       case EVENT_STOPPED:      
-        reverseBody = StringUtils::Format(EVENT_INFO, m_sessionCounter, eventStrings[state]);
+        reverseBody.Format(EVENT_INFO, eventStrings[state]);
         CLog::Log(LOGDEBUG, "AIRPLAY: sending event: %s", eventStrings[state]);
         break;
     }
     reverseHeader = "Content-Type: text/x-apple-plist+xml\r\n";
-    reverseHeader = StringUtils::Format("%sContent-Length: %d\r\n",reverseHeader.c_str(), reverseBody.size());
-    reverseHeader = StringUtils::Format("%sx-apple-session-id: %s\r\n",reverseHeader.c_str(), m_sessionId.c_str());
+    reverseHeader.Format("%sContent-Length: %d",reverseHeader.c_str(),reverseBody.size());
+    reverseHeader.Format("%sx-apple-session-id: %s\r\n",reverseHeader.c_str(),sessionId.c_str());
     m_lastEvent = state;
   }
 }
 
 void CAirPlayServer::CTCPClient::ComposeAuthRequestAnswer(CStdString& responseHeader, CStdString& responseBody)
 {
+  CStdString randomStr;
   int16_t random=rand();
-  CStdString randomStr = StringUtils::Format("%i", random);
+  randomStr.Format("%i", random);
   m_authNonce=XBMC::XBMC_MD5::GetMD5(randomStr);
-  responseHeader = StringUtils::Format(AUTH_REQUIRED, m_authNonce.c_str());
+  responseHeader.Format(AUTH_REQUIRED,m_authNonce);
   responseBody.clear();
 }
 
@@ -604,26 +528,27 @@ CStdString calcResponse(const CStdString& username,
 
   HA1 = XBMC::XBMC_MD5::GetMD5(username + ":" + realm + ":" + password);
   HA2 = XBMC::XBMC_MD5::GetMD5(method + ":" + digestUri);
-  StringUtils::ToLower(HA1);
-  StringUtils::ToLower(HA2);
-  response = XBMC::XBMC_MD5::GetMD5(HA1 + ":" + nonce + ":" + HA2);
-  StringUtils::ToLower(response);
-  return response;
+  response = XBMC::XBMC_MD5::GetMD5(HA1.ToLower() + ":" + nonce + ":" + HA2.ToLower());
+  return response.ToLower();
 }
 
 //helper function
 //from a string field1="value1", field2="value2" it parses the value to a field
 CStdString getFieldFromString(const CStdString &str, const char* field)
 {
-  vector<string> tmpAr1 = StringUtils::Split(str, ",");
-  for(vector<string>::const_iterator i = tmpAr1.begin(); i != tmpAr1.end(); ++i)
+  CStdString tmpStr;
+  CStdStringArray tmpAr1;
+  CStdStringArray tmpAr2;
+
+  StringUtils::SplitString(str, ",", tmpAr1);
+
+  for(unsigned int i = 0;i<tmpAr1.size();i++)
   {
-    if (i->find(field) != std::string::npos)
+    if (tmpAr1[i].Find(field) != -1)
     {
-      vector<string> tmpAr2 = StringUtils::Split(*i, "=");
-      if (tmpAr2.size() == 2)
+      if (StringUtils::SplitString(tmpAr1[i], "=", tmpAr2) == 2)
       {
-        StringUtils::Replace(tmpAr2[1], "\"", "");//remove quotes
+        tmpAr2[1].Remove('\"');//remove quotes
         return tmpAr2[1];
       }
     }
@@ -696,50 +621,31 @@ bool CAirPlayServer::CTCPClient::checkAuthorization(const CStdString& authStr,
   return m_bAuthenticated;
 }
 
-void CAirPlayServer::backupVolume()
-{
-  CSingleLock lock(ServerInstanceLock);
-  
-  if (ServerInstance && ServerInstance->m_origVolume == -1)
-    ServerInstance->m_origVolume = (int)g_application.GetVolume();
-}
-
-void CAirPlayServer::restoreVolume()
-{
-  CSingleLock lock(ServerInstanceLock);
-
-  if (ServerInstance && ServerInstance->m_origVolume != -1 && CSettings::Get().GetBool("services.airplayvolumecontrol"))
-  {
-    g_application.SetVolume((float)ServerInstance->m_origVolume);
-    ServerInstance->m_origVolume = -1;
-  }
-}
-
 int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
-                                                CStdString& responseBody)
+                                                CStdString& responseBody,
+                                                CStdString& reverseHeader,
+                                                CStdString& reverseBody,
+                                                CStdString& sessionId)
 {
   CStdString method = m_httpParser->getMethod();
   CStdString uri = m_httpParser->getUri();
   CStdString queryString = m_httpParser->getQueryString();
   CStdString body = m_httpParser->getBody();
   CStdString contentType = m_httpParser->getValue("content-type");
-  m_sessionId = m_httpParser->getValue("x-apple-session-id");
+  sessionId = m_httpParser->getValue("x-apple-session-id");
   CStdString authorization = m_httpParser->getValue("authorization");
   int status = AIRPLAY_STATUS_OK;
   bool needAuth = false;
-  
-  if (m_sessionId.empty())
-    m_sessionId = "00000000-0000-0000-0000-000000000000";
 
   if (ServerInstance->m_usePassword && !m_bAuthenticated)
   {
     needAuth = true;
   }
 
-  size_t startQs = uri.find('?');
-  if (startQs != std::string::npos)
+  int startQs = uri.Find('?');
+  if (startQs != -1)
   {
-    uri.erase(startQs);
+    uri = uri.Left(startQs);
   }
 
   // This is the socket which will be used for reverse HTTP
@@ -767,16 +673,18 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
       }
       else if (rate == 0)
       {
-        if (g_application.m_pPlayer->IsPlaying() && !g_application.m_pPlayer->IsPaused())
+        if (g_application.m_pPlayer && g_application.m_pPlayer->IsPlaying() && !g_application.m_pPlayer->IsPaused())
         {
           CApplicationMessenger::Get().MediaPause();
+          ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_PAUSED);
         }
       }
       else
       {
-        if (g_application.m_pPlayer->IsPausedPlayback())
+        if (g_application.m_pPlayer && g_application.m_pPlayer->IsPlaying() && g_application.m_pPlayer->IsPaused())
         {
           CApplicationMessenger::Get().MediaPause();
+          ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_PLAYING);
         }
       }
   }
@@ -798,11 +706,10 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
       }
       else if (volume >= 0 && volume <= 1)
       {
-        float oldVolume = g_application.GetVolume();
+        int oldVolume = g_application.GetVolume();
         volume *= 100;
-        if(oldVolume != volume && CSettings::Get().GetBool("services.airplayvolumecontrol"))
+        if(oldVolume != (int)volume)
         {
-          backupVolume();
           g_application.SetVolume(volume);          
           CApplicationMessenger::Get().ShowVolumeBar(oldVolume < volume);
         }
@@ -876,37 +783,33 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
     {
       CAirPlayServer::m_isPlaying++;        
       // Get URL to play
-      std::string contentLocation = "Content-Location: ";
-      size_t start = body.find(contentLocation);
-      if (start == std::string::npos)
+      int start = body.Find("Content-Location: ");
+      if (start == -1)
         return AIRPLAY_STATUS_NOT_IMPLEMENTED;
-      start += contentLocation.size();
-      int end = body.find('\n', start);
-      location = body.substr(start, end - start);
+      start += strlen("Content-Location: ");
+      int end = body.Find('\n', start);
+      location = body.Mid(start, end - start);
 
-      std::string startPosition = "Start-Position: ";
-      start = body.find(startPosition);
-      if (start != std::string::npos)
+      start = body.Find("Start-Position");
+      if (start != -1)
       {
-        start += startPosition.size();
-        int end = body.find('\n', start);
-        std::string positionStr = body.substr(start, end - start);
+        start += strlen("Start-Position: ");
+        int end = body.Find('\n', start);
+        CStdString positionStr = body.Mid(start, end - start);
         position = (float)atof(positionStr.c_str());
       }
     }
 
     if (status != AIRPLAY_STATUS_NEED_AUTH)
     {
-      CStdString userAgent(CURL::Encode("AppleCoreMedia/1.0.0.8F455 (AppleTV; U; CPU OS 4_3 like Mac OS X; de_de)"));
+      CStdString userAgent="AppleCoreMedia/1.0.0.8F455 (AppleTV; U; CPU OS 4_3 like Mac OS X; de_de)";
+      CURL::Encode(userAgent);
       location += "|User-Agent=" + userAgent;
 
       CFileItem fileToPlay(location, false);
       fileToPlay.SetProperty("StartPercent", position*100.0f);
-      ServerInstance->AnnounceToClients(EVENT_LOADING);
-      // froce to internal dvdplayer cause it is the only
-      // one who will work well with airplay
-      g_application.m_eForcedNextPlayer = EPC_DVDPLAYER;
       CApplicationMessenger::Get().MediaPlay(fileToPlay);
+      ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_PLAYING);
     }
   }
 
@@ -922,10 +825,10 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
     {
       CLog::Log(LOGDEBUG, "AIRPLAY: got GET request %s", uri.c_str());
       
-      if (g_application.m_pPlayer->GetTotalTime())
+      if (g_application.m_pPlayer && g_application.m_pPlayer->GetTotalTime())
       {
         float position = ((float) g_application.m_pPlayer->GetTime()) / 1000;
-        responseBody = StringUtils::Format("duration: %.6f\r\nposition: %.6f\r\n", (float)g_application.m_pPlayer->GetTotalTime() / 1000, position);
+        responseBody.Format("duration: %d\r\nposition: %f", g_application.m_pPlayer->GetTotalTime() / 1000, position);
       }
       else 
       {
@@ -936,7 +839,7 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
     {
       const char* found = strstr(queryString.c_str(), "position=");
       
-      if (found && g_application.m_pPlayer->HasPlayer())
+      if (found && g_application.m_pPlayer)
       {
         int64_t position = (int64_t) (atof(found + strlen("position=")) * 1000.0);
         g_application.m_pPlayer->SeekTime(position);
@@ -962,8 +865,9 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
       }
       else //if we are not playing and get the stop request - we just wanna stop picture streaming
       {
-        CApplicationMessenger::Get().SendAction(ACTION_PREVIOUS_MENU);
+        g_windowManager.PreviousWindow();
       }
+      ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_STOPPED);
     }
   }
 
@@ -1019,35 +923,44 @@ int CAirPlayServer::CTCPClient::ProcessRequest( CStdString& responseHeader,
     {
       status = AIRPLAY_STATUS_NEED_AUTH;
     }
-    else if (g_application.m_pPlayer->HasPlayer())
+    else if (g_application.m_pPlayer)
     {
       if (g_application.m_pPlayer->GetTotalTime())
       {
         position = ((float) g_application.m_pPlayer->GetTime()) / 1000;
         duration = ((float) g_application.m_pPlayer->GetTotalTime()) / 1000;
-        playing = !g_application.m_pPlayer->IsPaused();
+        playing = g_application.m_pPlayer ? !g_application.m_pPlayer->IsPaused() : false;
         cachePosition = position + (duration * g_application.m_pPlayer->GetCachePercentage() / 100.0f);
       }
 
-      responseBody = StringUtils::Format(PLAYBACK_INFO, duration, cachePosition, position, (playing ? 1 : 0), duration);
+      responseBody.Format(PLAYBACK_INFO, duration, cachePosition, position, (playing ? 1 : 0), duration);
       responseHeader = "Content-Type: text/x-apple-plist+xml\r\n";
 
       if (g_application.m_pPlayer->IsCaching())
       {
-        CAirPlayServer::ServerInstance->AnnounceToClients(EVENT_LOADING);
+        ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_LOADING);
+      }
+      else if (playing)
+      {
+        ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_PLAYING);
+      }
+      else
+      {
+        ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_PAUSED);
       }
     }
     else
     {
-      responseBody = StringUtils::Format(PLAYBACK_INFO_NOT_READY, duration, cachePosition, position, (playing ? 1 : 0), duration);
+      responseBody.Format(PLAYBACK_INFO_NOT_READY, duration, cachePosition, position, (playing ? 1 : 0), duration);
       responseHeader = "Content-Type: text/x-apple-plist+xml\r\n";     
+      ComposeReverseEvent(reverseHeader, reverseBody, sessionId, EVENT_STOPPED);
     }
   }
 
   else if (uri == "/server-info")
   {
     CLog::Log(LOGDEBUG, "AIRPLAY: got request %s", uri.c_str());
-    responseBody = StringUtils::Format(SERVER_INFO, g_application.getNetwork().GetFirstConnectedInterface()->GetMacAddress().c_str());
+    responseBody.Format(SERVER_INFO, g_application.getNetwork().GetFirstConnectedInterface()->GetMacAddress());
     responseHeader = "Content-Type: text/x-apple-plist+xml\r\n";
   }
 
