@@ -1,28 +1,8 @@
-/*
- *      Copyright (C) 2012-2013 Team XBMC
- *      http://xbmc.org
- *
- *  This Program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- *  This Program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, see
- *  <http://www.gnu.org/licenses/>.
- *
- */
-#include <Platinum/Source/Platinum/Platinum.h>
-
 #include "UPnPServer.h"
 #include "UPnPInternal.h"
 #include "Application.h"
-#include "view/GUIViewState.h"
+#include "GUIViewState.h"
+#include "Platinum.h"
 #include "video/VideoThumbLoader.h"
 #include "music/Artist.h"
 #include "music/MusicThumbLoader.h"
@@ -31,22 +11,15 @@
 #include "filesystem/MusicDatabaseDirectory.h"
 #include "filesystem/SpecialProtocol.h"
 #include "filesystem/VideoDatabaseDirectory.h"
-#include "guilib/WindowIDs.h"
+#include "guilib/Key.h"
 #include "music/tags/MusicInfoTag.h"
-#include "settings/AdvancedSettings.h"
-#include "settings/Settings.h"
+#include "settings/GUISettings.h"
 #include "utils/log.h"
 #include "utils/md5.h"
 #include "utils/StringUtils.h"
 #include "utils/URIUtils.h"
-#include "Util.h"
 #include "music/MusicDatabase.h"
 #include "video/VideoDatabase.h"
-#include "guilib/GUIWindowManager.h"
-#include "xbmc/GUIUserMessages.h"
-#include "utils/FileUtils.h"
-
-NPT_SET_LOCAL_LOGGER("xbmc.upnp.server")
 
 using namespace std;
 using namespace ANNOUNCEMENT;
@@ -57,12 +30,12 @@ namespace UPNP
 
 NPT_UInt32 CUPnPServer::m_MaxReturnedItems = 0;
 
-const char* audio_containers[] = { "musicdb://genres/", "musicdb://artists/", "musicdb://albums/",
-                                   "musicdb://songs/", "musicdb://recentlyaddedalbums/", "musicdb://years/",
-                                   "musicdb://singles/" };
+const char* audio_containers[] = { "musicdb://1/", "musicdb://2/", "musicdb://3/",
+                                   "musicdb://4/", "musicdb://6/", "musicdb://9/",
+                                   "musicdb://10/" };
 
-const char* video_containers[] = { "library://video/movies/titles.xml/", "library://video/tvshows/titles.xml/",
-                                   "videodb://recentlyaddedmovies/", "videodb://recentlyaddedepisodes/"  };
+const char* video_containers[] = { "videodb://1/2/", "videodb://2/2/", "videodb://4/",
+                                   "videodb://5/"  };
 
 /*----------------------------------------------------------------------
 |   CUPnPServer::CUPnPServer
@@ -76,7 +49,7 @@ CUPnPServer::CUPnPServer(const char* friendly_name, const char* uuid /*= NULL*/,
 
 CUPnPServer::~CUPnPServer()
 {
-    ANNOUNCEMENT::CAnnouncementManager::Get().RemoveAnnouncer(this);
+    ANNOUNCEMENT::CAnnouncementManager::RemoveAnnouncer(this);
 }
 
 /*----------------------------------------------------------------------
@@ -102,10 +75,7 @@ CUPnPServer::SetupServices()
     PLT_Service* service = NULL;
     NPT_Result result = FindServiceById("urn:upnp-org:serviceId:ContentDirectory", service);
     if (service)
-    {
-      service->SetStateVariable("SearchCapabilities", "upnp:class");
-      service->SetStateVariable("SortCapabilities", "res@duration,res@size,res@bitrate,dc:date,dc:title,dc:size,upnp:album,upnp:artist,upnp:albumArtist,upnp:episodeNumber,upnp:genre,upnp:originalTrackNumber,upnp:rating,upnp:episodeCount,upnp:episodeSeason,xbmc:rating,xbmc:dateadded,xbmc:votes");
-    }
+      service->SetStateVariable("SortCapabilities", "res@duration,res@size,res@bitrate,dc:date,dc:title,dc:size,upnp:album,upnp:artist,upnp:albumArtist,upnp:episodeNumber,upnp:genre,upnp:originalTrackNumber,upnp:rating");
 
     m_scanning = true;
     OnScanCompleted(AudioLibrary);
@@ -113,7 +83,7 @@ CUPnPServer::SetupServices()
     OnScanCompleted(VideoLibrary);
 
     // now safe to start passing on new notifications
-    ANNOUNCEMENT::CAnnouncementManager::Get().AddAnnouncer(this);
+    ANNOUNCEMENT::CAnnouncementManager::AddAnnouncer(this);
 
     return result;
 }
@@ -163,7 +133,7 @@ CUPnPServer::PropagateUpdates()
     string buffer;
     map<string,pair<bool, unsigned long> >::iterator itr;
 
-    if (m_scanning || !CSettings::Get().GetBool("services.upnpannounce"))
+    if (m_scanning || !g_guiSettings.GetBool("services.upnpannounce"))
         return;
 
     NPT_CHECK_LABEL(FindServiceById("urn:upnp-org:serviceId:ContentDirectory", service), failed);
@@ -205,19 +175,10 @@ CUPnPServer::SetupIcons()
 {
     NPT_String file_root = CSpecialProtocol::TranslatePath("special://xbmc/media/").c_str();
     AddIcon(
-        PLT_DeviceIcon("image/png", 256, 256, 8, "/icon256x256.png"),
+        PLT_DeviceIcon("image/png", 256, 256, 24, "/icon-flat-256x256.png"),
         file_root);
     AddIcon(
-        PLT_DeviceIcon("image/png", 120, 120, 8, "/icon120x120.png"),
-        file_root);
-    AddIcon(
-        PLT_DeviceIcon("image/png", 48, 48, 8, "/icon48x48.png"),
-        file_root);
-    AddIcon(
-        PLT_DeviceIcon("image/png", 32, 32, 8, "/icon32x32.png"),
-        file_root);
-    AddIcon(
-        PLT_DeviceIcon("image/png", 16, 16, 8, "/icon16x16.png"),
+        PLT_DeviceIcon("image/png", 120, 120, 24, "/icon-flat-120x120.png"),
         file_root);
     return NPT_SUCCESS;
 }
@@ -240,7 +201,7 @@ NPT_String CUPnPServer::BuildSafeResourceUri(const NPT_HttpUrl &rooturi,
     else
       filename = URIUtils::GetFileName(file_path);
 
-    filename = CURL::Encode(filename);
+    CURL::Encode(filename);
     md5state.append(file_path);
     md5state.getDigest(md5);
     md5 += "/" + filename;
@@ -307,23 +268,23 @@ CUPnPServer::Build(CFileItemPtr                  item,
 
                     if (params.GetSongId() >= 0 ) {
                         CSong song;
-                        if (db.GetSong(params.GetSongId(), song))
+                        if (db.GetSongById(params.GetSongId(), song))
                             item->GetMusicInfoTag()->SetSong(song);
                     }
                     else if (params.GetAlbumId() >= 0 ) {
                         CAlbum album;
-                        if (db.GetAlbum(params.GetAlbumId(), album, false))
+                        if (db.GetAlbumInfo(params.GetAlbumId(), album, NULL))
                             item->GetMusicInfoTag()->SetAlbum(album);
                     }
                     else if (params.GetArtistId() >= 0 ) {
                         CArtist artist;
-                        if (db.GetArtist(params.GetArtistId(), artist, false))
+                        if (db.GetArtistInfo(params.GetArtistId(), artist, false))
                             item->GetMusicInfoTag()->SetArtist(artist);
                     }
                 }
 
 
-                if (item->GetLabel().empty()) {
+                if (item->GetLabel().IsEmpty()) {
                     /* if no label try to grab it from node type */
                     CStdString label;
                     if (CMusicDatabaseDirectory::GetLabel((const char*)path, label)) {
@@ -333,7 +294,7 @@ CUPnPServer::Build(CFileItemPtr                  item,
                 }
             }
         } else if (file_path.StartsWith("library://") || file_path.StartsWith("videodb://")) {
-            if (path == "library://video/" ) {
+            if (path == "library://video" ) {
                 item->SetLabel("Video Library");
                 item->SetLabelPreformated(true);
             } else {
@@ -346,15 +307,13 @@ CUPnPServer::Build(CFileItemPtr                  item,
 
                     if (params.GetMovieId() >= 0 )
                         db.GetMovieInfo((const char*)path, *item->GetVideoInfoTag(), params.GetMovieId());
-                    else if (params.GetMVideoId() >= 0 )
-                        db.GetMusicVideoInfo((const char*)path, *item->GetVideoInfoTag(), params.GetMVideoId());
                     else if (params.GetEpisodeId() >= 0 )
                         db.GetEpisodeInfo((const char*)path, *item->GetVideoInfoTag(), params.GetEpisodeId());
                     else if (params.GetTvShowId() >= 0 )
                         db.GetTvShowInfo((const char*)path, *item->GetVideoInfoTag(), params.GetTvShowId());
                 }
 
-                if (item->GetVideoInfoTag()->m_type == MediaTypeTvShow || item->GetVideoInfoTag()->m_type == MediaTypeSeason) {
+                if (item->GetVideoInfoTag()->m_type == "tvshow" || item->GetVideoInfoTag()->m_type == "season") {
                     // for tvshows and seasons, iEpisode and playCount are
                     // invalid
                     item->GetVideoInfoTag()->m_iEpisode = (int)item->GetProperty("totalepisodes").asInteger();
@@ -362,13 +321,13 @@ CUPnPServer::Build(CFileItemPtr                  item,
                 }
 
                 // try to grab title from tag
-                if (item->HasVideoInfoTag() && !item->GetVideoInfoTag()->m_strTitle.empty()) {
+                if (item->HasVideoInfoTag() && !item->GetVideoInfoTag()->m_strTitle.IsEmpty()) {
                     item->SetLabel(item->GetVideoInfoTag()->m_strTitle);
                     item->SetLabelPreformated(true);
                 }
 
                 // try to grab it from the folder
-                if (item->GetLabel().empty()) {
+                if (item->GetLabel().IsEmpty()) {
                     CStdString label;
                     if (CVideoDatabaseDirectory::GetLabel((const char*)path, label)) {
                         item->SetLabel(label);
@@ -443,38 +402,37 @@ CUPnPServer::Announce(AnnouncementFlag flag, const char *sender, const char *mes
         // we always update 'recently added' nodes along with the specific container,
         // as we don't differentiate 'updates' from 'adds' in RPC interface
         if (flag == VideoLibrary) {
-            if(item_type == MediaTypeEpisode) {
+            if(item_type == "episode") {
                 CVideoDatabase db;
                 if (!db.Open()) return;
                 int show_id = db.GetTvShowForEpisode(item_id);
                 int season_id = db.GetSeasonForEpisode(item_id);
-                UpdateContainer(StringUtils::Format("videodb://tvshows/titles/%d/", show_id));
-                UpdateContainer(StringUtils::Format("videodb://tvshows/titles/%d/%d/?tvshowid=%d", show_id, season_id, show_id));
-                UpdateContainer("videodb://recentlyaddedepisodes/");
+                UpdateContainer(StringUtils::Format("videodb://2/2/%d/", show_id));
+                UpdateContainer(StringUtils::Format("videodb://2/2/%d/%d/?tvshowid=%d", show_id, season_id, show_id));
+                UpdateContainer("videodb://5/");
             }
-            else if(item_type == MediaTypeTvShow) {
-                UpdateContainer("library://video/tvshows/titles.xml/");
-                UpdateContainer("videodb://recentlyaddedepisodes/");
+            else if(item_type == "tvshow") {
+                UpdateContainer("videodb://2/2/");
+                UpdateContainer("videodb://5/");
             }
-            else if(item_type == MediaTypeMovie) {
-                UpdateContainer("library://video/movies/titles.xml/");
-                UpdateContainer("videodb://recentlyaddedmovies/");
+            else if(item_type == "movie") {
+                UpdateContainer("videodb://1/2/");
+                UpdateContainer("videodb://4/");
             }
-            else if(item_type == MediaTypeMusicVideo) {
-                UpdateContainer("library://video/musicvideos/titles.xml/");
-                UpdateContainer("videodb://recentlyaddedmusicvideos/");
+            else if(item_type == "musicvideo") {
+                UpdateContainer("videodb://4/");
             }
         }
-        else if (flag == AudioLibrary && item_type == MediaTypeSong) {
+        else if (flag == AudioLibrary && item_type == "song") {
             // we also update the 'songs' container is maybe a performance drop too
             // high? would need to check if slow clients even cache at all anyway
             CMusicDatabase db;
             CAlbum album;
             if (!db.Open()) return;
             if (db.GetAlbumFromSong(item_id, album)) {
-                UpdateContainer(StringUtils::Format("musicdb://albums/%ld", album.idAlbum));
-                UpdateContainer("musicdb://songs/");
-                UpdateContainer("musicdb://recentlyaddedalbums/");
+                UpdateContainer(StringUtils::Format("musicdb://3/%ld", album.idAlbum));
+                UpdateContainer("musicdb://4/");
+                UpdateContainer("musicdb://6/");
             }
         }
     }
@@ -489,32 +447,24 @@ static NPT_String TranslateWMPObjectId(NPT_String id)
         id = "virtualpath://upnproot/";
     } else if (id == "15") {
         // Xbox 360 asking for videos
-        id = "library://video/";
+        id = "library://video";
     } else if (id == "16") {
         // Xbox 360 asking for photos
     } else if (id == "107") {
         // Sonos uses 107 for artists root container id
-        id = "musicdb://artists/";
+        id = "musicdb://2/";
     } else if (id == "7") {
         // Sonos uses 7 for albums root container id
-        id = "musicdb://albums/";
+        id = "musicdb://3/";
     } else if (id == "4") {
         // Sonos uses 4 for tracks root container id
-        id = "musicdb://songs/";
+        id = "musicdb://4/";
     }
 
     CLog::Log(LOGDEBUG, "UPnP Translated id to '%s'", (const char*)id);
     return id;
 }
 
-NPT_Result
-ObjectIDValidate(const NPT_String& id)
-{
-    if (CFileUtils::RemoteAccessAllowed(id.GetChars()))
-        return NPT_SUCCESS;
-    return NPT_ERROR_NO_SUCH_FILE;
-}
-
 /*----------------------------------------------------------------------
 |   CUPnPServer::OnBrowseMetadata
 +---------------------------------------------------------------------*/
@@ -534,16 +484,12 @@ CUPnPServer::OnBrowseMetadata(PLT_ActionReference&          action,
     NPT_String                     didl;
     NPT_Reference<PLT_MediaObject> object;
     NPT_String                     id = TranslateWMPObjectId(object_id);
+    vector<CStdString>             paths;
     CFileItemPtr                   item;
     NPT_Reference<CThumbLoader>    thumb_loader;
 
     CLog::Log(LOGINFO, "Received UPnP Browse Metadata request for object '%s'", (const char*)object_id);
 
-    if(NPT_FAILED(ObjectIDValidate(id))) {
-        action->SetError(701, "Incorrect ObjectID.");
-        return NPT_FAILURE;
-    }
-
     if (id.StartsWith("virtualpath://")) {
         id.TrimRight("/");
         if (id == "virtualpath://upnproot") {
@@ -552,7 +498,6 @@ CUPnPServer::OnBrowseMetadata(PLT_ActionReference&          action,
             item->SetLabel("Root");
             item->SetLabelPreformated(true);
             object = Build(item, true, context, thumb_loader);
-            object->m_ParentID = "-1";
         } else {
             return NPT_FAILURE;
         }
@@ -560,28 +505,16 @@ CUPnPServer::OnBrowseMetadata(PLT_ActionReference&          action,
         // determine if it's a container by calling CDirectory::Exists
         item.reset(new CFileItem((const char*)id, CDirectory::Exists((const char*)id)));
 
-        // attempt to determine the parent of this item
+        // determine parent id for shared paths only
+        // otherwise let db find out
         CStdString parent;
-        if (URIUtils::IsVideoDb((const char*)id) || URIUtils::IsMusicDb((const char*)id) || StringUtils::StartsWithNoCase((const char*)id, "library://video/")) {
-            if (!URIUtils::GetParentPath((const char*)id, parent)) {
-                parent = "0";
-            }
-        }
-        else {
-            // non-library objects - playlists / sources
-            //
-            // we could instead store the parents in a hash during every browse
-            // or could handle this in URIUtils::GetParentPath() possibly,
-            // however this is quicker to implement and subsequently purge when a
-            // better solution presents itself
-            CStdString child_id((const char*)id);
-            if      (StringUtils::StartsWithNoCase(child_id, "special://musicplaylists/"))          parent = "musicdb://";
-            else if (StringUtils::StartsWithNoCase(child_id, "special://videoplaylists/"))          parent = "library://video/";
-            else if (StringUtils::StartsWithNoCase(child_id, "sources://video/"))                   parent = "library://video/";
-            else if (StringUtils::StartsWithNoCase(child_id, "special://profile/playlists/music/")) parent = "special://musicplaylists/";
-            else if (StringUtils::StartsWithNoCase(child_id, "special://profile/playlists/video/")) parent = "special://videoplaylists/";
-            else parent = "sources://video/"; // this can only match video sources
-        }
+        if (!URIUtils::GetParentPath((const char*)id, parent)) parent = "0";
+
+//#ifdef WMP_ID_MAPPING
+//        if (!id.StartsWith("musicdb://") && !id.StartsWith("videodb://")) {
+//            parent = "";
+//        }
+//#endif
 
         if (item->IsVideoDb()) {
             thumb_loader = NPT_Reference<CThumbLoader>(new CVideoThumbLoader());
@@ -590,7 +523,7 @@ CUPnPServer::OnBrowseMetadata(PLT_ActionReference&          action,
             thumb_loader = NPT_Reference<CThumbLoader>(new CMusicThumbLoader());
         }
         if (!thumb_loader.IsNull()) {
-            thumb_loader->OnLoaderStart();
+            thumb_loader->Initialize();
         }
         object = Build(item, true, context, thumb_loader, parent.empty()?NULL:parent.c_str());
     }
@@ -637,11 +570,6 @@ CUPnPServer::OnBrowseDirectChildren(PLT_ActionReference&          action,
 
     CLog::Log(LOGINFO, "UPnP: Received Browse DirectChildren request for object '%s', with sort criteria %s", object_id, sort_criteria);
 
-    if(NPT_FAILED(ObjectIDValidate(parent_id))) {
-        action->SetError(701, "Incorrect ObjectID.");
-        return NPT_FAILURE;
-    }
-
     items.SetPath(CStdString(parent_id));
 
     // guard against loading while saving to the same cache file
@@ -665,21 +593,14 @@ CUPnPServer::OnBrowseDirectChildren(PLT_ActionReference&          action,
             items.Add(item);
 
             // video library
-            item.reset(new CFileItem("library://video/", true));
+            item.reset(new CFileItem("library://video", true));
             item->SetLabel("Video Library");
             item->SetLabelPreformated(true);
             items.Add(item);
 
-            items.Sort(SortByLabel, SortOrderAscending);
+            items.Sort(SORT_METHOD_LABEL, SortOrderAscending);
         } else {
-            // this is the only way to hide unplayable items in the 'files'
-            // view as we cannot tell what context (eg music vs video) the
-            // request came from
-            string supported = g_advancedSettings.m_pictureExtensions + "|"
-                             + g_advancedSettings.m_videoExtensions + "|"
-                             + g_advancedSettings.m_musicExtensions + "|"
-                             + g_advancedSettings.m_discStubExtensions;
-            CDirectory::GetDirectory((const char*)parent_id, items, supported);
+            CDirectory::GetDirectory((const char*)parent_id, items);
             DefaultSortItems(items);
         }
 
@@ -689,22 +610,6 @@ CUPnPServer::OnBrowseDirectChildren(PLT_ActionReference&          action,
         }
     }
 
-    // as there's no library://music support, manually add playlists and music
-    // video nodes
-    if (items.GetPath() == "musicdb://") {
-      CFileItemPtr playlists(new CFileItem("special://musicplaylists/", true));
-      playlists->SetLabel(g_localizeStrings.Get(136));
-      items.Add(playlists);
-
-      CVideoDatabase database;
-      database.Open();
-      if (database.HasContent(VIDEODB_CONTENT_MUSICVIDEOS)) {
-          CFileItemPtr mvideos(new CFileItem("library://video/musicvideos/", true));
-          mvideos->SetLabel(g_localizeStrings.Get(20389));
-          items.Add(mvideos);
-      }
-    }
-
     // Don't pass parent_id if action is Search not BrowseDirectChildren, as
     // we want the engine to determine the best parent id, not necessarily the one
     // passed
@@ -743,26 +648,20 @@ CUPnPServer::BuildResponse(PLT_ActionReference&          action,
     // we will reuse this ThumbLoader for all items
     NPT_Reference<CThumbLoader> thumb_loader;
 
-    if (URIUtils::IsVideoDb(items.GetPath()) ||
-        StringUtils::StartsWithNoCase(items.GetPath(), "library://video/") ||
-        StringUtils::StartsWithNoCase(items.GetPath(), "special://profile/playlists/video/")) {
-
+    if (URIUtils::IsVideoDb(items.GetPath()) || items.GetPath().Left(15) == "library://video") {
         thumb_loader = NPT_Reference<CThumbLoader>(new CVideoThumbLoader());
     }
-    else if (URIUtils::IsMusicDb(items.GetPath()) ||
-        StringUtils::StartsWithNoCase(items.GetPath(), "special://profile/playlists/music/")) {
-
+    else if (URIUtils::IsMusicDb(items.GetPath())) {
         thumb_loader = NPT_Reference<CThumbLoader>(new CMusicThumbLoader());
     }
     if (!thumb_loader.IsNull()) {
-        thumb_loader->OnLoaderStart();
+        thumb_loader->Initialize();
     }
 
     // this isn't pretty but needed to properly hide the addons node from clients
-    if (StringUtils::StartsWith(items.GetPath(), "library")) {
+    if (items.GetPath().Left(7) == "library") {
         for (int i=0; i<items.Size(); i++) {
-            if (StringUtils::StartsWith(items[i]->GetPath(), "addons") ||
-                StringUtils::EndsWith(items[i]->GetPath(), "/addons.xml/"))
+            if (items[i]->GetPath().Left(6) == "addons")
                 items.Remove(i);
         }
     }
@@ -858,7 +757,7 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
             count = count?count-1:0;
 
             // genre
-            if (id.StartsWith("musicdb://genres/")) {
+            if (id.StartsWith("musicdb://1/")) {
                 // all tracks of all genres
                 if (count == 1)
                     id += "-1/-1/-1/";
@@ -868,14 +767,14 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
                 // all tracks of a specific genre of a specfic artist
                 else if (count == 3)
                     id += "-1/";
-            } else if (id.StartsWith("musicdb://artists/")) {
+            } else if (id.StartsWith("musicdb://2/")) {
                 // all tracks by all artists
                 if (count == 1)
                     id += "-1/-1/";
                 // all tracks of a specific artist
                 else if (count == 2)
                     id += "-1/";
-            } else if (id.StartsWith("musicdb://albums/")) {
+            } else if (id.StartsWith("musicdb://3/")) {
                 // all albums ?
                 if (count == 1) id += "-1/";
             }
@@ -897,30 +796,31 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
         if (genre.GetLength() > 0) {
             // all tracks by genre filtered by artist and/or album
             CStdString strPath;
-            strPath = StringUtils::Format("musicdb://genres/%ld/%ld/%ld/",
-                                          database.GetGenreByName((const char*)genre),
-                                          database.GetArtistByName((const char*)artist), // will return -1 if no artist
-                                          database.GetAlbumByName((const char*)album));  // will return -1 if no album
+            strPath.Format("musicdb://1/%ld/%ld/%ld/",
+                database.GetGenreByName((const char*)genre),
+                database.GetArtistByName((const char*)artist), // will return -1 if no artist
+                database.GetAlbumByName((const char*)album));  // will return -1 if no album
 
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         } else if (artist.GetLength() > 0) {
             // all tracks by artist name filtered by album if passed
             CStdString strPath;
-            strPath = StringUtils::Format("musicdb://artists/%ld/%ld/",
-                                          database.GetArtistByName((const char*)artist),
-                                          database.GetAlbumByName((const char*)album)); // will return -1 if no album
+            strPath.Format("musicdb://2/%ld/%ld/",
+                database.GetArtistByName((const char*)artist),
+                database.GetAlbumByName((const char*)album)); // will return -1 if no album
 
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         } else if (album.GetLength() > 0) {
             // all tracks by album name
-            CStdString strPath = StringUtils::Format("musicdb://albums/%ld/",
-                                                     database.GetAlbumByName((const char*)album));
+            CStdString strPath;
+            strPath.Format("musicdb://3/%ld/",
+                database.GetAlbumByName((const char*)album));
 
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         }
 
         // browse all songs
-        return OnBrowseDirectChildren(action, "musicdb://songs/", filter, starting_index, requested_count, sort_criteria, context);
+        return OnBrowseDirectChildren(action, "musicdb://4/", filter, starting_index, requested_count, sort_criteria, context);
     } else if (NPT_String(search_criteria).Find("object.container.album.musicAlbum") >= 0) {
         // sonos filters by genre
         NPT_String genre = FindSubCriteria(search_criteria, "upnp:genre");
@@ -936,30 +836,33 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
         database.Open();
 
         if (genre.GetLength() > 0) {
-            CStdString strPath = StringUtils::Format("musicdb://genres/%ld/%ld/",
-                                                     database.GetGenreByName((const char*)genre),
-                                                     database.GetArtistByName((const char*)artist)); // no artist should return -1
+            CStdString strPath;
+            strPath.Format("musicdb://1/%ld/%ld/",
+                database.GetGenreByName((const char*)genre),
+                database.GetArtistByName((const char*)artist)); // no artist should return -1
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         } else if (artist.GetLength() > 0) {
-            CStdString strPath = StringUtils::Format("musicdb://artists/%ld/",
-                                                     database.GetArtistByName((const char*)artist));
+            CStdString strPath;
+            strPath.Format("musicdb://2/%ld/",
+                database.GetArtistByName((const char*)artist));
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         }
 
         // all albums
-        return OnBrowseDirectChildren(action, "musicdb://albums/", filter, starting_index, requested_count, sort_criteria, context);
+        return OnBrowseDirectChildren(action, "musicdb://3/", filter, starting_index, requested_count, sort_criteria, context);
     } else if (NPT_String(search_criteria).Find("object.container.person.musicArtist") >= 0) {
         // Sonos filters by genre
         NPT_String genre = FindSubCriteria(search_criteria, "upnp:genre");
         if (genre.GetLength() > 0) {
             CMusicDatabase database;
             database.Open();
-            CStdString strPath = StringUtils::Format("musicdb://genres/%ld/", database.GetGenreByName((const char*)genre));
+            CStdString strPath;
+            strPath.Format("musicdb://1/%ld/", database.GetGenreByName((const char*)genre));
             return OnBrowseDirectChildren(action, strPath.c_str(), filter, starting_index, requested_count, sort_criteria, context);
         }
-        return OnBrowseDirectChildren(action, "musicdb://artists/", filter, starting_index, requested_count, sort_criteria, context);
+        return OnBrowseDirectChildren(action, "musicdb://2/", filter, starting_index, requested_count, sort_criteria, context);
     }  else if (NPT_String(search_criteria).Find("object.container.genre.musicGenre") >= 0) {
-        return OnBrowseDirectChildren(action, "musicdb://genres/", filter, starting_index, requested_count, sort_criteria, context);
+        return OnBrowseDirectChildren(action, "musicdb://1/", filter, starting_index, requested_count, sort_criteria, context);
     } else if (NPT_String(search_criteria).Find("object.container.playlistContainer") >= 0) {
         return OnBrowseDirectChildren(action, "special://musicplaylists/", filter, starting_index, requested_count, sort_criteria, context);
     } else if (NPT_String(search_criteria).Find("object.item.videoItem") >= 0) {
@@ -971,14 +874,15 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
         return NPT_SUCCESS;
       }
 
-      if (!database.GetMoviesNav("videodb://movies/titles/", items)) {
+      if (!database.GetMoviesNav("videodb://1/2/", items)) {
         action->SetError(800, "Internal Error");
         return NPT_SUCCESS;
       }
       itemsall.Append(items);
       items.Clear();
 
-      if (!database.GetEpisodesByWhere("videodb://tvshows/titles/", "", items)) {
+      // TODO - set proper base url for this
+      if (!database.GetEpisodesByWhere("videodb://2/0/", "", items, false)) {
         action->SetError(800, "Internal Error");
         return NPT_SUCCESS;
       }
@@ -994,142 +898,6 @@ CUPnPServer::OnSearchContainer(PLT_ActionReference&          action,
   return NPT_FAILURE;
 }
 
-/*----------------------------------------------------------------------
-|   CUPnPServer::OnUpdateObject
-+---------------------------------------------------------------------*/
-NPT_Result
-CUPnPServer::OnUpdateObject(PLT_ActionReference&             action,
-                            const char*                      object_id,
-                            NPT_Map<NPT_String,NPT_String>&  current_vals,
-                            NPT_Map<NPT_String,NPT_String>&  new_vals,
-                            const PLT_HttpRequestContext&    context)
-{
-    CStdString path(CURL::Decode(object_id));
-    CFileItem updated;
-    updated.SetPath(path);
-    CLog::Log(LOGINFO, "UPnP: OnUpdateObject: %s from %s", path.c_str(),
-                       (const char*) context.GetRemoteAddress().GetIpAddress().ToString());
-
-    NPT_String playCount, position;
-    int err;
-    const char* msg = NULL;
-    bool updatelisting(false);
-
-    // we pause eventing as multiple announces may happen in this operation
-    PLT_Service* service = NULL;
-    NPT_CHECK_LABEL(FindServiceById("urn:upnp-org:serviceId:ContentDirectory", service), error);
-    NPT_CHECK_LABEL(service->PauseEventing(), error);
-
-    if (updated.IsVideoDb()) {
-        CVideoDatabase db;
-        NPT_CHECK_LABEL(!db.Open(), error);
-
-        // must first determine type of file from object id
-        VIDEODATABASEDIRECTORY::CQueryParams params;
-        VIDEODATABASEDIRECTORY::CDirectoryNode::GetDatabaseInfo(path.c_str(), params);
-
-        int id = -1;
-        VIDEODB_CONTENT_TYPE content_type;
-        if ((id = params.GetMovieId()) >= 0 )
-            content_type = VIDEODB_CONTENT_MOVIES;
-        else if ((id = params.GetEpisodeId()) >= 0 )
-            content_type = VIDEODB_CONTENT_EPISODES;
-        else if ((id = params.GetMVideoId()) >= 0 )
-            content_type = VIDEODB_CONTENT_MUSICVIDEOS;
-        else {
-            err = 701;
-            msg = "No such object";
-            goto failure;
-        }
-
-        CStdString file_path;
-        db.GetFilePathById(id, file_path, content_type);
-        CVideoInfoTag tag;
-        db.LoadVideoInfo(file_path, tag);
-        updated.SetFromVideoInfoTag(tag);
-        CLog::Log(LOGINFO, "UPNP: Translated to %s", file_path.c_str());
-
-        position = new_vals["lastPlaybackPosition"];
-        playCount = new_vals["playCount"];
-
-        if (!position.IsEmpty()
-              && position.Compare(current_vals["lastPlaybackPosition"]) != 0) {
-            NPT_UInt32 resume;
-            NPT_CHECK_LABEL(position.ToInteger32(resume), args);
-
-            if (resume <= 0)
-                db.ClearBookMarksOfFile(file_path, CBookmark::RESUME);
-            else {
-                CBookmark bookmark;
-                bookmark.timeInSeconds = resume;
-                bookmark.totalTimeInSeconds = resume + 100; // not required to be correct
-
-                db.AddBookMarkToFile(file_path, bookmark, CBookmark::RESUME);
-            }
-            if (playCount.IsEmpty()) {
-              CVariant data;
-              data["id"] = updated.GetVideoInfoTag()->m_iDbId;
-              data["type"] = updated.GetVideoInfoTag()->m_type;
-              ANNOUNCEMENT::CAnnouncementManager::Get().Announce(ANNOUNCEMENT::VideoLibrary, "xbmc", "OnUpdate", data);
-            }
-            updatelisting = true;
-        }
-
-        if (!playCount.IsEmpty()
-              && playCount.Compare(current_vals["playCount"]) != 0) {
-
-            NPT_UInt32 count;
-            NPT_CHECK_LABEL(playCount.ToInteger32(count), args);
-            db.SetPlayCount(updated, count);
-            updatelisting = true;
-        }
-
-        // we must load the changed settings before propagating to local UI
-        if (updatelisting) {
-            db.LoadVideoInfo(file_path, tag);
-            updated.SetFromVideoInfoTag(tag);
-        }
-
-    } else if (updated.IsMusicDb()) {
-      //TODO implement this
-
-    } else {
-        err = 701;
-        msg = "No such object";
-        goto failure;
-    }
-
-    if (updatelisting) {
-        updated.SetPath(path);
-        if (updated.IsVideoDb())
-             CUtil::DeleteVideoDatabaseDirectoryCache();
-        else if (updated.IsMusicDb())
-             CUtil::DeleteMusicDatabaseDirectoryCache();
-
-        CFileItemPtr msgItem(new CFileItem(updated));
-        CGUIMessage message(GUI_MSG_NOTIFY_ALL, g_windowManager.GetActiveWindow(), 0, GUI_MSG_UPDATE_ITEM, 1, msgItem);
-        g_windowManager.SendThreadMessage(message);
-    }
-
-    NPT_CHECK_LABEL(service->PauseEventing(false), error);
-    return NPT_SUCCESS;
-
-args:
-    err = 402;
-    msg = "Invalid args";
-    goto failure;
-
-error:
-    err = 501;
-    msg = "Internal error";
-
-failure:
-    CLog::Log(LOGERROR, "UPNP: OnUpdateObject failed with err %d:%s", err, msg);
-    action->SetError(err, msg);
-    service->PauseEventing(false);
-    return NPT_FAILURE;
-}
-
 /*----------------------------------------------------------------------
 |   CUPnPServer::ServeFile
 +---------------------------------------------------------------------*/
@@ -1208,65 +976,53 @@ bool
 CUPnPServer::SortItems(CFileItemList& items, const char* sort_criteria)
 {
   CStdString criteria(sort_criteria);
-  if (criteria.empty()) {
+  if (criteria.IsEmpty()) {
     return false;
   }
 
   bool sorted = false;
-  vector<string> tokens = StringUtils::Split(criteria, ",");
-  for (vector<string>::reverse_iterator itr = tokens.rbegin(); itr != tokens.rend(); itr++) {
-    SortDescription sorting;
+  CStdStringArray tokens = StringUtils::SplitString(criteria, ",");
+  for (vector<CStdString>::reverse_iterator itr = tokens.rbegin(); itr != tokens.rend(); itr++) {
     /* Platinum guarantees 1st char is - or + */
-    sorting.sortOrder = StringUtils::StartsWith(*itr, "+") ? SortOrderAscending : SortOrderDescending;
-    CStdString method = itr->substr(1);
+    SortOrder order = itr->Left(1).Equals("+") ? SortOrderAscending : SortOrderDescending;
+    CStdString method = itr->Mid(1);
+
+    SORT_METHOD scheme = SORT_METHOD_LABEL_IGNORE_THE;
 
     /* resource specific */
     if (method.Equals("res@duration"))
-      sorting.sortBy = SortByTime;
+      scheme = SORT_METHOD_DURATION;
     else if (method.Equals("res@size"))
-      sorting.sortBy = SortBySize;
+      scheme = SORT_METHOD_SIZE;
     else if (method.Equals("res@bitrate"))
-      sorting.sortBy = SortByBitrate;
+      scheme = SORT_METHOD_BITRATE;
 
     /* dc: */
     else if (method.Equals("dc:date"))
-      sorting.sortBy = SortByDate;
+      scheme = SORT_METHOD_DATE;
     else if (method.Equals("dc:title"))
-    {
-      sorting.sortBy = SortByTitle;
-      sorting.sortAttributes = SortAttributeIgnoreArticle;
-    }
+      scheme = SORT_METHOD_TITLE_IGNORE_THE;
 
     /* upnp: */
     else if (method.Equals("upnp:album"))
-      sorting.sortBy = SortByAlbum;
+      scheme = SORT_METHOD_ALBUM;
     else if (method.Equals("upnp:artist") || method.Equals("upnp:albumArtist"))
-      sorting.sortBy = SortByArtist;
+      scheme = SORT_METHOD_ARTIST;
     else if (method.Equals("upnp:episodeNumber"))
-      sorting.sortBy = SortByEpisodeNumber;
-    else if (method.Equals("upnp:episodeCount"))
-      sorting.sortBy = SortByNumberOfEpisodes;
-    else if (method.Equals("upnp:episodeSeason"))
-      sorting.sortBy = SortBySeason;
+      scheme = SORT_METHOD_EPISODE;
     else if (method.Equals("upnp:genre"))
-      sorting.sortBy = SortByGenre;
+      scheme = SORT_METHOD_GENRE;
     else if (method.Equals("upnp:originalTrackNumber"))
-      sorting.sortBy = SortByTrackNumber;
+      scheme = SORT_METHOD_TRACKNUM;
     else if(method.Equals("upnp:rating"))
-      sorting.sortBy = SortByMPAA;
-    else if (method.Equals("xbmc:rating"))
-      sorting.sortBy = SortByRating;
-    else if (method.Equals("xbmc:dateadded"))
-      sorting.sortBy = SortByDateAdded;
-    else if (method.Equals("xbmc:votes"))
-      sorting.sortBy = SortByVotes;
+      scheme = SORT_METHOD_SONG_RATING;
     else {
       CLog::Log(LOGINFO, "UPnP: unsupported sort criteria '%s' passed", method.c_str());
       continue; // needed so unidentified sort methods don't re-sort by label
     }
 
-    CLog::Log(LOGINFO, "UPnP: Sorting by method %d, order %d, attributes %d", sorting.sortBy, sorting.sortOrder, sorting.sortAttributes);
-    items.Sort(sorting);
+    CLog::Log(LOGINFO, "UPnP: Sorting by %d, %d", scheme, order);
+    items.Sort(scheme, order);
     sorted = true;
   }
 
@@ -1279,8 +1035,7 @@ CUPnPServer::DefaultSortItems(CFileItemList& items)
   CGUIViewState* viewState = CGUIViewState::GetViewState(items.IsVideoDb() ? WINDOW_VIDEO_NAV : -1, items);
   if (viewState)
   {
-    SortDescription sorting = viewState->GetSortMethod();
-    items.Sort(sorting.sortBy, sorting.sortOrder, sorting.sortAttributes);
+    items.Sort(viewState->GetSortMethod(), viewState->GetSortOrder());
     delete viewState;
   }
 }
