@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2005-2008 Team XBMC
+ *      Copyright (C) 2005-2012 Team XBMC
  *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
@@ -13,32 +13,47 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with XBMC; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
- *  http://www.gnu.org/copyleft/gpl.html
+ *  along with XBMC; see the file COPYING.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "WIN32Util.h"
-#include "GUISettings.h"
-#include "../Util.h"
-#include "FileSystem/cdioSupport.h"
+#include "settings/GUISettings.h"
+#include "Util.h"
+#include "utils/URIUtils.h"
+#include "storage/cdioSupport.h"
 #include "PowrProf.h"
 #include "WindowHelper.h"
 #include "Application.h"
 #include <shlobj.h>
-#include "SpecialProtocol.h"
+#include "filesystem/SpecialProtocol.h"
 #include "my_ntddscsi.h"
 #if _MSC_VER > 1400
 #include "Setupapi.h"
 #endif
-#include "MediaManager.h"
-#include "WindowingFactory.h"
-#include "LocalizeStrings.h"
-#include "log.h"
-#include "StringUtils.h"
+#include "storage/MediaManager.h"
+#include "windowing/WindowingFactory.h"
+#include "guilib/LocalizeStrings.h"
+#include "utils/log.h"
+#include "utils/StringUtils.h"
+#include "DllPaths_win32.h"
+#include "FileSystem/File.h"
+#include "utils/URIUtils.h"
 
-#define DLL_ENV_PATH "special://xbmc/system/;special://xbmc/system/players/dvdplayer/;special://xbmc/system/players/paplayer/;special://xbmc/system/python/"
+// default Broadcom registy bits (setup when installing a CrystalHD card)
+#define BC_REG_PATH       "Software\\Broadcom\\MediaPC"
+#define BC_REG_PRODUCT    "CrystalHD" // 70012/70015
+#define BC_BCM_DLL        "bcmDIL.dll"
+#define BC_REG_INST_PATH  "InstallPath"
+
+#define DLL_ENV_PATH "special://xbmc/system/;" \
+                     "special://xbmc/system/players/dvdplayer/;" \
+                     "special://xbmc/system/players/paplayer/;" \
+                     "special://xbmc/system/cdrip/;" \
+                     "special://xbmc/system/python/;" \
+                     "special://xbmc/system/webserver/;" \
+                     "special://xbmc/"
 
 extern HWND g_hWnd;
 
@@ -53,67 +68,6 @@ CWIN32Util::~CWIN32Util(void)
 {
 }
 
-
-const CStdString CWIN32Util::GetNextFreeDriveLetter()
-{
-  for(int iDrive='a';iDrive<='z';iDrive++)
-  {
-    CStdString strDrive;
-    strDrive.Format("%c:",iDrive);
-    int iType = GetDriveType(strDrive);
-    if(iType == DRIVE_NO_ROOT_DIR && iDrive != 'a' && iDrive != 'b')
-      return strDrive;
-  }
-  return StringUtils::EmptyString;
-}
-
-CStdString CWIN32Util::MountShare(const CStdString &smbPath, const CStdString &strUser, const CStdString &strPass, DWORD *dwError)
-{
-  NETRESOURCE nr;
-  memset(&nr,0,sizeof(nr));
-  CStdString strRemote = smbPath;
-  CStdString strDrive = CWIN32Util::GetNextFreeDriveLetter();
-
-  if(strDrive == StringUtils::EmptyString)
-    return StringUtils::EmptyString;
-
-  strRemote.Replace('/', '\\');
-
-  nr.lpRemoteName = (LPTSTR)(LPCTSTR)strRemote.c_str();
-  nr.lpLocalName  = (LPTSTR)(LPCTSTR)strDrive.c_str();
-  nr.dwType       = RESOURCETYPE_DISK;
-
-  DWORD dwRes = WNetAddConnection2(&nr,(LPCTSTR)strPass.c_str(), (LPCTSTR)strUser.c_str(), NULL);
-
-  if(dwError != NULL)
-    *dwError = dwRes;
-
-  if(dwRes != NO_ERROR)
-  {
-    CLog::Log(LOGERROR, "Can't mount %s to %s. Error code %d",strRemote.c_str(), strDrive.c_str(),dwRes);
-    return StringUtils::EmptyString;
-  }
-
-  return strDrive;
-}
-
-DWORD CWIN32Util::UmountShare(const CStdString &strPath)
-{
-  return WNetCancelConnection2((LPCTSTR)strPath.c_str(),NULL,true);
-}
-
-CStdString CWIN32Util::MountShare(const CStdString &strPath, DWORD *dwError)
-{
-  CURL url(strPath);
-  CStdString strPassword = url.GetPassWord();
-  CStdString strUserName = url.GetUserName();
-  CStdString strPathToShare = "\\\\"+url.GetHostName() + "\\" + url.GetShareName();
-  if(!url.GetUserName().IsEmpty())
-    return CWIN32Util::MountShare(strPathToShare, strUserName, strPassword, dwError);
-  else
-    return CWIN32Util::MountShare(strPathToShare, "", "", dwError);
-}
-
 CStdString CWIN32Util::URLEncode(const CURL &url)
 {
   /* due to smb wanting encoded urls we have to build it manually */
@@ -152,14 +106,16 @@ CStdString CWIN32Util::URLEncode(const CURL &url)
   return flat;
 }
 
-int CWIN32Util::GetDriveStatus(const CStdString &strPath)
+int CWIN32Util::GetDriveStatus(const CStdString &strPath, bool bStatusEx)
 {
   HANDLE hDevice;               // handle to the drive to be examined
   int iResult;                  // results flag
   ULONG ulChanges=0;
   DWORD dwBytesReturned;
   T_SPDT_SBUF sptd_sb;  //SCSI Pass Through Direct variable.
-  byte DataBuf[16];  //Buffer for holding data to/from drive.
+  byte DataBuf[8];  //Buffer for holding data to/from drive.
+
+  CLog::Log(LOGDEBUG, __FUNCTION__": Requesting status for drive %s.", strPath.c_str());
 
   hDevice = CreateFile( strPath.c_str(),                  // drive
                         0,                                // no access to the drive
@@ -171,9 +127,11 @@ int CWIN32Util::GetDriveStatus(const CStdString &strPath)
 
   if (hDevice == INVALID_HANDLE_VALUE)                    // cannot open the drive
   {
+    CLog::Log(LOGERROR, __FUNCTION__": Failed to CreateFile for %s.", strPath.c_str());
     return -1;
   }
 
+  CLog::Log(LOGDEBUG, __FUNCTION__": Requesting media status for drive %s.", strPath.c_str());
   iResult = DeviceIoControl((HANDLE) hDevice,             // handle to device
                              IOCTL_STORAGE_CHECK_VERIFY2, // dwIoControlCode
                              NULL,                        // lpInBuffer
@@ -188,8 +146,12 @@ int CWIN32Util::GetDriveStatus(const CStdString &strPath)
   if(iResult == 1)
     return 2;
 
+  // don't request the tray status as we often doesn't need it
+  if(!bStatusEx)
+    return 0;
+
   hDevice = CreateFile( strPath.c_str(),
-                        GENERIC_READ | GENERIC_WRITE, 
+                        GENERIC_READ | GENERIC_WRITE,
                         FILE_SHARE_READ | FILE_SHARE_WRITE,
                         NULL,
                         OPEN_EXISTING,
@@ -198,18 +160,19 @@ int CWIN32Util::GetDriveStatus(const CStdString &strPath)
 
   if (hDevice == INVALID_HANDLE_VALUE)
   {
+    CLog::Log(LOGERROR, __FUNCTION__": Failed to CreateFile2 for %s.", strPath.c_str());
     return -1;
   }
 
   sptd_sb.sptd.Length=sizeof(SCSI_PASS_THROUGH_DIRECT);
-  sptd_sb.sptd.PathId=0; 
+  sptd_sb.sptd.PathId=0;
   sptd_sb.sptd.TargetId=0;
   sptd_sb.sptd.Lun=0;
   sptd_sb.sptd.CdbLength=10;
   sptd_sb.sptd.SenseInfoLength=MAX_SENSE_LEN;
   sptd_sb.sptd.DataIn=SCSI_IOCTL_DATA_IN;
-  sptd_sb.sptd.DataTransferLength=8;
-  sptd_sb.sptd.TimeOutValue=108000;
+  sptd_sb.sptd.DataTransferLength=sizeof(DataBuf);
+  sptd_sb.sptd.TimeOutValue=2;
   sptd_sb.sptd.DataBuffer=(PVOID)&(DataBuf);
   sptd_sb.sptd.SenseInfoOffset=sizeof(SCSI_PASS_THROUGH_DIRECT);
 
@@ -230,9 +193,11 @@ int CWIN32Util::GetDriveStatus(const CStdString &strPath)
   sptd_sb.sptd.Cdb[14]=0;
   sptd_sb.sptd.Cdb[15]=0;
 
+  ZeroMemory(DataBuf, 8);
   ZeroMemory(sptd_sb.SenseBuf, MAX_SENSE_LEN);
 
   //Send the command to drive
+  CLog::Log(LOGDEBUG, __FUNCTION__": Requesting tray status for drive %s.", strPath.c_str());
   iResult = DeviceIoControl((HANDLE) hDevice,
                             IOCTL_SCSI_PASS_THROUGH_DIRECT,
                             (PVOID)&sptd_sb, (DWORD)sizeof(sptd_sb),
@@ -252,6 +217,7 @@ int CWIN32Util::GetDriveStatus(const CStdString &strPath)
     else
       return 2; // tray closed, media present
   }
+  CLog::Log(LOGERROR, __FUNCTION__": Could not determine tray status %d", GetLastError());
   return -1;
 }
 
@@ -287,7 +253,7 @@ bool CWIN32Util::PowerManagement(PowerState State)
   }
   // Get the LUID for the shutdown privilege.
   LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);
-  tkp.PrivilegeCount = 1;  // one privilege to set   
+  tkp.PrivilegeCount = 1;  // one privilege to set
   tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
   // Get the shutdown privilege for this process.
   AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);
@@ -326,6 +292,16 @@ bool CWIN32Util::PowerManagement(PowerState State)
 #endif
 }
 
+int CWIN32Util::BatteryLevel()
+{
+  SYSTEM_POWER_STATUS SystemPowerStatus;
+
+  if (GetSystemPowerStatus(&SystemPowerStatus) && SystemPowerStatus.BatteryLifePercent != 255)
+      return SystemPowerStatus.BatteryLifePercent;
+
+  return 0;
+}
+
 bool CWIN32Util::XBMCShellExecute(const CStdString &strPath, bool bWaitForScriptExit)
 {
   CStdString strCommand = strPath;
@@ -353,22 +329,27 @@ bool CWIN32Util::XBMCShellExecute(const CStdString &strPath, bool bWaitForScript
 
   strExe.Replace("\"","");
 
-  strWorkingDir = strExe; 
-  iIndex = strWorkingDir.ReverseFind('\\'); 
-  if(iIndex != -1) 
-  { 
-    strWorkingDir[iIndex+1] = '\0'; 
-  } 
+  strWorkingDir = strExe;
+  iIndex = strWorkingDir.ReverseFind('\\');
+  if(iIndex != -1)
+  {
+    strWorkingDir[iIndex+1] = '\0';
+  }
+
+  CStdStringW WstrExe, WstrParams, WstrWorkingDir;
+  g_charsetConverter.utf8ToW(strExe, WstrExe);
+  g_charsetConverter.utf8ToW(strParams, WstrParams);
+  g_charsetConverter.utf8ToW(strWorkingDir, WstrWorkingDir);
 
   bool ret;
-  SHELLEXECUTEINFO ShExecInfo = {0};
-  ShExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);
+  SHELLEXECUTEINFOW ShExecInfo = {0};
+  ShExecInfo.cbSize = sizeof(SHELLEXECUTEINFOW);
   ShExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
   ShExecInfo.hwnd = NULL;
   ShExecInfo.lpVerb = NULL;
-  ShExecInfo.lpFile = strExe.c_str();
-  ShExecInfo.lpParameters = strParams.c_str();
-  ShExecInfo.lpDirectory = strWorkingDir.c_str();
+  ShExecInfo.lpFile = WstrExe.c_str();
+  ShExecInfo.lpParameters = WstrParams.c_str();
+  ShExecInfo.lpDirectory = WstrWorkingDir.c_str();
   ShExecInfo.nShow = SW_SHOW;
   ShExecInfo.hInstApp = NULL;
 
@@ -376,7 +357,7 @@ bool CWIN32Util::XBMCShellExecute(const CStdString &strPath, bool bWaitForScript
 
   LockSetForegroundWindow(LSFW_UNLOCK);
   ShowWindow(g_hWnd,SW_MINIMIZE);
-  ret = ShellExecuteEx(&ShExecInfo) == TRUE;
+  ret = ShellExecuteExW(&ShExecInfo) == TRUE;
   g_windowHelper.SetHANDLE(ShExecInfo.hProcess);
 
   // ShellExecute doesn't return the window of the started process
@@ -396,7 +377,6 @@ bool CWIN32Util::XBMCShellExecute(const CStdString &strPath, bool bWaitForScript
 
 std::vector<CStdString> CWIN32Util::GetDiskUsage()
 {
-  CStdString strRet;
   vector<CStdString> result;
   ULARGE_INTEGER ULTotal= { { 0 } };
   ULARGE_INTEGER ULTotalFree= { { 0 } };
@@ -405,11 +385,13 @@ std::vector<CStdString> CWIN32Util::GetDiskUsage()
   DWORD dwStrLength= GetLogicalDriveStrings( 0, pcBuffer );
   if( dwStrLength != 0 )
   {
+    CStdString strRet;
+
     dwStrLength+= 1;
     pcBuffer= new char [dwStrLength];
     GetLogicalDriveStrings( dwStrLength, pcBuffer );
     int iPos= 0;
-    do 
+    do
     {
       CStdString strDrive = pcBuffer + iPos;
       if( DRIVE_FIXED == GetDriveType( strDrive.c_str()  ) &&
@@ -445,18 +427,43 @@ int CWIN32Util::GetDesktopColorDepth()
   return (int)devmode.dmBitsPerPel;
 }
 
-CStdString CWIN32Util::GetProfilePath()
+CStdString CWIN32Util::GetSpecialFolder(int csidl)
 {
   CStdString strProfilePath;
   WCHAR szPath[MAX_PATH];
 
-  if(SUCCEEDED(SHGetFolderPathW(NULL,CSIDL_APPDATA|CSIDL_FLAG_CREATE,NULL,0,szPath)))
+  if(SUCCEEDED(SHGetFolderPathW(NULL,csidl,NULL,0,szPath)))
   {
     g_charsetConverter.wToUTF8(szPath, strProfilePath);
     strProfilePath = UncToSmb(strProfilePath);
-  }  
+  }
   else
-    CUtil::GetHomePath(strProfilePath);
+    strProfilePath = "";
+
+  return strProfilePath;
+}
+
+CStdString CWIN32Util::GetSystemPath()
+{
+  return GetSpecialFolder(CSIDL_SYSTEM);
+}
+
+CStdString CWIN32Util::GetProfilePath()
+{
+  CStdString strProfilePath;
+  CStdString strHomePath;
+
+  CUtil::GetHomePath(strHomePath);
+
+  if(g_application.PlatformDirectoriesEnabled())
+    strProfilePath = URIUtils::AddFileToFolder(GetSpecialFolder(CSIDL_APPDATA|CSIDL_FLAG_CREATE), "XBMC");
+  else
+    strProfilePath = URIUtils::AddFileToFolder(strHomePath , "portable_data");
+
+  if (strProfilePath.length() == 0)
+    strProfilePath = strHomePath;
+
+  URIUtils::AddSlashAtEnd(strProfilePath);
 
   return strProfilePath;
 }
@@ -472,6 +479,17 @@ CStdString CWIN32Util::UncToSmb(const CStdString &strPath)
   return strRetPath;
 }
 
+CStdString CWIN32Util::SmbToUnc(const CStdString &strPath)
+{
+  CStdString strRetPath(strPath);
+  if(strRetPath.Left(6).Equals("smb://"))
+  {
+    strRetPath.Replace("smb://","\\\\");
+    strRetPath.Replace("/","\\");
+  }
+  return strRetPath;
+}
+
 void CWIN32Util::ExtendDllPath()
 {
   CStdStringW strEnvW;
@@ -497,27 +515,27 @@ void CWIN32Util::ExtendDllPath()
 HRESULT CWIN32Util::ToggleTray(const char cDriveLetter)
 {
   BOOL bRet= FALSE;
-  DWORD dwReq;
+  DWORD dwReq = 0;
   char cDL = cDriveLetter;
   if( !cDL )
   {
-    char* dvdDevice = CLibcdio::GetInstance()->GetDeviceFileName();
+    CStdString dvdDevice = g_mediaManager.TranslateDevicePath("");
     if(dvdDevice == "")
       return S_FALSE;
-    cDL = dvdDevice[4];
+    cDL = dvdDevice[0];
   }
-  
-  CStdString strVolFormat; 
+
+  CStdString strVolFormat;
   strVolFormat.Format( _T("\\\\.\\%c:" ), cDL);
-  HANDLE hDrive= CreateFile( strVolFormat.c_str(), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, 
+  HANDLE hDrive= CreateFile( strVolFormat.c_str(), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-  CStdString strRootFormat; 
+  CStdString strRootFormat;
   strRootFormat.Format( _T("%c:\\"), cDL);
-  if( ( hDrive != INVALID_HANDLE_VALUE || GetLastError() == NO_ERROR) && 
+  if( ( hDrive != INVALID_HANDLE_VALUE || GetLastError() == NO_ERROR) &&
       ( GetDriveType( strRootFormat ) == DRIVE_CDROM ) )
   {
     DWORD dwDummy;
-    dwReq = (GetDriveStatus(strVolFormat) == 1) ? IOCTL_STORAGE_LOAD_MEDIA : IOCTL_STORAGE_EJECT_MEDIA;
+    dwReq = (GetDriveStatus(strVolFormat, true) == 1) ? IOCTL_STORAGE_LOAD_MEDIA : IOCTL_STORAGE_EJECT_MEDIA;
     bRet = DeviceIoControl( hDrive, dwReq, NULL, 0, NULL, 0, &dwDummy, NULL);
     CloseHandle( hDrive );
   }
@@ -526,7 +544,10 @@ HRESULT CWIN32Util::ToggleTray(const char cDriveLetter)
   if(dwReq == IOCTL_STORAGE_EJECT_MEDIA && bRet == 1)
   {
     strRootFormat.Format( _T("%c:"), cDL);
-    g_application.getApplicationMessenger().OpticalUnMount(strRootFormat);
+    CMediaSource share;
+    share.strPath = strRootFormat;
+    share.strName = share.strPath;
+    g_mediaManager.RemoveAutoSource(share);
   }
   return bRet? S_OK : S_FALSE;
 }
@@ -536,18 +557,18 @@ HRESULT CWIN32Util::EjectTray(const char cDriveLetter)
   char cDL = cDriveLetter;
   if( !cDL )
   {
-    char* dvdDevice = CLibcdio::GetInstance()->GetDeviceFileName();
+    CStdString dvdDevice = g_mediaManager.TranslateDevicePath("");
     if(dvdDevice == "")
       return S_FALSE;
-    cDL = dvdDevice[4];
+    cDL = dvdDevice[0];
   }
-  
-  CStdString strVolFormat; 
+
+  CStdString strVolFormat;
   strVolFormat.Format( _T("\\\\.\\%c:" ), cDL);
 
-  if(GetDriveStatus(strVolFormat) != 1)
+  if(GetDriveStatus(strVolFormat, true) != 1)
     return ToggleTray(cDL);
-  else 
+  else
     return S_OK;
 }
 
@@ -556,27 +577,27 @@ HRESULT CWIN32Util::CloseTray(const char cDriveLetter)
   char cDL = cDriveLetter;
   if( !cDL )
   {
-    char* dvdDevice = CLibcdio::GetInstance()->GetDeviceFileName();
+    CStdString dvdDevice = g_mediaManager.TranslateDevicePath("");
     if(dvdDevice == "")
       return S_FALSE;
-    cDL = dvdDevice[4];
+    cDL = dvdDevice[0];
   }
-  
-  CStdString strVolFormat; 
+
+  CStdString strVolFormat;
   strVolFormat.Format( _T("\\\\.\\%c:" ), cDL);
 
-  if(GetDriveStatus(strVolFormat) == 1)
+  if(GetDriveStatus(strVolFormat, true) == 1)
     return ToggleTray(cDL);
-  else 
+  else
     return S_OK;
 }
 
-// safe removal of USB drives: 
+// safe removal of USB drives:
 // http://www.codeproject.com/KB/system/RemoveDriveByLetter.aspx
 // http://www.techtalkz.com/microsoft-device-drivers/250734-remove-usb-device-c-3.html
 
 #if _MSC_VER > 1400
-DEVINST CWIN32Util::GetDrivesDevInstByDiskNumber(long DiskNumber) 
+DEVINST CWIN32Util::GetDrivesDevInstByDiskNumber(long DiskNumber)
 {
 
   GUID* guid = (GUID*)(void*)&GUID_DEVINTERFACE_DISK;
@@ -601,7 +622,7 @@ DEVINST CWIN32Util::GetDrivesDevInstByDiskNumber(long DiskNumber)
 
   spdid.cbSize = sizeof(spdid);
 
-  while ( true ) 
+  while ( true )
   {
     bRet = SetupDiEnumDeviceInterfaces(hDevInfo, NULL, guid, dwIndex, &devInterfaceData);
     if (!bRet)
@@ -612,7 +633,7 @@ DEVINST CWIN32Util::GetDrivesDevInstByDiskNumber(long DiskNumber)
     dwSize = 0;
     SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid, NULL, 0, &dwSize, NULL);
 
-    if ( dwSize ) 
+    if ( dwSize )
     {
       pspdidd = (PSP_DEVICE_INTERFACE_DETAIL_DATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwSize);
       if ( pspdidd == NULL )
@@ -624,17 +645,17 @@ DEVINST CWIN32Util::GetDrivesDevInstByDiskNumber(long DiskNumber)
 
       long res = SetupDiGetDeviceInterfaceDetail(hDevInfo, &spdid,
       pspdidd, dwSize, &dwSize, &spdd);
-      if ( res ) 
+      if ( res )
       {
         HANDLE hDrive = CreateFile(pspdidd->DevicePath, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, NULL, NULL);
-        if ( hDrive != INVALID_HANDLE_VALUE ) 
+        if ( hDrive != INVALID_HANDLE_VALUE )
         {
           STORAGE_DEVICE_NUMBER sdn;
           DWORD dwBytesReturned = 0;
           res = DeviceIoControl(hDrive, IOCTL_STORAGE_GET_DEVICE_NUMBER, NULL, 0, &sdn, sizeof(sdn), &dwBytesReturned, NULL);
-          if ( res ) 
+          if ( res )
           {
-            if ( DiskNumber == (long)sdn.DeviceNumber ) 
+            if ( DiskNumber == (long)sdn.DeviceNumber )
             {
               CloseHandle(hDrive);
               SetupDiDestroyDeviceInfoList(hDevInfo);
@@ -659,7 +680,7 @@ bool CWIN32Util::EjectDrive(const char cDriveLetter)
   if( !cDriveLetter )
     return false;
 
-  CStdString strVolFormat; 
+  CStdString strVolFormat;
   strVolFormat.Format( _T("\\\\.\\%c:" ), cDriveLetter);
 
   long DiskNumber = -1;
@@ -752,55 +773,76 @@ BOOL CWIN32Util::IsCurrentUserLocalAdministrator()
 {
   BOOL b;
   SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
-  PSID AdministratorsGroup; 
+  PSID AdministratorsGroup;
   b = AllocateAndInitializeSid(
       &NtAuthority,
       2,
       SECURITY_BUILTIN_DOMAIN_RID,
       DOMAIN_ALIAS_RID_ADMINS,
       0, 0, 0, 0, 0, 0,
-      &AdministratorsGroup); 
+      &AdministratorsGroup);
   if(b)
   {
-    if (!CheckTokenMembership( NULL, AdministratorsGroup, &b)) 
+    if (!CheckTokenMembership( NULL, AdministratorsGroup, &b))
     {
          b = FALSE;
-    } 
-    FreeSid(AdministratorsGroup); 
+    }
+    FreeSid(AdministratorsGroup);
   }
 
   return(b);
 }
 
-void CWIN32Util::GetDrivesByType(VECSOURCES &localDrives, Drive_Types eDriveType)
+void CWIN32Util::GetDrivesByType(VECSOURCES &localDrives, Drive_Types eDriveType, bool bonlywithmedia)
 {
-  CMediaSource share;
   WCHAR* pcBuffer= NULL;
   DWORD dwStrLength= GetLogicalDriveStringsW( 0, pcBuffer );
   if( dwStrLength != 0 )
   {
+    CMediaSource share;
+
     dwStrLength+= 1;
     pcBuffer= new WCHAR [dwStrLength];
     GetLogicalDriveStringsW( dwStrLength, pcBuffer );
 
-    UINT uDriveType;
-    int iPos= 0, nResult;
+    int iPos= 0;
     WCHAR cVolumeName[100];
     do{
+      int nResult = 0;
       cVolumeName[0]= L'\0';
-      uDriveType= GetDriveTypeW( pcBuffer + iPos  );
-      if(uDriveType != DRIVE_REMOVABLE)
-        nResult= GetVolumeInformationW( pcBuffer + iPos, cVolumeName, 100, 0, 0, 0, NULL, 25);
+
+      CStdStringW strWdrive = pcBuffer + iPos;
+
+      UINT uDriveType= GetDriveTypeW( strWdrive.c_str()  );
+      // don't use GetVolumeInformation on fdd's as the floppy controller may be enabled in Bios but
+      // no floppy HW is attached which causes huge delays.
+      if(!strWdrive.Left(2).Equals(L"A:") && !strWdrive.Left(2).Equals(L"B:"))
+        nResult= GetVolumeInformationW( strWdrive.c_str() , cVolumeName, 100, 0, 0, 0, NULL, 25);
+      if(nResult == 0 && bonlywithmedia)
+      {
+        iPos += (wcslen( pcBuffer + iPos) + 1 );
+        continue;
+      }
+
+      // usb hard drives are reported as DRIVE_FIXED and won't be returned by queries with REMOVABLE_DRIVES set
+      // so test for usb hard drives
+      /*if(uDriveType == DRIVE_FIXED)
+      {
+        if(IsUsbDevice(strWdrive))
+          uDriveType = DRIVE_REMOVABLE;
+      }*/
+
       share.strPath= share.strName= "";
 
       bool bUseDCD= false;
-      if( uDriveType > DRIVE_UNKNOWN && 
+      if( uDriveType > DRIVE_UNKNOWN &&
         (( eDriveType == ALL_DRIVES && (uDriveType == DRIVE_FIXED || uDriveType == DRIVE_REMOTE || uDriveType == DRIVE_CDROM || uDriveType == DRIVE_REMOVABLE )) ||
          ( eDriveType == LOCAL_DRIVES && (uDriveType == DRIVE_FIXED || uDriveType == DRIVE_REMOTE)) ||
          ( eDriveType == REMOVABLE_DRIVES && ( uDriveType == DRIVE_REMOVABLE )) ||
          ( eDriveType == DVD_DRIVES && ( uDriveType == DRIVE_CDROM ))))
       {
-        share.strPath= pcBuffer + iPos;
+        //share.strPath = strWdrive;
+        g_charsetConverter.wToUTF8(strWdrive, share.strPath);
         if( cVolumeName[0] != L'\0' )
           g_charsetConverter.wToUTF8(cVolumeName, share.strName);
         if( uDriveType == DRIVE_CDROM && nResult)
@@ -847,21 +889,31 @@ void CWIN32Util::GetDrivesByType(VECSOURCES &localDrives, Drive_Types eDriveType
              CMediaSource::SOURCE_TYPE_UNKNOWN );
         }
 
-        localDrives.push_back(share);
+        AddOrReplace(localDrives, share);
       }
       iPos += (wcslen( pcBuffer + iPos) + 1 );
     } while( wcslen( pcBuffer + iPos ) > 0 );
-    if( pcBuffer != NULL)
-      delete[] pcBuffer;
+    delete[] pcBuffer;
   }
 }
 
+std::string CWIN32Util::GetFirstOpticalDrive()
+{
+  VECSOURCES vShare;
+  std::string strdevice = "\\\\.\\";
+  CWIN32Util::GetDrivesByType(vShare, DVD_DRIVES);
+  if(!vShare.empty())
+    return strdevice.append(vShare.front().strPath);
+  else
+    return "";
+}
+
 bool CWIN32Util::IsAudioCD(const CStdString& strPath)
 {
   CStdString strDrive = strPath;
   char cVolumenName[256];
   char cFSName[256];
-  CUtil::AddSlashAtEnd(strDrive);
+  URIUtils::AddSlashAtEnd(strDrive);
   if(GetVolumeInformation(strDrive.c_str(), cVolumenName, 255, NULL, NULL, NULL, cFSName, 255)==0)
     return false;
   return (strncmp(cFSName,"CDFS",4)==0 && strncmp(cVolumenName,"Audio CD",4)==0);
@@ -872,7 +924,7 @@ CStdString CWIN32Util::GetDiskLabel(const CStdString& strPath)
   CStdString strDrive = strPath;
   char cVolumenName[128];
   char cFSName[128];
-  CUtil::AddSlashAtEnd(strDrive);
+  URIUtils::AddSlashAtEnd(strDrive);
   if(GetVolumeInformation(strDrive.c_str(), cVolumenName, 127, NULL, NULL, NULL, cFSName, 127)==0)
     return "";
   return CStdString(cVolumenName).TrimRight(" ");
@@ -1287,3 +1339,260 @@ extern "C" {
     return NULL;
   }
 }
+
+
+bool CWIN32Util::Is64Bit()
+{
+  bool bRet= false;
+  typedef VOID (WINAPI *LPFN_GETNATIVESYSTEMINFO) (LPSYSTEM_INFO);
+  LPFN_GETNATIVESYSTEMINFO fnGetNativeSystemInfo= ( LPFN_GETNATIVESYSTEMINFO ) GetProcAddress( GetModuleHandle( TEXT( "kernel32" ) ), "GetNativeSystemInfo" );
+  SYSTEM_INFO sSysInfo;
+  memset( &sSysInfo, 0, sizeof( sSysInfo ) );
+  if (fnGetNativeSystemInfo != NULL)
+  {
+    fnGetNativeSystemInfo(&sSysInfo);
+    if (sSysInfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64) bRet= true;
+  }
+  return bRet;
+}
+
+LONG CWIN32Util::UtilRegGetValue( const HKEY hKey, const char *const pcKey, DWORD *const pdwType, char **const ppcBuffer, DWORD *const pdwSizeBuff, const DWORD dwSizeAdd )
+{
+  DWORD dwSize;
+  LONG lRet= RegQueryValueEx(hKey, pcKey, NULL, pdwType, NULL, &dwSize );
+  if (lRet == ERROR_SUCCESS)
+  {
+    if (ppcBuffer)
+    {
+      char *pcValue=*ppcBuffer;
+      if (!pcValue || !pdwSizeBuff || dwSize +dwSizeAdd > *pdwSizeBuff) pcValue= (char*)realloc(pcValue, dwSize +dwSizeAdd);
+      lRet= RegQueryValueEx(hKey,pcKey,NULL,NULL,(LPBYTE)pcValue,&dwSize);
+
+      if ( lRet == ERROR_SUCCESS || *ppcBuffer ) *ppcBuffer= pcValue;
+      else free( pcValue );
+    }
+    if (pdwSizeBuff) *pdwSizeBuff= dwSize +dwSizeAdd;
+  }
+  return lRet;
+}
+
+bool CWIN32Util::UtilRegOpenKeyEx( const HKEY hKeyParent, const char *const pcKey, const REGSAM rsAccessRights, HKEY *hKey, const bool bReadX64 )
+{
+  const REGSAM rsAccessRightsTmp= ( Is64Bit() ? rsAccessRights | ( bReadX64 ? KEY_WOW64_64KEY : KEY_WOW64_32KEY ) : rsAccessRights );
+  bool bRet= ( ERROR_SUCCESS == RegOpenKeyEx(hKeyParent, pcKey, 0, rsAccessRightsTmp, hKey));
+  return bRet;
+}
+
+bool CWIN32Util::GetCrystalHDLibraryPath(CStdString &strPath)
+{
+  // support finding library by windows registry
+  HKEY hKey;
+  CStdString strRegKey;
+
+  CLog::Log(LOGDEBUG, "CrystalHD: detecting CrystalHD installation path");
+  strRegKey.Format("%s\\%s", BC_REG_PATH, BC_REG_PRODUCT );
+
+  if( CWIN32Util::UtilRegOpenKeyEx( HKEY_LOCAL_MACHINE, strRegKey.c_str(), KEY_READ, &hKey ))
+  {
+    DWORD dwType;
+    char *pcPath= NULL;
+    if( CWIN32Util::UtilRegGetValue( hKey, BC_REG_INST_PATH, &dwType, &pcPath, NULL, sizeof( pcPath ) ) == ERROR_SUCCESS )
+    {
+      strPath = URIUtils::AddFileToFolder(pcPath, BC_BCM_DLL);
+      CLog::Log(LOGDEBUG, "CrystalHD: got CrystalHD installation path (%s)", strPath.c_str());
+      return true;
+    }
+    else
+    {
+      CLog::Log(LOGDEBUG, "CrystalHD: getting CrystalHD installation path failed");
+    }
+  }
+  else
+  {
+    CLog::Log(LOGDEBUG, "CrystalHD: CrystalHD software seems to be not installed.");
+  }
+  // check for dll in system dir
+  if(XFILE::CFile::Exists(DLL_PATH_LIBCRYSTALHD))
+    return true;
+  else
+    return false;
+}
+
+// Retrieve the filename of the process that currently has the focus.
+// Typically this will be some process using the system tray grabbing
+// the focus and causing XBMC to minimise. Logging the offending
+// process name can help the user fix the problem.
+bool CWIN32Util::GetFocussedProcess(CStdString &strProcessFile)
+{
+  strProcessFile = "";
+
+  // Get the window that has the focus
+  HWND hfocus = GetForegroundWindow();
+  if (!hfocus)
+    return false;
+
+  // Get the process ID from the window handle
+  DWORD pid = 0;
+  GetWindowThreadProcessId(hfocus, &pid);
+
+  // Use OpenProcess to get the process handle from the process ID
+  HANDLE hproc = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, 0, pid);
+  if (!hproc)
+    return false;
+
+  // Load QueryFullProcessImageName dynamically because it isn't available
+  // in all versions of Windows.
+  char procfile[MAX_PATH+1];
+  DWORD procfilelen = MAX_PATH;
+
+  HINSTANCE hkernel32 = LoadLibrary("kernel32.dll");
+  if (hkernel32)
+  {
+    DWORD (WINAPI *pQueryFullProcessImageNameA)(HANDLE,DWORD,LPTSTR,PDWORD);
+    pQueryFullProcessImageNameA = (DWORD (WINAPI *)(HANDLE,DWORD,LPTSTR,PDWORD)) GetProcAddress(hkernel32, "QueryFullProcessImageNameA");
+    if (pQueryFullProcessImageNameA)
+      if (pQueryFullProcessImageNameA(hproc, 0, procfile, &procfilelen))
+        strProcessFile = procfile;
+    FreeLibrary(hkernel32);
+  }
+
+  // If QueryFullProcessImageName failed fall back to GetModuleFileNameEx.
+  // Note this does not work across x86-x64 boundaries.
+  if (strProcessFile == "")
+  {
+    HINSTANCE hpsapi = LoadLibrary("psapi.dll");
+    if (hpsapi)
+    {
+      DWORD (WINAPI *pGetModuleFileNameExA)(HANDLE,HMODULE,LPTSTR,DWORD);
+      pGetModuleFileNameExA = (DWORD (WINAPI*)(HANDLE,HMODULE,LPTSTR,DWORD)) GetProcAddress(hpsapi, "GetModuleFileNameExA");
+      if (pGetModuleFileNameExA)
+        if (pGetModuleFileNameExA(hproc, NULL, procfile, MAX_PATH))
+          strProcessFile = procfile;
+      FreeLibrary(hpsapi);
+    }
+  }
+
+  CloseHandle(hproc);
+
+  return true;
+}
+
+// Adjust the src rectangle so that the dst is always contained in the target rectangle.
+void CWIN32Util::CropSource(CRect& src, CRect& dst, CRect target)
+{
+  if(dst.x1 < target.x1)
+  {
+    src.x1 -= (dst.x1 - target.x1)
+            * (src.x2 - src.x1)
+            / (dst.x2 - dst.x1);
+    dst.x1  = target.x1;
+  }
+  if(dst.y1 < target.y1)
+  {
+    src.y1 -= (dst.y1 - target.y1)
+            * (src.y2 - src.y1)
+            / (dst.y2 - dst.y1);
+    dst.y1  = target.y1;
+  }
+  if(dst.x2 > target.x2)
+  {
+    src.x2 -= (dst.x2 - target.x2)
+            * (src.x2 - src.x1)
+            / (dst.x2 - dst.x1);
+    dst.x2  = target.x2;
+  }
+  if(dst.y2 > target.y2)
+  {
+    src.y2 -= (dst.y2 - target.y2)
+            * (src.y2 - src.y1)
+            / (dst.y2 - dst.y1);
+    dst.y2  = target.y2;
+  }
+  // Callers expect integer coordinates.
+  src.x1 = floor(src.x1);
+  src.y1 = floor(src.y1);
+  src.x2 = ceil(src.x2);
+  src.y2 = ceil(src.y2);
+  dst.x1 = floor(dst.x1);
+  dst.y1 = floor(dst.y1);
+  dst.x2 = ceil(dst.x2);
+  dst.y2 = ceil(dst.y2);
+}
+
+void CWinIdleTimer::StartZero()
+{
+  SetThreadExecutionState(ES_SYSTEM_REQUIRED|ES_DISPLAY_REQUIRED);
+  CStopWatch::StartZero();
+}
+
+extern "C"
+{
+  /* case-independent string matching, similar to strstr but
+  * matching */
+  char * strcasestr(const char* haystack, const char* needle)
+  {
+    int i;
+    int nlength = (int) strlen (needle);
+    int hlength = (int) strlen (haystack);
+
+    if (nlength > hlength) return NULL;
+    if (hlength <= 0) return NULL;
+    if (nlength <= 0) return (char *)haystack;
+    /* hlength and nlength > 0, nlength <= hlength */
+    for (i = 0; i <= (hlength - nlength); i++)
+    {
+      if (strncasecmp (haystack + i, needle, nlength) == 0)
+      {
+        return (char *)haystack + i;
+      }
+    }
+    /* substring not found */
+    return NULL;
+  }
+}
+
+// detect if a drive is a usb device
+// code taken from http://banderlogi.blogspot.com/2011/06/enum-drive-letters-attached-for-usb.html
+
+bool CWIN32Util::IsUsbDevice(const CStdStringW &strWdrive)
+{
+  CStdStringW strWDevicePath;
+  strWDevicePath.Format(L"\\\\.\\%s",strWdrive.Left(2));
+
+  HANDLE deviceHandle = CreateFileW(
+    strWDevicePath.c_str(),
+   0,                // no access to the drive
+   FILE_SHARE_READ | // share mode
+   FILE_SHARE_WRITE,
+   NULL,             // default security attributes
+   OPEN_EXISTING,    // disposition
+   0,                // file attributes
+   NULL);            // do not copy file attributes
+
+  if(deviceHandle == INVALID_HANDLE_VALUE)
+    return false;
+
+  // setup query
+  STORAGE_PROPERTY_QUERY query;
+  memset(&query, 0, sizeof(query));
+  query.PropertyId = StorageDeviceProperty;
+  query.QueryType = PropertyStandardQuery;
+
+  // issue query
+  DWORD bytes;
+  STORAGE_DEVICE_DESCRIPTOR devd;
+  STORAGE_BUS_TYPE busType = BusTypeUnknown;
+
+  if (DeviceIoControl(deviceHandle,
+   IOCTL_STORAGE_QUERY_PROPERTY,
+   &query, sizeof(query),
+   &devd, sizeof(devd),
+   &bytes, NULL))
+  {
+   busType = devd.BusType;
+  }
+
+  CloseHandle(deviceHandle);
+
+  return BusTypeUsb == busType;
+ }
\ No newline at end of file
