@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,12 +35,11 @@
 #include "PartyModeManager.h"
 #include "dialogs/GUIDialogMediaSource.h"
 #include "GUIWindowFileManager.h"
-#include "filesystem/FavouritesDirectory.h"
+#include "Favourites.h"
 #include "utils/LabelFormatter.h"
 #include "dialogs/GUIDialogProgress.h"
-#include "profiles/ProfilesManager.h"
 #include "settings/AdvancedSettings.h"
-#include "settings/Settings.h"
+#include "settings/GUISettings.h"
 #include "URL.h"
 
 #include "dialogs/GUIDialogSmartPlaylistEditor.h"
@@ -50,9 +49,9 @@
 #include "dialogs/GUIDialogOK.h"
 #include "playlists/PlayList.h"
 #include "storage/MediaManager.h"
+#include "settings/Settings.h"
 #include "utils/StringUtils.h"
 #include "utils/URIUtils.h"
-#include "guilib/Key.h"
 #include "guilib/LocalizeStrings.h"
 #include "utils/TimeUtils.h"
 #include "filesystem/File.h"
@@ -61,15 +60,16 @@
 #include "utils/FileUtils.h"
 #include "guilib/GUIEditControl.h"
 #include "guilib/GUIKeyboardFactory.h"
+#ifdef HAS_PYTHON
+#include "interfaces/python/XBPython.h"
+#endif
 #include "interfaces/Builtins.h"
-#include "interfaces/generic/ScriptInvocationManager.h"
 #include "dialogs/GUIDialogKaiToast.h"
 #include "dialogs/GUIDialogMediaFilter.h"
 #include "filesystem/SmartPlaylistDirectory.h"
 #if defined(TARGET_ANDROID)
 #include "xbmc/android/activity/XBMCApp.h"
 #endif
-#include "FileItemListModification.h"
 
 #define CONTROL_BTNVIEWASICONS       2
 #define CONTROL_BTNSORTBY            3
@@ -118,11 +118,12 @@ void CGUIMediaWindow::LoadAdditionalTags(TiXmlElement *root)
   if (element && element->FirstChild())
   { // format is <views>50,29,51,95</views>
     CStdString allViews = element->FirstChild()->Value();
-    vector<string> views = StringUtils::Split(allViews, ",");
-    for (vector<string>::const_iterator i = views.begin(); i != views.end(); ++i)
+    CStdStringArray views;
+    StringUtils::SplitString(allViews, ",", views);
+    for (unsigned int i = 0; i < views.size(); i++)
     {
-      int controlID = atol(i->c_str());
-      CGUIControl *control = GetControl(controlID);
+      int controlID = atol(views[i].c_str());
+      CGUIControl *control = (CGUIControl *)GetControl(controlID);
       if (control && control->IsContainer())
         m_viewControl.AddView(control);
     }
@@ -203,7 +204,8 @@ bool CGUIMediaWindow::OnAction(const CAction &action)
 
   if (action.GetID() >= ACTION_FILTER_SMS2 && action.GetID() <= ACTION_FILTER_SMS9)
   {
-    CStdString filter = StringUtils::Format("%i", (int)(action.GetID() - ACTION_FILTER_SMS2 + 2));
+    CStdString filter;
+    filter.Format("%i", (int)(action.GetID() - ACTION_FILTER_SMS2 + 2));
     CGUIMessage message(GUI_MSG_NOTIFY_ALL, GetID(), 0, GUI_MSG_FILTER_ITEMS, 1); // 1 for append
     message.SetStringParam(filter);
     OnMessage(message);
@@ -291,7 +293,12 @@ bool CGUIMediaWindow::OnMessage(CGUIMessage& message)
         return true;
       }
       else if (iControl == CONTROL_BTN_FILTER)
-        return Filter(false);
+      {
+        if (m_canFilterAdvanced)
+          return true;
+
+        return Filter();
+      }
       else if (m_viewControl.HasControl(iControl))  // list/thumb control
       {
         int iItem = m_viewControl.GetSelectedItem();
@@ -395,7 +402,9 @@ bool CGUIMediaWindow::OnMessage(CGUIMessage& message)
         else if (newItem)
         { // need to remove the disc cache
           CFileItemList items;
-          items.SetPath(URIUtils::GetDirectory(newItem->GetPath()));
+          CStdString path;
+          URIUtils::GetDirectory(newItem->GetPath(), path);
+          items.SetPath(path);
           items.RemoveDiscCache(GetID());
         }
       }
@@ -410,16 +419,17 @@ bool CGUIMediaWindow::OnMessage(CGUIMessage& message)
       }
       else if (message.GetParam1() == GUI_MSG_FILTER_ITEMS && IsActive())
       {
-        CStdString filter = GetProperty("filter").asString();
+        CStdString filter;
         // check if this is meant for advanced filtering
         if (message.GetParam2() != 10)
         {
+          filter = GetProperty("filter").asString();
           if (message.GetParam2() == 1) // append
             filter += message.GetStringParam();
           else if (message.GetParam2() == 2)
           { // delete
             if (filter.size())
-              filter.erase(filter.size() - 1);
+              filter = filter.Left(filter.size() - 1);
           }
           else
             filter = message.GetStringParam();
@@ -485,26 +495,21 @@ bool CGUIMediaWindow::OnMessage(CGUIMessage& message)
       if (m_vecItems->GetPath() == "?")
         m_vecItems->SetPath("");
       CStdString dir = message.GetStringParam(0);
-      const std::string &ret = message.GetStringParam(1);
-      bool returning = StringUtils::EqualsNoCase(ret, "return");
-      if (!dir.empty())
+      const CStdString &ret = message.GetStringParam(1);
+      bool returning = ret.CompareNoCase("return") == 0;
+      if (!dir.IsEmpty())
       {
+        m_history.ClearPathHistory();
         // ensure our directory is valid
         dir = GetStartFolder(dir);
-        bool resetHistory = false;
-        if (!returning || !m_history.IsInHistory(dir))
+        if (!returning || m_vecItems->GetPath().Left(dir.GetLength()) != dir)
         { // we're not returning to the same path, so set our directory to the requested path
           m_vecItems->SetPath(dir);
-          resetHistory = true;
         }
         // check for network up
         if (URIUtils::IsRemote(m_vecItems->GetPath()) && !WaitForNetwork())
-        {
           m_vecItems->SetPath("");
-          resetHistory = true;
-        }
-        if (resetHistory)
-          SetHistoryForPath(m_vecItems->GetPath());
+        SetHistoryForPath(m_vecItems->GetPath());
       }
       if (message.GetParam1() != WINDOW_INVALID)
       { // first time to this window - make sure we set the root path
@@ -532,27 +537,41 @@ void CGUIMediaWindow::UpdateButtons()
     else
     {
       CONTROL_ENABLE(CONTROL_BTNSORTASC);
-      SET_CONTROL_SELECTED(GetID(), CONTROL_BTNSORTASC, m_guiState->GetDisplaySortOrder() != SortOrderAscending);
+      if (m_guiState->GetDisplaySortOrder() == SortOrderAscending)
+      {
+        CGUIMessage msg(GUI_MSG_DESELECTED, GetID(), CONTROL_BTNSORTASC);
+        g_windowManager.SendMessage(msg);
+      }
+      else
+      {
+        CGUIMessage msg(GUI_MSG_SELECTED, GetID(), CONTROL_BTNSORTASC);
+        g_windowManager.SendMessage(msg);
+      }
     }
 
     // Update list/thumb control
     m_viewControl.SetCurrentView(m_guiState->GetViewAsControl());
 
     // Update sort by button
-    if (!m_guiState->HasMultipleSortMethods())
+    if (m_guiState->GetSortMethod()==SORT_METHOD_NONE)
+    {
       CONTROL_DISABLE(CONTROL_BTNSORTBY);
+    }
     else
+    {
       CONTROL_ENABLE(CONTROL_BTNSORTBY);
-
-    CStdString sortLabel = StringUtils::Format(g_localizeStrings.Get(550).c_str(),
-                                               g_localizeStrings.Get(m_guiState->GetSortMethodLabel()).c_str());
+    }
+    CStdString sortLabel;
+    sortLabel.Format(g_localizeStrings.Get(550).c_str(), g_localizeStrings.Get(m_guiState->GetSortMethodLabel()).c_str());
     SET_CONTROL_LABEL(CONTROL_BTNSORTBY, sortLabel);
   }
 
-  CStdString items = StringUtils::Format("%i %s", m_vecItems->GetObjectCount(), g_localizeStrings.Get(127).c_str());
+  CStdString items;
+  items.Format("%i %s", m_vecItems->GetObjectCount(), g_localizeStrings.Get(127).c_str());
   SET_CONTROL_LABEL(CONTROL_LABELFILES, items);
 
-  SET_CONTROL_LABEL2(CONTROL_BTN_FILTER, GetProperty("filter").asString());
+  if (!m_canFilterAdvanced)
+    SET_CONTROL_LABEL2(CONTROL_BTN_FILTER, GetProperty("filter").asString());
 }
 
 void CGUIMediaWindow::ClearFileItems()
@@ -569,30 +588,35 @@ void CGUIMediaWindow::SortItems(CFileItemList &items)
 
   if (guiState.get())
   {
-    SortDescription sorting = guiState->GetSortMethod();
-    sorting.sortOrder = guiState->GetDisplaySortOrder();
+    bool sorted = false;
+    SORT_METHOD sortMethod = guiState->GetSortMethod();
     // If the sort method is "sort by playlist" and we have a specific
     // sort order available we can use the specified sort order to do the sorting
     // We do this as the new SortBy methods are a superset of the SORT_METHOD methods, thus
     // not all are available. This may be removed once SORT_METHOD_* have been replaced by
     // SortBy.
-    if ((sorting.sortBy == SortByPlaylistOrder) && items.HasProperty(PROPERTY_SORT_ORDER))
+    if ((sortMethod == SORT_METHOD_PLAYLIST_ORDER) && items.HasProperty(PROPERTY_SORT_ORDER))
     {
       SortBy sortBy = (SortBy)items.GetProperty(PROPERTY_SORT_ORDER).asInteger();
       if (sortBy != SortByNone && sortBy != SortByPlaylistOrder && sortBy != SortByProgramCount)
       {
+        SortDescription sorting;
         sorting.sortBy = sortBy;
         sorting.sortOrder = items.GetProperty(PROPERTY_SORT_ASCENDING).asBoolean() ? SortOrderAscending : SortOrderDescending;
-        sorting.sortAttributes = CSettings::Get().GetBool("filelists.ignorethewhensorting") ? SortAttributeIgnoreArticle : SortAttributeNone;
+        sorting.sortAttributes = g_guiSettings.GetBool("filelists.ignorethewhensorting") ? SortAttributeIgnoreArticle : SortAttributeNone;
 
         // if the sort order is descending, we need to switch the original sort order, as we assume
-        // in CGUIViewState::AddPlaylistOrder that SortByPlaylistOrder is ascending.
+        // in CGUIViewState::AddPlaylistOrder that SORT_METHOD_PLAYLIST_ORDER is ascending.
         if (guiState->GetDisplaySortOrder() == SortOrderDescending)
           sorting.sortOrder = sorting.sortOrder == SortOrderDescending ? SortOrderAscending : SortOrderDescending;
+
+        items.Sort(sorting);
+        sorted = true;
       }
     }
 
-    items.Sort(sorting);
+    if (!sorted)
+      items.Sort(sortMethod, guiState->GetDisplaySortOrder());
   }
 }
 
@@ -614,7 +638,8 @@ void CGUIMediaWindow::FormatItemLabels(CFileItemList &items, const LABEL_MASKS &
       fileFormatter.FormatLabels(pItem.get());
   }
 
-  if (items.GetSortMethod() == SortByLabel)
+  if(items.GetSortMethod() == SORT_METHOD_LABEL_IGNORE_THE
+  || items.GetSortMethod() == SORT_METHOD_LABEL)
     items.ClearSortState();
 }
 
@@ -629,7 +654,7 @@ void CGUIMediaWindow::FormatAndSort(CFileItemList &items)
     viewState->GetSortMethodLabelMasks(labelMasks);
     FormatItemLabels(items, labelMasks);
 
-    items.Sort(viewState->GetSortMethod().sortBy, viewState->GetDisplaySortOrder(), viewState->GetSortMethod().sortAttributes);
+    items.Sort(viewState->GetSortMethod(), viewState->GetDisplaySortOrder());
   }
 }
 
@@ -640,21 +665,18 @@ void CGUIMediaWindow::FormatAndSort(CFileItemList &items)
   */
 bool CGUIMediaWindow::GetDirectory(const CStdString &strDirectory, CFileItemList &items)
 {
-  const CURL pathToUrl(strDirectory);
-
   // cleanup items
   if (items.Size())
     items.Clear();
 
   CStdString strParentPath = m_history.GetParentPath();
 
-  CLog::Log(LOGDEBUG,"CGUIMediaWindow::GetDirectory (%s)",
-            CURL::GetRedacted(strDirectory).c_str());
-  CLog::Log(LOGDEBUG,"  ParentPath = [%s]", CURL::GetRedacted(strParentPath).c_str());
+  CLog::Log(LOGDEBUG,"CGUIMediaWindow::GetDirectory (%s)", strDirectory.c_str());
+  CLog::Log(LOGDEBUG,"  ParentPath = [%s]", strParentPath.c_str());
 
   // see if we can load a previously cached folder
   CFileItemList cachedItems(strDirectory);
-  if (!strDirectory.empty() && cachedItems.Load(GetID()))
+  if (!strDirectory.IsEmpty() && cachedItems.Load(GetID()))
   {
     items.Assign(cachedItems);
   }
@@ -662,10 +684,10 @@ bool CGUIMediaWindow::GetDirectory(const CStdString &strDirectory, CFileItemList
   {
     unsigned int time = XbmcThreads::SystemClockMillis();
 
-    if (strDirectory.empty())
+    if (strDirectory.IsEmpty())
       SetupShares();
 
-    if (!m_rootDir.GetDirectory(pathToUrl, items))
+    if (!m_rootDir.GetDirectory(strDirectory, items))
       return false;
 
     // took over a second, and not normally cached, so cache it
@@ -677,7 +699,7 @@ bool CGUIMediaWindow::GetDirectory(const CStdString &strDirectory, CFileItemList
       m_history.RemoveParentPath();
   }
 
-  if (m_guiState.get() && !m_guiState->HideParentDirItems() && !items.GetPath().empty())
+  if (m_guiState.get() && !m_guiState->HideParentDirItems() && !items.GetPath().IsEmpty())
   {
     CFileItemPtr pItem(new CFileItem(".."));
     pItem->SetPath(strParentPath);
@@ -687,7 +709,7 @@ bool CGUIMediaWindow::GetDirectory(const CStdString &strDirectory, CFileItemList
   }
 
   int iWindow = GetID();
-  vector<string> regexps;
+  CStdStringArray regexps;
 
   // TODO: Do we want to limit the directories we apply the video ones to?
   if (iWindow == WINDOW_VIDEO_NAV)
@@ -746,56 +768,87 @@ bool CGUIMediaWindow::Update(const CStdString &strDirectory, bool updateFilterPa
   CStdString filter;
   CURL url(directory);
   if (canfilter && url.HasOption("filter"))
+  {
+    filter = url.GetOption("filter");
     directory = RemoveParameterFromPath(directory, "filter");
+  }
 
   CFileItemList items;
   if (!GetDirectory(directory, items))
   {
-    CLog::Log(LOGERROR,"CGUIMediaWindow::GetDirectory(%s) failed", url.GetRedacted().c_str());
-    // Try to return to the previous directory, if not the same
-    // else fallback to root
-    if (strDirectory.Equals(strCurrentDirectory) || !Update(m_history.RemoveParentPath()))
-      Update(""); // Fallback to root
+    CLog::Log(LOGERROR,"CGUIMediaWindow::GetDirectory(%s) failed", strDirectory.c_str());
+    // if the directory is the same as the old directory, then we'll return
+    // false.  Else, we assume we can get the previous directory
+    if (strDirectory.Equals(strCurrentDirectory))
+      return false;
 
-    // Return false to be able to eg. show
+    // We assume, we can get the parent
+    // directory again, but we have to
+    // return false to be able to eg. show
     // an error message.
+    Update(m_history.RemoveParentPath());
     return false;
   }
 
-  if (items.GetLabel().empty())
+  if (items.GetLabel().IsEmpty())
     items.SetLabel(CUtil::GetTitleFromPath(items.GetPath(), true));
   
   ClearFileItems();
   m_vecItems->Copy(items);
 
-  // check the given path for filter data
-  UpdateFilterPath(strDirectory, items, updateFilterPath);
+  // only set the filter path if it hasn't been marked
+  // as preset or if it's empty
+  if (updateFilterPath || m_strFilterPath.empty())
+  {
+    if (items.HasProperty(PROPERTY_PATH_DB))
+      m_strFilterPath = items.GetProperty(PROPERTY_PATH_DB).asString();
+    else
+      m_strFilterPath = items.GetPath();
+  }
+  
+  // maybe the filter path can contain a filter
+  if (!canfilter && CanContainFilter(m_strFilterPath))
+    canfilter = true;
+
+  // check if the filter path contains a filter
+  CURL filterPathUrl(m_strFilterPath);
+  if (canfilter && filter.empty())
+  {
+    if (filterPathUrl.HasOption("filter"))
+      filter = filterPathUrl.GetOption("filter");
+  }
+
+  // check if there is a filter and re-apply it
+  if (canfilter && !filter.empty())
+  {
+    if (!m_filter.LoadFromJson(filter))
+    {
+      CLog::Log(LOGWARNING, "CGUIMediaWindow::Update: unable to load existing filter (%s)", filter.c_str());
+      m_filter.Reset();
+      m_strFilterPath = m_vecItems->GetPath();
+    }
+    else
+    {
+      // add the filter to the filter path
+      filterPathUrl.SetOption("filter", filter);
+      m_strFilterPath = filterPathUrl.Get();
+    }
+  }
     
   // if we're getting the root source listing
   // make sure the path history is clean
-  if (strDirectory.empty())
+  if (strDirectory.IsEmpty())
     m_history.ClearPathHistory();
 
   int iWindow = GetID();
   int showLabel = 0;
-  if (strDirectory.empty())
-  {
-    if (iWindow == WINDOW_PICTURES)
-      showLabel = 997;
-    else if (iWindow == WINDOW_MUSIC_FILES)
-      showLabel = 998;
-    else if (iWindow == WINDOW_FILES || iWindow == WINDOW_PROGRAMS)
-      showLabel = 1026;
-  }
-  if (m_vecItems->GetPath().Equals("sources://video/"))
-    showLabel = 999;
-  else if (m_vecItems->GetPath().Equals("sources://music/"))
-    showLabel = 998;
-  else if (m_vecItems->GetPath().Equals("sources://pictures/"))
-    showLabel = 997;
-  else if (m_vecItems->GetPath().Equals("sources://programs/") ||
-           m_vecItems->GetPath().Equals("sources://files/"))
+  if (strDirectory.IsEmpty() && (iWindow == WINDOW_MUSIC_FILES ||
+                                 iWindow == WINDOW_FILES ||
+                                 iWindow == WINDOW_PICTURES ||
+                                 iWindow == WINDOW_PROGRAMS))
     showLabel = 1026;
+  if (strDirectory.Equals("sources://video/"))
+    showLabel = 999;
   if (showLabel && (m_vecItems->Size() == 0 || !m_guiState->DisableAddSourceButtons())) // add 'add source button'
   {
     CStdString strLabel = g_localizeStrings.Get(showLabel);
@@ -832,7 +885,8 @@ bool CGUIMediaWindow::Update(const CStdString &strDirectory, bool updateFilterPa
   OnCacheFileItems(*m_vecItems);
 
   // Filter and group the items if necessary
-  OnFilterItems(GetProperty("filter").asString());
+  CStdString titleFilter = GetProperty("filter").asString();
+  OnFilterItems(titleFilter);
 
   // Ask the devived class if it wants to do custom list operations,
   // eg. changing the label
@@ -862,8 +916,7 @@ bool CGUIMediaWindow::Update(const CStdString &strDirectory, bool updateFilterPa
   if (!bSelectedFound)
     m_viewControl.SetSelectedItem(0);
 
-  if (iWindow != WINDOW_PVR ||
-      (iWindow == WINDOW_PVR && StringUtils::StartsWith(m_vecItems->GetPath(), "pvr://recordings/")))
+  if (iWindow != WINDOW_PVR || (iWindow == WINDOW_PVR && m_vecItems->GetPath().Left(17) == "pvr://recordings/"))
     m_history.AddPath(m_vecItems->GetPath(), m_strFilterPath);
 
   //m_history.DumpPathHistory();
@@ -893,7 +946,7 @@ bool CGUIMediaWindow::Refresh(bool clearCache /* = false */)
 // It's used to load tag info for music.
 void CGUIMediaWindow::OnPrepareFileItems(CFileItemList &items)
 {
-  CFileItemListModification::Get().Modify(items);
+
 }
 
 // \brief This function will be called by Update() before
@@ -933,10 +986,10 @@ bool CGUIMediaWindow::OnClick(int iItem)
     return true;
   }
 
-  if (!pItem->m_bIsFolder && pItem->IsFileFolder(EFILEFOLDER_MASK_ONCLICK))
+  if (!pItem->m_bIsFolder && pItem->IsFileFolder())
   {
     XFILE::IFileDirectory *pFileDirectory = NULL;
-    pFileDirectory = XFILE::CFileDirectoryFactory::Create(pItem->GetURL(), pItem.get(), "");
+    pFileDirectory = XFILE::CFileDirectoryFactory::Create(pItem->GetPath(), pItem.get(), "");
     if(pFileDirectory)
       pItem->m_bIsFolder = true;
     else if(pItem->m_bIsFolder)
@@ -951,8 +1004,10 @@ bool CGUIMediaWindow::OnClick(int iItem)
     AddonPtr addon;
     if (CAddonMgr::Get().GetAddon(url.GetHostName(), addon, ADDON_SCRIPT))
     {
-      if (!CScriptInvocationManager::Get().Stop(addon->LibPath()))
-        CScriptInvocationManager::Get().Execute(addon->LibPath(), addon);
+#ifdef HAS_PYTHON
+      if (!g_pythonParser.StopScript(addon->LibPath()))
+        g_pythonParser.evalFile(addon->LibPath(),addon);
+#endif
       return true;
     }
   }
@@ -962,8 +1017,8 @@ bool CGUIMediaWindow::OnClick(int iItem)
     if ( pItem->m_bIsShareOrDrive )
     {
       const CStdString& strLockType=m_guiState->GetLockType();
-      if (CProfilesManager::Get().GetMasterProfile().getLockMode() != LOCK_MODE_EVERYONE)
-        if (!strLockType.empty() && !g_passwordManager.IsItemUnlocked(pItem.get(), strLockType))
+      if (g_settings.GetMasterProfile().getLockMode() != LOCK_MODE_EVERYONE)
+        if (!strLockType.IsEmpty() && !g_passwordManager.IsItemUnlocked(pItem.get(), strLockType))
             return true;
 
       if (!HaveDiscOrConnection(pItem->GetPath(), pItem->m_iDriveType))
@@ -971,8 +1026,8 @@ bool CGUIMediaWindow::OnClick(int iItem)
     }
 
     // check for the partymode playlist items - they may not exist yet
-    if ((pItem->GetPath() == CProfilesManager::Get().GetUserDataItem("PartyMode.xsp")) ||
-        (pItem->GetPath() == CProfilesManager::Get().GetUserDataItem("PartyMode-Video.xsp")))
+    if ((pItem->GetPath() == g_settings.GetUserDataItem("PartyMode.xsp")) ||
+        (pItem->GetPath() == g_settings.GetUserDataItem("PartyMode-Video.xsp")))
     {
       // party mode playlist item - if it doesn't exist, prompt for user to define it
       if (!XFILE::CFile::Exists(pItem->GetPath()))
@@ -1027,22 +1082,22 @@ bool CGUIMediaWindow::OnClick(int iItem)
       g_windowManager.ActivateWindow(WINDOW_MUSIC_PLAYLIST_EDITOR,"newplaylist://");
       return true;
     }
-    else if (StringUtils::StartsWithNoCase(pItem->GetPath(), "newsmartplaylist://"))
+    else if (pItem->GetPath().Left(19).Equals("newsmartplaylist://"))
     {
       m_vecItems->RemoveDiscCache(GetID());
-      if (CGUIDialogSmartPlaylistEditor::NewPlaylist(pItem->GetPath().substr(19)))
+      if (CGUIDialogSmartPlaylistEditor::NewPlaylist(pItem->GetPath().Mid(19)))
         Refresh();
       return true;
     }
-    else if (StringUtils::StartsWithNoCase(pItem->GetPath(), "addons://more/"))
+    else if (pItem->GetPath().Left(14).Equals("addons://more/"))
     {
-      CBuiltins::Execute("ActivateWindow(AddonBrowser,addons://all/xbmc.addon." + pItem->GetPath().substr(14) + ",return)");
+      CBuiltins::Execute("ActivateWindow(AddonBrowser,addons://all/xbmc.addon." + pItem->GetPath().Mid(14) + ",return)");
       return true;
     }
 
     // If karaoke song is being played AND popup autoselector is enabled, the playlist should not be added
-    bool do_not_add_karaoke = CSettings::Get().GetBool("karaoke.enabled") &&
-      CSettings::Get().GetBool("karaoke.autopopupselector") && pItem->IsKaraoke();
+    bool do_not_add_karaoke = g_guiSettings.GetBool("karaoke.enabled") &&
+      g_guiSettings.GetBool("karaoke.autopopupselector") && pItem->IsKaraoke();
     bool autoplay = m_guiState.get() && m_guiState->AutoPlayNextItem();
 
     if (m_vecItems->IsPlugin())
@@ -1108,21 +1163,21 @@ bool CGUIMediaWindow::HaveDiscOrConnection(const CStdString& strPath, int iDrive
 // \brief Shows a standard errormessage for a given pItem.
 void CGUIMediaWindow::ShowShareErrorMessage(CFileItem* pItem)
 {
-  if (!pItem->m_bIsShareOrDrive)
-    return;
+  if (pItem->m_bIsShareOrDrive)
+  {
+    int idMessageText=0;
+    const CURL& url=pItem->GetAsUrl();
+    const CStdString& strHostName=url.GetHostName();
 
-  int idMessageText = 0;
-  CURL url(pItem->GetPath());
-  const CStdString& strHostName = url.GetHostName();
+    if (pItem->m_iDriveType != CMediaSource::SOURCE_TYPE_REMOTE) //  Local shares incl. dvd drive
+      idMessageText=15300;
+    else if (url.GetProtocol() == "smb" && strHostName.IsEmpty()) //  smb workgroup
+      idMessageText=15303;
+    else  //  All other remote shares
+      idMessageText=15301;
 
-  if (url.GetProtocol() == "smb" && strHostName.empty()) //  smb workgroup
-    idMessageText = 15303; // Workgroup not found
-  else if (pItem->m_iDriveType == CMediaSource::SOURCE_TYPE_REMOTE || URIUtils::IsRemote(pItem->GetPath()))
-    idMessageText = 15301; // Could not connect to network server
-  else
-    idMessageText = 15300; // Path not found or invalid
-
-  CGUIDialogOK::ShowAndGetInput(220, idMessageText, 0, 0);
+    CGUIDialogOK::ShowAndGetInput(220, idMessageText, 0, 0);
+  }
 }
 
 // \brief The functon goes up one level in the directory tree
@@ -1139,7 +1194,7 @@ void CGUIMediaWindow::GoParentFolder()
   // in case the path history is messed up and the current folder is on
   // the stack more than once, keep going until there's nothing left or they
   // dont match anymore.
-  while (!strParent.empty())
+  while (!strParent.IsEmpty())
   {
     URIUtils::AddSlashAtEnd(strParent);
     if (strParent.Equals(strPath))
@@ -1196,13 +1251,11 @@ void CGUIMediaWindow::GetDirectoryHistoryString(const CFileItem* pItem, CStdStri
       // and use as history string, m_strPath
       // can change for new discs
       CStdString strLabel = pItem->GetLabel();
-      size_t nPosOpen = strLabel.find('(');
-      size_t nPosClose = strLabel.rfind(')');
-      if (nPosOpen != std::string::npos &&
-          nPosClose != std::string::npos &&
-          nPosClose > nPosOpen)
+      int nPosOpen = strLabel.Find('(');
+      int nPosClose = strLabel.ReverseFind(')');
+      if (nPosOpen > -1 && nPosClose > -1 && nPosClose > nPosOpen)
       {
-        strLabel.erase(nPosOpen + 1, (nPosClose) - (nPosOpen + 1));
+        strLabel.Delete(nPosOpen + 1, (nPosClose) - (nPosOpen + 1));
         strHistoryString = strLabel;
       }
       else
@@ -1221,9 +1274,7 @@ void CGUIMediaWindow::GetDirectoryHistoryString(const CFileItem* pItem, CStdStri
   {
     // Could be a cue item, all items of a cue share the same filename
     // so add the offsets to build the history string
-    strHistoryString = StringUtils::Format("%ld%ld",
-                                           pItem->m_lStartOffset,
-                                           pItem->m_lEndOffset);
+    strHistoryString.Format("%ld%ld", pItem->m_lStartOffset, pItem->m_lEndOffset);
     strHistoryString += pItem->GetPath();
   }
   else
@@ -1237,7 +1288,7 @@ void CGUIMediaWindow::GetDirectoryHistoryString(const CFileItem* pItem, CStdStri
     strHistoryString = RemoveParameterFromPath(strHistoryString, "filter");
 
   URIUtils::RemoveSlashAtEnd(strHistoryString);
-  StringUtils::ToLower(strHistoryString);
+  strHistoryString.ToLower();
 }
 
 // \brief Call this function to create a directory history for the
@@ -1246,7 +1297,7 @@ void CGUIMediaWindow::SetHistoryForPath(const CStdString& strDirectory)
 {
   // Make sure our shares are configured
   SetupShares();
-  if (!strDirectory.empty())
+  if (!strDirectory.IsEmpty())
   {
     // Build the directory history for default path
     CStdString strPath, strParentPath;
@@ -1254,7 +1305,7 @@ void CGUIMediaWindow::SetHistoryForPath(const CStdString& strDirectory)
     URIUtils::RemoveSlashAtEnd(strPath);
 
     CFileItemList items;
-    m_rootDir.GetDirectory(CURL(), items);
+    m_rootDir.GetDirectory("", items);
 
     m_history.ClearPathHistory();
 
@@ -1310,13 +1361,13 @@ bool CGUIMediaWindow::OnPlayMedia(int iItem)
   g_playlistPlayer.SetCurrentPlaylist(PLAYLIST_NONE);
   CFileItemPtr pItem=m_vecItems->Get(iItem);
 
-  CLog::Log(LOGDEBUG, "%s %s", __FUNCTION__, CURL::GetRedacted(pItem->GetPath()).c_str());
+  CLog::Log(LOGDEBUG, "%s %s", __FUNCTION__, pItem->GetPath().c_str());
 
   bool bResult = false;
   if (pItem->IsInternetStream() || pItem->IsPlayList())
     bResult = g_application.PlayMedia(*pItem, m_guiState->GetPlaylist());
   else
-    bResult = g_application.PlayFile(*pItem) == PLAYBACK_OK;
+    bResult = g_application.PlayFile(*pItem);
 
   if (pItem->m_lStartOffset == STARTOFFSET_RESUME)
     pItem->m_lStartOffset = 0;
@@ -1336,21 +1387,6 @@ bool CGUIMediaWindow::OnPlayAndQueueMedia(const CFileItemPtr &item)
     g_playlistPlayer.ClearPlaylist(iPlaylist);
     g_playlistPlayer.Reset();
     int mediaToPlay = 0;
-    
-    // first try to find mainDVD file (VIDEO_TS.IFO). 
-    // If we find this we should not allow to queue VOB files
-    std::string mainDVD; 
-    for (int i = 0; i < m_vecItems->Size(); i++) 
-    { 
-      std::string path = URIUtils::GetFileName(m_vecItems->Get(i)->GetPath()); 
-      if (StringUtils::EqualsNoCase(path, "VIDEO_TS.IFO")) 
-      { 
-        mainDVD = path; 
-        break; 
-      } 
-    }
-
-    // now queue...
     for ( int i = 0; i < m_vecItems->Size(); i++ )
     {
       CFileItemPtr nItem = m_vecItems->Get(i);
@@ -1358,7 +1394,7 @@ bool CGUIMediaWindow::OnPlayAndQueueMedia(const CFileItemPtr &item)
       if (nItem->m_bIsFolder)
         continue;
 
-      if (!nItem->IsPlayList() && !nItem->IsZIP() && !nItem->IsRAR() && (!nItem->IsDVDFile() || (URIUtils::GetFileName(nItem->GetPath()) == mainDVD)))
+      if (!nItem->IsPlayList() && !nItem->IsZIP() && !nItem->IsRAR())
         g_playlistPlayer.Add(iPlaylist, nItem);
 
       if (item->IsSamePath(nItem.get()))
@@ -1438,7 +1474,7 @@ void CGUIMediaWindow::OnDeleteItem(int iItem)
   if (item->IsPlayList())
     item->m_bIsFolder = false;
 
-  if (CProfilesManager::Get().GetCurrentProfile().getLockMode() != LOCK_MODE_EVERYONE && CProfilesManager::Get().GetCurrentProfile().filesLocked())
+  if (g_settings.GetCurrentProfile().getLockMode() != LOCK_MODE_EVERYONE && g_settings.GetCurrentProfile().filesLocked())
     if (!g_passwordManager.IsMasterLockUnlocked(true))
       return;
 
@@ -1452,7 +1488,7 @@ void CGUIMediaWindow::OnRenameItem(int iItem)
 {
   if ( iItem < 0 || iItem >= m_vecItems->Size()) return;
 
-  if (CProfilesManager::Get().GetCurrentProfile().getLockMode() != LOCK_MODE_EVERYONE && CProfilesManager::Get().GetCurrentProfile().filesLocked())
+  if (g_settings.GetCurrentProfile().getLockMode() != LOCK_MODE_EVERYONE && g_settings.GetCurrentProfile().filesLocked())
     if (!g_passwordManager.IsMasterLockUnlocked(true))
       return;
 
@@ -1466,13 +1502,7 @@ void CGUIMediaWindow::OnInitWindow()
 {
   // initial fetch is done unthreaded to ensure the items are setup prior to skin animations kicking off
   m_rootDir.SetAllowThreads(false);
-
-  // the start directory may change during Refresh
-  bool updateStartDirectory = (m_startDirectory == m_vecItems->GetPath());
   Refresh();
-  if (updateStartDirectory)
-    m_startDirectory = m_vecItems->GetPath();
-
   m_rootDir.SetAllowThreads(true);
 
   if (m_iSelectedItem > -1)
@@ -1538,11 +1568,11 @@ void CGUIMediaWindow::GetContextButtons(int itemNumber, CContextButtons &buttons
   CStdString action;
   for (int i = CONTEXT_BUTTON_USER1; i <= CONTEXT_BUTTON_USER10; i++)
   {
-    label = StringUtils::Format("contextmenulabel(%i)", i - CONTEXT_BUTTON_USER1);
+    label.Format("contextmenulabel(%i)", i - CONTEXT_BUTTON_USER1);
     if (item->GetProperty(label).empty())
       break;
 
-    action = StringUtils::Format("contextmenuaction(%i)", i - CONTEXT_BUTTON_USER1);
+    action.Format("contextmenuaction(%i)", i - CONTEXT_BUTTON_USER1);
     if (item->GetProperty(action).empty())
       break;
 
@@ -1554,18 +1584,13 @@ void CGUIMediaWindow::GetContextButtons(int itemNumber, CContextButtons &buttons
 
   // TODO: FAVOURITES Conditions on masterlock and localisation
   if (!item->IsParentFolder() && !item->GetPath().Equals("add") && !item->GetPath().Equals("newplaylist://") &&
-      !StringUtils::StartsWithNoCase(item->GetPath(), "newsmartplaylist://") && !StringUtils::StartsWithNoCase(item->GetPath(), "newtag://") &&
-      !StringUtils::StartsWithNoCase(item->GetPath(), "addons://more/") && !StringUtils::StartsWithNoCase(item->GetPath(), "musicsearch://"))
+      !item->GetPath().Left(19).Equals("newsmartplaylist://") && !item->GetPath().Left(9).Equals("newtag://"))
   {
-    if (XFILE::CFavouritesDirectory::IsFavourite(item.get(), GetID()))
+    if (CFavourites::IsFavourite(item.get(), GetID()))
       buttons.Add(CONTEXT_BUTTON_ADD_FAVOURITE, 14077);     // Remove Favourite
     else
       buttons.Add(CONTEXT_BUTTON_ADD_FAVOURITE, 14076);     // Add To Favourites;
   }
-
-  if (item->IsFileFolder(EFILEFOLDER_MASK_ONBROWSE))
-    buttons.Add(CONTEXT_BUTTON_BROWSE_INTO, 37015);
-
 }
 
 bool CGUIMediaWindow::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
@@ -1575,29 +1600,18 @@ bool CGUIMediaWindow::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
   case CONTEXT_BUTTON_ADD_FAVOURITE:
     {
       CFileItemPtr item = m_vecItems->Get(itemNumber);
-      XFILE::CFavouritesDirectory::AddOrRemove(item.get(), GetID());
+      CFavourites::AddOrRemove(item.get(), GetID());
       return true;
     }
   case CONTEXT_BUTTON_PLUGIN_SETTINGS:
     {
-      CFileItemPtr item = m_vecItems->Get(itemNumber);
-      // CONTEXT_BUTTON_PLUGIN_SETTINGS can be called for plugin item
-      // or script item; or for the plugin directory current listing.
-      bool isPluginOrScriptItem = (item && (item->IsPlugin() || item->IsScript()));
-      CURL plugin(isPluginOrScriptItem ? item->GetPath() : m_vecItems->GetPath());
+      CURL plugin(m_vecItems->Get(itemNumber)->GetPath());
       ADDON::AddonPtr addon;
       if (CAddonMgr::Get().GetAddon(plugin.GetHostName(), addon))
         if (CGUIDialogAddonSettings::ShowAndGetInput(addon))
           Refresh();
       return true;
     }
-  case CONTEXT_BUTTON_BROWSE_INTO:
-    {
-      CFileItemPtr item = m_vecItems->Get(itemNumber);
-      if(Update(item->GetPath()))
-        return true;
-      return true;
-    }
   case CONTEXT_BUTTON_USER1:
   case CONTEXT_BUTTON_USER2:
   case CONTEXT_BUTTON_USER3:
@@ -1609,7 +1623,8 @@ bool CGUIMediaWindow::OnContextButton(int itemNumber, CONTEXT_BUTTON button)
   case CONTEXT_BUTTON_USER9:
   case CONTEXT_BUTTON_USER10:
     {
-      CStdString action = StringUtils::Format("contextmenuaction(%i)", button - CONTEXT_BUTTON_USER1);
+      CStdString action;
+      action.Format("contextmenuaction(%i)", button - CONTEXT_BUTTON_USER1);
       CApplicationMessenger::Get().ExecBuiltIn(m_vecItems->Get(itemNumber)->GetProperty(action).asString());
       return true;
     }
@@ -1656,55 +1671,6 @@ bool CGUIMediaWindow::WaitForNetwork() const
   return true;
 }
 
-void CGUIMediaWindow::UpdateFilterPath(const CStdString &strDirectory, const CFileItemList &items, bool updateFilterPath)
-{
-  bool canfilter = CanContainFilter(strDirectory);
-
-  CStdString filter;
-  CURL url(strDirectory);
-  if (canfilter && url.HasOption("filter"))
-    filter = url.GetOption("filter");
-
-  // only set the filter path if it hasn't been marked
-  // as preset or if it's empty
-  if (updateFilterPath || m_strFilterPath.empty())
-  {
-    if (items.HasProperty(PROPERTY_PATH_DB))
-      m_strFilterPath = items.GetProperty(PROPERTY_PATH_DB).asString();
-    else
-      m_strFilterPath = items.GetPath();
-  }
-  
-  // maybe the filter path can contain a filter
-  if (!canfilter && CanContainFilter(m_strFilterPath))
-    canfilter = true;
-
-  // check if the filter path contains a filter
-  CURL filterPathUrl(m_strFilterPath);
-  if (canfilter && filter.empty())
-  {
-    if (filterPathUrl.HasOption("filter"))
-      filter = filterPathUrl.GetOption("filter");
-  }
-
-  // check if there is a filter and re-apply it
-  if (canfilter && !filter.empty())
-  {
-    if (!m_filter.LoadFromJson(filter))
-    {
-      CLog::Log(LOGWARNING, "CGUIMediaWindow::UpdateFilterPath(): unable to load existing filter (%s)", filter.c_str());
-      m_filter.Reset();
-      m_strFilterPath = m_vecItems->GetPath();
-    }
-    else
-    {
-      // add the filter to the filter path
-      filterPathUrl.SetOption("filter", filter);
-      m_strFilterPath = filterPathUrl.Get();
-    }
-  }
-}
-
 void CGUIMediaWindow::OnFilterItems(const CStdString &filter)
 {
   CFileItemPtr currentItem;
@@ -1718,8 +1684,7 @@ void CGUIMediaWindow::OnFilterItems(const CStdString &filter)
   
   m_viewControl.Clear();
   
-  CFileItemList items;
-  items.Copy(*m_vecItems, false); // use the original path - it'll likely be relied on for other things later.
+  CFileItemList items(m_vecItems->GetPath()); // use the original path - it'll likely be relied on for other things later.
   items.Append(*m_unfilteredItems);
   bool filtered = GetFilteredItems(filter, items);
 
@@ -1768,21 +1733,25 @@ void CGUIMediaWindow::OnFilterItems(const CStdString &filter)
     }
   }
 
-  SetProperty("filter", filter);
-  if (filtered && m_canFilterAdvanced)
+  if (filtered)
   {
-    // to be able to select the same item as before we need to adjust
-    // the path of the item i.e. add or remove the "filter=" URL option
-    // but that's only necessary for folder items
-    if (currentItem.get() != NULL && currentItem->m_bIsFolder)
+    if (!m_canFilterAdvanced)
+      SetProperty("filter", filter);
+    else
     {
-      CURL curUrl(currentItemPath), newUrl(m_strFilterPath);
-      if (newUrl.HasOption("filter"))
-        curUrl.SetOption("filter", newUrl.GetOption("filter"));
-      else if (curUrl.HasOption("filter"))
-        curUrl.RemoveOption("filter");
+      // to be able to select the same item as before we need to adjust
+      // the path of the item i.e. add or remove the "filter=" URL option
+      // but that's only necessary for folder items
+      if (currentItem.get() != NULL && currentItem->m_bIsFolder)
+      {
+        CURL curUrl(currentItemPath), newUrl(m_strFilterPath);
+        if (newUrl.HasOption("filter"))
+          curUrl.SetOption("filter", newUrl.GetOption("filter"));
+        else if (curUrl.HasOption("filter"))
+          curUrl.RemoveOption("filter");
 
-      currentItemPath = curUrl.Get();
+        currentItemPath = curUrl.Get();
+      }
     }
   }
 
@@ -1806,16 +1775,14 @@ void CGUIMediaWindow::OnFilterItems(const CStdString &filter)
 
 bool CGUIMediaWindow::GetFilteredItems(const CStdString &filter, CFileItemList &items)
 {
-  bool result = false;
   if (m_canFilterAdvanced)
-    result = GetAdvanceFilteredItems(items);
+    return GetAdvanceFilteredItems(items);
 
   CStdString trimmedFilter(filter);
-  StringUtils::TrimLeft(trimmedFilter);
-  StringUtils::ToLower(trimmedFilter);
+  trimmedFilter.TrimLeft().ToLower();
   
-  if (trimmedFilter.empty())
-    return result;
+  if (trimmedFilter.IsEmpty())
+    return true;
 
   CFileItemList filteredItems(items.GetPath()); // use the original path - it'll likely be relied on for other things later.
   bool numericMatch = StringUtils::IsNaturalNumber(trimmedFilter);
@@ -1871,7 +1838,7 @@ bool CGUIMediaWindow::GetAdvanceFilteredItems(CFileItemList &items)
   for (int j = 0; j < resultItems.Size(); j++)
   {
     CStdString itemPath = RemoveParameterFromPath(resultItems[j]->GetPath(), "filter");
-    StringUtils::ToLower(itemPath);
+    itemPath.ToLower();
 
     lookup[itemPath] = resultItems[j];
   }
@@ -1892,7 +1859,7 @@ bool CGUIMediaWindow::GetAdvanceFilteredItems(CFileItemList &items)
     // by comparing their paths (but ignoring any special
     // options because they differ from filter to filter)
     CStdString path = RemoveParameterFromPath(item->GetPath(), "filter");
-    StringUtils::ToLower(path);
+    path.ToLower();
 
     map<CStdString, CFileItemPtr>::iterator itItem = lookup.find(path);
     if (itItem != lookup.end())
@@ -1923,10 +1890,10 @@ bool CGUIMediaWindow::IsFiltered()
          (m_canFilterAdvanced && !m_filter.IsEmpty());
 }
 
-bool CGUIMediaWindow::Filter(bool advanced /* = true */)
+bool CGUIMediaWindow::Filter()
 {
   // basic filtering
-  if (!m_canFilterAdvanced || !advanced)
+  if (!m_canFilterAdvanced)
   {
     const CGUIControl *btnFilter = GetControl(CONTROL_BTN_FILTER);
     if (btnFilter != NULL && btnFilter->GetControlType() == CGUIControl::GUICONTROL_EDIT)
